@@ -25,6 +25,7 @@
 
 #include "ANT.h"
 #include "ANTMessage.h"
+#include "TrainSidebar.h" // for RT_MODE_{ERGO,SPIN,CALIBRATE}
 #include <QMessageBox>
 #include <QTime>
 #include <QProgressDialog>
@@ -44,6 +45,9 @@
 // network key
 const unsigned char ANT::key[8] = { 0xB9, 0xA5, 0x21, 0xFB, 0xBD, 0x72, 0xC3, 0x45 };
 
+// !!! THE INITIALISATION OF ant_sensor_types BELOW MUST MATCH THE 
+// !!! ENUM FOR CHANNELTYPE IN ANTChannel.h (SORRY, ITS HORRIBLE)
+
 // supported sensor types
 const ant_sensor_type_t ANT::ant_sensor_types[] = {
   { true, ANTChannel::CHANNEL_TYPE_UNUSED, 0, 0, 0, 0, "Unused", '?', "" },
@@ -57,8 +61,12 @@ const ant_sensor_type_t ANT::ant_sensor_types[] = {
                 ANT_SPORT_FREQUENCY, ANT_SPORT_NETWORK_NUMBER, "Cadence", 'c', ":images/IconCadence.png" },
   { true, ANTChannel::CHANNEL_TYPE_SandC, ANT_SPORT_SandC_PERIOD, ANT_SPORT_SandC_TYPE,
                 ANT_SPORT_FREQUENCY, ANT_SPORT_NETWORK_NUMBER, "Speed + Cadence", 'd', ":images/IconCadence.png" },
-  { false, ANTChannel::CHANNEL_TYPE_KICKR, ANT_SPORT_KICKR_PERIOD, ANT_SPORT_POWER_TYPE,
-                ANT_KICKR_FREQUENCY, ANT_SPORT_NETWORK_NUMBER, "Kickr", 'k', ":images/IconCadence.png" },
+  { true, ANTChannel::CHANNEL_TYPE_MOXY, ANT_SPORT_MOXY_PERIOD, ANT_SPORT_MOXY_TYPE,
+                ANT_MOXY_FREQUENCY, ANT_SPORT_NETWORK_NUMBER, "Moxy", 'm', ":images/IconMoxy.png" },
+  { true, ANTChannel::CHANNEL_TYPE_CONTROL, ANT_SPORT_CONTROL_PERIOD, ANT_SPORT_CONTROL_TYPE,
+                ANT_SPORT_FREQUENCY, ANT_SPORT_NETWORK_NUMBER, "Remote Control", 'r', ":images/IconCadence.png" },
+  { true, ANTChannel::CHANNEL_TYPE_TACX_VORTEX, ANT_SPORT_TACX_VORTEX_PERIOD, ANT_SPORT_TACX_VORTEX_TYPE,
+                ANT_TACX_VORTEX_FREQUENCY, DEFAULT_NETWORK_NUMBER, "Tacx Vortex Smart", 'v', ":images/IconPower.png" },
   { false, ANTChannel::CHANNEL_TYPE_GUARD, 0, 0, 0, 0, "", '\0', "" }
 };
 
@@ -86,6 +94,22 @@ ANT::ANT(QObject *parent, DeviceConfiguration *devConf) : QThread(parent), devCo
     powerchannels=0;
     configuring = false;
 
+    // kickr
+    kickrDeviceID = 0;
+    kickrChannel = -1;
+
+    // vortex
+    vortexID = vortexChannel = -1;
+
+    // current and desired modes/load/gradients
+    // set so first time through current != desired
+    currentMode = 0;
+    mode = RT_MODE_ERGO;
+    currentLoad = 0;
+    load = 100; // always set to something
+    currentGradient = 0;
+    gradient = 0.1;
+
     // state machine
     state = ST_WAIT_FOR_SYNC;
     length = bytes = 0;
@@ -159,6 +183,11 @@ void ANT::setWheelRpm(float x) {
     else telemetry.setSpeed(x * appsettings->value(NULL, GC_WHEELSIZE, 2100).toInt() / 1000 * 60 / 1000);
 }
 
+void ANT::setHb(double smo2, double thb)
+{
+    telemetry.setHb(smo2, thb);
+}
+
 /*======================================================================
  * Main thread functions; start, stop etc
  *====================================================================*/
@@ -222,6 +251,106 @@ void ANT::run()
     }
 }
 
+void
+ANT::setLoad(double load)
+{
+    if (this->load == load) return;
+
+    // load has changed
+    this->load = load;
+
+    // if we have a vortex trainer connected, relay the change in target power to the brake
+    if (vortexChannel != -1)
+    {
+        qDebug() << "setting vortex target power to " << load;
+        sendMessage(ANTMessage::tacxVortexSetPower(vortexChannel, vortexID, (int)load));
+    }
+}
+
+void ANT::refreshVortexLoad()
+{
+    if (vortexChannel == -1)
+        return;
+
+    sendMessage(ANTMessage::tacxVortexSetPower(vortexChannel, vortexID, (int)load));
+}
+
+void
+ANT::setGradient(double gradient)
+{
+    if (this->gradient == gradient) return;
+
+    // gradient changed
+    this->gradient = gradient;
+}
+void
+ANT::setMode(int mode)
+{
+    if (this->mode == mode) return;
+
+    // mode changed
+    this->mode = mode;
+}
+
+void
+ANT::kickrCommand()
+{
+    // we don't have a kickr !
+    if (kickrChannel < 0) return;
+
+    // mode changed ?
+    if (currentMode != mode) {
+
+        switch(mode) {
+        case RT_MODE_ERGO : // do nothing, just start sending ergo commands below
+            {
+            }
+           break;
+
+        case RT_MODE_SPIN : // need to setup for "sim" mode, so sending lots of 
+                            // config to overcome the default values
+            {
+            }
+            break;
+
+        case RT_MODE_CALIBRATE : // ??? maybe ???
+            //qDebug()<<"A: setup calib mode";
+            break;
+        }
+
+        currentMode = mode;
+        currentLoad = load;
+        currentGradient = gradient;
+    }
+
+    // load has changed ?
+    if (mode == RT_MODE_ERGO && load != currentLoad) { 
+        currentLoad = load;
+    }
+
+    // slope has changed in slope mode
+    if (mode == RT_MODE_SPIN && gradient != currentGradient) {
+        currentGradient = gradient;
+    }
+
+    // create message to send
+    ANTMessage toSend;
+
+    switch (mode) {
+
+    default:
+    case RT_MODE_ERGO: 
+        toSend = ANTMessage::kickrErgMode(kickrChannel, kickrDeviceID, load, false);
+        break;
+
+    case RT_MODE_SPIN: // gradient is between -1.0 and +1.0 so needs conversion from gradient
+        toSend = ANTMessage::kickrGrade(kickrChannel, kickrDeviceID, gradient/100.00f);
+        break;
+    }
+
+    sendMessage(toSend);
+}
+
 int
 ANT::start()
 {
@@ -266,7 +395,10 @@ ANT::setup()
             addDevice(0, ANTChannel::CHANNEL_TYPE_CADENCE, 2);
             addDevice(0, ANTChannel::CHANNEL_TYPE_HR, 3);
 
-            if (channels > 4) addDevice(0, ANTChannel::CHANNEL_TYPE_SandC, 4);
+            if (channels > 4) {
+                addDevice(0, ANTChannel::CHANNEL_TYPE_SandC, 4);
+                addDevice(0, ANTChannel::CHANNEL_TYPE_MOXY, 5);
+            }
         }
     }
 
@@ -323,7 +455,6 @@ ANT::stop()
     // Close the connections to ANT devices before we stop. Sending the
     // "close channel" ANT message seems to resolve an intermittent
     // issue of unresponsive USB2 stick on subsequent opens.
-
     if (antIDs.count()) {
         foreach(QString antid, antIDs) {
             if (antid.length()) {
@@ -336,6 +467,11 @@ ANT::stop()
         }
     }
 
+    // and close any channels that are open
+    for (int i=0; i<channels; i++)
+        if (antChannel[i]->status != ANTChannel::Closed)
+            antChannel[i]->close();
+
     // what state are we in anyway?
     pvars.lock();
     Status = 0; // Terminate it!
@@ -359,21 +495,17 @@ ANT::quit(int code)
 void
 ANT::getRealtimeData(RealtimeData &rtData)
 {
-    int mode = rtData.mode;
-    long load = rtData.getLoad();
-    double slope = rtData.getSlope();
-
     rtData = telemetry;
     rtData.mode = mode;
     rtData.setLoad(load);
-    rtData.setSlope(slope);
+    rtData.setSlope(gradient);
 }
 
 /*======================================================================
  * Channel management
  *====================================================================*/
 
-// returns 1 for success, 0 for fail.
+// returns -1 for fail otherwise channel number used
 int
 ANT::addDevice(int device_number, int device_type, int channel_number)
 {
@@ -381,7 +513,7 @@ ANT::addDevice(int device_number, int device_type, int channel_number)
     if (channel_number>-1) {
         //antChannel[channel_number]->close();
         antChannel[channel_number]->open(device_number, device_type);
-        return 1;
+        return channel_number;
     }
 
     // if we already have the device, then return.
@@ -391,10 +523,10 @@ ANT::addDevice(int device_number, int device_type, int channel_number)
     // and 0p on channel 1
     if (device_number != 0) {
         for (int i=0; i<channels; i++) {
-            if (((antChannel[i]->channel_type & 0xf ) == device_type) &&
+            if ((antChannel[i]->channel_type == device_type) &&
                 (antChannel[i]->device_number == device_number)) {
                 // send the channel found...
-                return 1;
+                return i;
             }
         }
     }
@@ -416,12 +548,12 @@ ANT::addDevice(int device_number, int device_type, int channel_number)
                 // increment the number of power channels
                 powerchannels++;
             }
-            return 1;
+            return i;
         }
     }
 
     // there are no unused channels.  fail.
-    return 0;
+    return -1;
 }
 
 // returns 1 for successfully removed, 0 for none found.
@@ -434,10 +566,6 @@ ANT::removeDevice(int device_number, int channel_type)
         ANTChannel *ac = antChannel[i];
 
         if ((ac->channel_type == channel_type) && (ac->device_number == device_number)) {
-
-            if ((ac->control_channel!=ac) && ac->control_channel)
-                removeDevice(device_number, ac->control_channel->channel_type);
-
             ac->close();
             ac->channel_type=ANTChannel::CHANNEL_TYPE_UNUSED;
             ac->device_number=0;
@@ -491,60 +619,6 @@ ANT::startWaitingSearch()
     return 0;
 }
 
-void
-ANT::associateControlChannels() {
-
-    // first, unassociate all control channels
-    for (int i=0; i<channels; i++) antChannel[i]->control_channel=NULL;
-
-    // then, associate cinqos:
-    //    new cinqos get their own selves for control
-    //    old cinqos, look for an open control channel
-    //       if found and open, associate
-    //       elif found and not open yet, nop
-    //       elif not found, open one
-    for (int i=0; i<channels; i++) {
-        ANTChannel *ac=antChannel[i];
-
-        switch (ac->channel_type) {
-            case ANTChannel::CHANNEL_TYPE_POWER:
-                if (ac->is_cinqo) {
-                    if (ac->is_old_cinqo) {
-                        ANTChannel *my_ant_channel;
-
-                        my_ant_channel=findDevice(ac->device_number, ANTChannel::CHANNEL_TYPE_QUARQ);
-                        if (!my_ant_channel) my_ant_channel=findDevice(ac->device_number, ANTChannel::CHANNEL_TYPE_FAST_QUARQ);
-                        if (!my_ant_channel) my_ant_channel=findDevice(ac->device_number, ANTChannel::CHANNEL_TYPE_FAST_QUARQ_NEW);
-
-                        if (my_ant_channel) {
-                            if (my_ant_channel->isSearching()) {
-                                // ignore if searching
-                            } else {
-                                ac->control_channel=my_ant_channel;
-                                ac->sendCinqoSuccess();
-                            }
-                        } else { // no ant channel, let's start one
-                            addDevice(ac->device_number, ANTChannel::CHANNEL_TYPE_QUARQ, -1);
-                        }
-                    } else { // new cinqo
-                        ac->control_channel=ac;
-                        ac->sendCinqoSuccess();
-                    }
-                } // is_cinqo
-                break;
-
-              case ANTChannel::CHANNEL_TYPE_FAST_QUARQ:
-              case ANTChannel::CHANNEL_TYPE_FAST_QUARQ_NEW:
-              case ANTChannel::CHANNEL_TYPE_QUARQ:
-                  ac->is_cinqo=1;
-                  ac->control_channel=ac;
-                  break;
-              default:
-                      ;
-        } // channel_type case
-    } // for-loop
-}
-
 // For serial device discovery
 bool
 ANT::discover(QString name)
@@ -579,7 +653,24 @@ Q_UNUSED(name);
 void
 ANT::channelInfo(int channel, int device_number, int device_id)
 {
+    qDebug()<<"** CONNECTION ESTABLISHED channel:"<<channel<<"device:"<<device_number<<"**";
     emit foundDevice(channel, device_number, device_id);
+
+    // KICKR DETECTED - ACT ACCORDINGLY !
+    // if we just got a power device and its recognised as being a kickr
+    // trainer then we will need to open up the comms channel, unless it
+    // has already been done
+    if (!configuring && antChannel[channel]->is_kickr && kickrDeviceID != device_number) {
+
+        kickrDeviceID = device_number;
+        kickrChannel = channel;
+
+        // lets go ?
+
+        // need to find a way to communicate back on error
+        qDebug()<<"kickr found."<<kickrDeviceID<<"on channel"<<kickrChannel;
+    }
+
     //qDebug()<<"found device number"<<device_number<<"type"<<device_id<<"on channel"<<channel
     //<< "is a "<<deviceTypeDescription(device_id) << "with code"<<deviceTypeCode(device_id);
 }
@@ -588,7 +679,7 @@ void
 ANT::dropInfo(int channel, int drops, int received)    // we dropped a message
 {
     double reliability = 100.00f - (100.00f * double(drops) / double(received));
-    //qDebug()<<"Channel"<<channel<<"reliability is"<< (int)(reliability)<<"%";
+    qDebug()<<"Channel"<<channel<<"reliability is"<< (int)(reliability)<<"%";
     emit signalStrength(channel, reliability);
     return;
 }
@@ -599,7 +690,7 @@ ANT::lostInfo(int number)    // we lost the connection
     if (number < 0 || number >= channels) return; // ignore out of bound
 
     emit lostDevice(number);
-    //qDebug()<<"lost info for channel"<<number;
+    qDebug()<<"lost info for channel"<<number;
 }
 
 void
@@ -607,7 +698,7 @@ ANT::staleInfo(int number)   // info is now stale - set to zero
 {
     if (number < 0 || number >= channels) return; // ignore out of bound
 
-    //qDebug()<<"stale info for channel"<<number;
+    qDebug()<<"stale info for channel"<<number;
 }
 
 void
@@ -616,7 +707,7 @@ ANT::slotSearchTimeout(int number) // search timed out
     if (number < 0 || number >= channels) return; // ignore out of bound
 
     emit searchTimeout(number);
-    //qDebug()<<"search timeout on channel"<<number;
+    qDebug()<<"search timeout on channel"<<number;
 }
 
 void
@@ -625,7 +716,7 @@ ANT::slotSearchComplete(int number) // search completed successfully
     if (number < 0 || number >= channels) return; // ignore out of bound
 
     emit searchComplete(number);
-    //qDebug()<<"search completed on channel"<<number;
+    qDebug()<<"search completed on channel"<<number;
 }
 
 /*----------------------------------------------------------------------
@@ -713,7 +804,7 @@ ANT::processMessage(void) {
 
     ANTMessage m(this, rxMessage); // for debug!
 
-//fprintf(stderr, "<< receive: ");
+//fprintf(stderr, "<< receive %i: ", rxMessage[ANT_OFFSET_CHANNEL_NUMBER]);
 //for(int i=0; i<m.length+3; i++) fprintf(stderr, "%02x ", m.data[i]);
 //fprintf(stderr, "\n");
 
@@ -793,9 +884,9 @@ int ANT::closePort()
 
 bool ANT::find()
 {
-#if defined GC_HAVE_LIBUSB
+#ifdef GC_HAVE_LIBUSB
     if (usb2->find() == true) return true;
-#ifdef WIN32
+#if defined(WIN32) && defined(GC_HAVE_USBXPRESS)
     if (USBXpress::find() == true) return true;
 #endif
 #endif
@@ -813,11 +904,15 @@ int ANT::openPort()
         usbMode = USB2;
         channels = 8;
         return rc;
-    } else if ((rc= USBXpress::open(&devicePort)) != -1) {
+    }
+#if defined(WIN32) && defined(GC_HAVE_USBXPRESS)
+    else if ((rc= USBXpress::open(&devicePort)) != -1) {
         usbMode = USB1;
         channels = 4;
         return rc;
-    } else {
+    }
+#endif
+    else {
         usbMode = USBNone;
         channels = 0;
         return -1;
@@ -893,9 +988,11 @@ int ANT::rawWrite(uint8_t *bytes, int size) // unix!!
 #ifdef WIN32
 #ifdef GC_HAVE_LIBUSB
     switch (usbMode) {
+#ifdef GC_HAVE_USBXPRESS
     case USB1:
         rc = USBXpress::write(&devicePort, bytes, size);
         break;
+#endif
     case USB2:
         rc = usb2->write((char *)bytes, size);
         break;
@@ -915,21 +1012,23 @@ int ANT::rawWrite(uint8_t *bytes, int size) // unix!!
     if (usbMode == USB2) {
         return usb2->write((char *)bytes, size);
     }
+
+    if (usbMode == USB1) {
+        int ibytes;
+
+        ioctl(devicePort, FIONREAD, &ibytes);
+
+        // timeouts are less critical for writing, since vols are low
+        rc= write(devicePort, bytes, size);
+
+        if (rc != -1) tcdrain(devicePort); // wait till its gone.
+
+        ioctl(devicePort, FIONREAD, &ibytes);
+        return rc;
+    }
 #endif
-
-    int ibytes;
-
-    ioctl(devicePort, FIONREAD, &ibytes);
-
-    // timeouts are less critical for writing, since vols are low
-    rc= write(devicePort, bytes, size);
-
-    if (rc != -1) tcdrain(devicePort); // wait till its gone.
-
-    ioctl(devicePort, FIONREAD, &ibytes);
-    return rc;
 #endif
-
+    return -1;
 
 }
 
@@ -938,9 +1037,11 @@ int ANT::rawRead(uint8_t bytes[], int size)
 #ifdef WIN32
 #ifdef GC_HAVE_LIBUSB
     switch (usbMode) {
+#ifdef GC_HAVE_USBXPRESS
     case USB1:
         return USBXpress::read(&devicePort, bytes, size);
         break;
+#endif
     case USB2:
         return usb2->read((char *)bytes, size);
         break;
@@ -1018,3 +1119,9 @@ const char * ANT::deviceTypeDescription(int type)
     } while (++st, st->type != ANTChannel::CHANNEL_TYPE_GUARD);
     return "Unknown device type";
 }
+
+void ANT::setVortexData(int channel, int id)
+{
+    vortexChannel = channel;
+    vortexID = id;
+}
