@@ -35,16 +35,16 @@ ANTChannel::init()
 {
     channel_type=CHANNEL_TYPE_UNUSED;
     channel_type_flags=0;
+    is_kickr=false;
+    is_moxy=false;
     is_cinqo=0;
     is_old_cinqo=0;
     is_alt=0;
-    control_channel=NULL;
     manufacturer_id=0;
     product_id=0;
     product_version=0;
     device_number=0;
     device_id=0;
-    channel_assigned=0;
     state=ANT_UNASSIGN_CHANNEL;
     blanked=1;
     messages_received=0;
@@ -53,6 +53,7 @@ ANTChannel::init()
     srm_offset=400; // default relatively arbitrary, but matches common 'indoors' values
     burstInit();
     value2=value=0;
+    status = Closed;
 }
 
 //
@@ -95,7 +96,12 @@ void ANTChannel::open(int device, int chan_type)
     device_number=device;
     setId();
 
-    attemptTransition(ANT_UNASSIGN_CHANNEL);
+    // lets open the channel
+    qDebug()<<"** OPENING CHANNEL"<<number<<"**";
+    status = Opening;
+
+    // start the transition process 
+    attemptTransition(TRANSITION_START);
 }
 
 // close an ant channel assignment
@@ -103,6 +109,11 @@ void ANTChannel::close()
 {
     emit lostInfo(number);
     lastMessage = ANTMessage();
+
+    // lets shutdown
+    qDebug()<<"** CLOSING CHANNEL"<<number<<"**";
+    status = Closing;
+
     parent->sendMessage(ANTMessage::close(number));
     init();
 }
@@ -118,17 +129,21 @@ void ANTChannel::receiveMessage(unsigned char *ant_message)
         break;
     case ANT_BROADCAST_DATA:
         broadcastEvent(ant_message);
+        //qDebug()<<"broadcast event:"<<number;
         break;
     case ANT_ACK_DATA:
         ackEvent(ant_message);
+        //qDebug()<<"ack data:"<<number;
         break;
     case ANT_CHANNEL_ID:
         channelId(ant_message);
+        //qDebug()<<"channel id:"<<number;
         break;
     case ANT_BURST_DATA:
         burstData(ant_message);
         break;
     default:
+        //qDebug()<<"dunno?"<<number;
         break; //errors silently ignored for now, would indicate hardware fault.
     }
 
@@ -148,20 +163,29 @@ void ANTChannel::receiveMessage(unsigned char *ant_message)
 // break existing code.
 void ANTChannel::channelEvent(unsigned char *ant_message) {
 
+    // we should reimplement this via ANTMessage at some point
     unsigned char *message=ant_message+2;
 
-//qDebug()<<"channel event:"<< ANTMessage::channelEventMessage(*(message+1));
-
+    //qDebug()<<number<<"channel event:"<< ANTMessage::channelEventMessage(*(message+1));
     if (MESSAGE_IS_RESPONSE_NO_ERROR(message)) {
 
+        //qDebug()<<number<<"channel event response no error";
         attemptTransition(RESPONSE_NO_ERROR_MESSAGE_ID(message));
 
     } else if (MESSAGE_IS_EVENT_CHANNEL_CLOSED(message)) {
 
-        parent->sendMessage(ANTMessage::unassignChannel(number));
+        //qDebug()<<number<<"channel event channel closed";
+        if (status != Closing) {
+            //qDebug()<<"we got closed!! re-open !!";
+            status = Opening;
+            attemptTransition(TRANSITION_START);
+        } else {
+            parent->sendMessage(ANTMessage::unassignChannel(number));
+        }
 
     } else if (MESSAGE_IS_EVENT_RX_SEARCH_TIMEOUT(message)) {
 
+        //qDebug()<<number<<"channel event rx timeout";
         // timeouts are normal for search channel
         if (channel_type_flags & CHANNEL_TYPE_QUICK_SEARCH) {
 
@@ -187,6 +211,7 @@ void ANTChannel::channelEvent(unsigned char *ant_message) {
 
     } else if (MESSAGE_IS_EVENT_RX_FAIL(message)) {
 
+        //qDebug()<<number<<"channel event rx fail";
         messages_dropped++;
         double t=get_timestamp();
 
@@ -198,11 +223,18 @@ void ANTChannel::channelEvent(unsigned char *ant_message) {
 
     } else if (MESSAGE_IS_EVENT_RX_ACKNOWLEDGED(message)) {
 
-        exit(-10);
+        //this cannot possibly be correct >> exit(-10);
 
     } else if (MESSAGE_IS_EVENT_TRANSFER_TX_COMPLETED(message)) {
+
         // do nothing
-    } 
+
+    }  else {
+
+        // usually a response event, so lets get a debug going
+        //qDebug()<<number<<"channel event other ....."<<QString("0x%1 0x%2 %3 %4")
+        //.arg(message[0], 1, 16).arg(message[1], 1, 16).arg(message[2]).arg(ANTMessage::channelEventMessage(message[2]));
+    }
 }
 
 // if this is a quarq cinqo then record that fact
@@ -225,6 +257,14 @@ void ANTChannel::checkCinqo()
     }
 }
 
+// are we a moxy ?
+void ANTChannel::checkMoxy()
+{
+    // we are a moxy !
+    if (!is_moxy && manufacturer_id==76)
+        is_moxy=1;
+}
+
 // notify we have a cinqo, does nothing
 void ANTChannel::sendCinqoSuccess() {}
 
@@ -236,10 +276,12 @@ void ANTChannel::sendCinqoSuccess() {}
 //
 void ANTChannel::broadcastEvent(unsigned char *ant_message)
 {
+    //qDebug()<<number<<"broadcast event !";
     ANTMessage antMessage(parent, ant_message);
     bool savemessage = true; // flag to stop lastmessage being
                              // overwritten for standard power
                              // messages
+    bool telemetry=false;
 
     unsigned char *message=ant_message+2;
     double timestamp=get_timestamp();
@@ -256,7 +298,7 @@ void ANTChannel::broadcastEvent(unsigned char *ant_message)
 
         // The recent bug with not being able to "pair" intermittently, was caused
         // by the write below failing (and any write really, but the one below being
-        // pretty critical) -- because the USB stick needed a USB reset which we know
+        // pretty critical) -- because the USB stick needed a USB reset which we now
         // do every time we open the USB device
         parent->sendMessage(ANTMessage::requestMessage(number, ANT_CHANNEL_ID));
         blanking_timestamp=get_timestamp();
@@ -271,6 +313,10 @@ void ANTChannel::broadcastEvent(unsigned char *ant_message)
         product_version&=0x00ff;
         product_version|=(PRODUCT_SW_REV(message))<<8;
         checkCinqo();
+        checkMoxy();
+
+        // TODO in some case ? eg a cadence sensor we have telemetry too
+        //telemetry = true;
 
     } else if (MESSAGE_IS_MANUFACTURER(message)) {
 
@@ -281,7 +327,12 @@ void ANTChannel::broadcastEvent(unsigned char *ant_message)
         product_id=MANUFACTURER_MODEL_NUMBER_ID(message);
         checkCinqo();
 
-    } else {
+    }
+    else {
+        telemetry = true;
+    }
+
+    if (telemetry) {
         //
         // We got some telemetry on this channel
         //
@@ -291,9 +342,6 @@ void ANTChannel::broadcastEvent(unsigned char *ant_message)
 
            // Power
            case CHANNEL_TYPE_POWER:
-           case CHANNEL_TYPE_QUARQ:
-           case CHANNEL_TYPE_FAST_QUARQ:
-           case CHANNEL_TYPE_FAST_QUARQ_NEW:
 
                 // what kind of power device is this?
                 switch(antMessage.data_page) {
@@ -320,7 +368,7 @@ void ANTChannel::broadcastEvent(unsigned char *ant_message)
                                 // will be zero
                                 srm_offset = antMessage.srmOffset;
                                 is_alt ? parent->setAltWatts(0) : parent->setWatts(0);
-                                parent->setCadence(0);
+                                parent->setSecondaryCadence(0);
                                 value2=value=0;
                                 break;
 
@@ -361,11 +409,11 @@ void ANTChannel::broadcastEvent(unsigned char *ant_message)
                         float cadence = 2000.0 * 60 * (antMessage.eventCount - lastMessage.eventCount) / period;
                         float power = 3.14159 * nm_torque * cadence / 30;
 
-                        // ignore the occassional spikes (reed switch)
+                        // ignore the occasional spikes (reed switch)
                         if (power >= 0 && power < 2501 && cadence >=0 && cadence < 256) {
                             value2 = value = power;
                             is_alt ? parent->setAltWatts(power) : parent->setWatts(power);
-                            parent->setCadence(cadence);
+                            parent->setSecondaryCadence(cadence);
                         }
 
                     } else {
@@ -374,7 +422,7 @@ void ANTChannel::broadcastEvent(unsigned char *ant_message)
                         if (nullCount >= 4) { // 4 messages on an SRM
                             value2 = value = 0;
                             is_alt ? parent->setAltWatts(0) : parent->setWatts(0);
-                            parent->setCadence(0);
+                            parent->setSecondaryCadence(0);
                         }
                     }
                 }
@@ -428,13 +476,17 @@ void ANTChannel::broadcastEvent(unsigned char *ant_message)
                         stdNullCount = 0;
                         is_alt ? parent->setAltWatts(antMessage.instantPower) : parent->setWatts(antMessage.instantPower);
                         value2 = value = antMessage.instantPower;
-                        parent->setCadence(antMessage.instantCadence); // cadence
+                        parent->setSecondaryCadence(antMessage.instantCadence); // cadence
+                        antMessage.pedalPowerContribution ? parent->setLRBalance(antMessage.pedalPower) : parent->setLRBalance(0);
                     } else {
                        stdNullCount++;
                        if (stdNullCount >= 6) { //6 for standard power according to specs
-                           parent->setCadence(0);
+                           parent->setSecondaryCadence(0);
                            is_alt ? parent->setAltWatts(0) : parent->setWatts(0);
+                           parent->setLRBalance(0);
                            value2 = value = 0;
+                           parent->setTE(0,0);
+                           parent->setPS(0,0);
                        }
                     }
                     lastStdPwrMessage = antMessage;
@@ -442,6 +494,29 @@ void ANTChannel::broadcastEvent(unsigned char *ant_message)
                 }
                 break;
 
+                case ANT_TE_AND_PS_POWER: // 0x13 - optional extension to standard power / event Count is defined to be in sync with 0x10 - so not separate calculation
+                                          // and just take whatever is delivered - data may not be sent for every power reading - but minimum every 5th pwr message
+                {
+                    uint8_t events = antMessage.eventCount - lastStdPwrMessage.eventCount;
+                    if (events) {
+                        // provide valid values only
+                        if (antMessage.leftTorqueEffectiveness != 0xFF && antMessage.rightTorqueEffectiveness != 0xFF) {
+                            parent->setTE((antMessage.leftTorqueEffectiveness / 2),(antMessage.rightTorqueEffectiveness / 2));  // values are given in 1/2 %
+                        } else {
+                            parent->setTE(0,0);
+                        }
+                        // provide valid values only and handle single and combined PS option (which is allowed in 0x13)
+                        if (antMessage.leftOrCombinedPedalSmoothness != 0xFF && antMessage.rightTorqueEffectiveness != 0xFF) {
+                            if (antMessage.rightPedalSmoothness == 0xFE) {
+                                parent->setPS((antMessage.leftOrCombinedPedalSmoothness / 2), 0);
+                            } else {
+                                parent->setPS((antMessage.leftOrCombinedPedalSmoothness / 2), (antMessage.rightPedalSmoothness / 2)); // values are given in 1/2 %
+                            }
+                        }
+                    }
+                }
+
+                break;
 
                 //
                 // Quarq - Crank torque
@@ -459,14 +534,14 @@ void ANTChannel::broadcastEvent(unsigned char *ant_message)
                         float cadence = 2048.0 * 60.0 * events / period;
                         float power = 3.14159 * nm_torque * cadence / 30;
 
-                        parent->setCadence(cadence);
+                        parent->setSecondaryCadence(cadence);
                         is_alt ? parent->setAltWatts(power) : parent->setWatts(power);
                         value2 = value = power;
 
                     } else {
                         nullCount++;
                         if (nullCount >= 4) { // 4 on a quarq according to specs
-                            parent->setCadence(0);
+                            parent->setSecondaryCadence(0);
                             is_alt ? parent->setAltWatts(0) : parent->setWatts(0);
                             value2 = value = 0;
                         }
@@ -508,9 +583,15 @@ void ANTChannel::broadcastEvent(unsigned char *ant_message)
                uint16_t time = antMessage.crankMeasurementTime - lastMessage.crankMeasurementTime;
                uint16_t revs = antMessage.crankRevolutions - lastMessage.crankRevolutions;
                if (time) {
+                   nullCount = 0;
                    float cadence = 1024*60*revs / time;
                    parent->setCadence(cadence);
                    value2 = value = cadence;
+               } else {
+                   nullCount++;
+                   if (nullCount >= 12) { parent->setCadence(0);
+                                          value = 0;
+                   }
                }
            }
            break;
@@ -524,13 +605,17 @@ void ANTChannel::broadcastEvent(unsigned char *ant_message)
                if (time) {
                    nullCount = 0;
                    float cadence = 1024*60*revs / time;
-                   parent->setCadence(cadence);
+
+                   if (is_moxy) /* do nothing for now */ ; //XXX fixme when moxy arrives XXX
+                   else parent->setCadence(cadence);
                    value = cadence;
+
                } else {
+
                    nullCount++;
-                   if (nullCount >= 12) { parent->setCadence(0);
-                                          value = 0;
-                    }
+                   if (!is_moxy && nullCount >= 12) { parent->setCadence(0);
+                        value = 0;
+                   }
                }
 
                // now speed ...
@@ -540,12 +625,14 @@ void ANTChannel::broadcastEvent(unsigned char *ant_message)
                    dualNullCount = 0;
 
                    float rpm = 1024*60*revs / time;
-                   parent->setWheelRpm(rpm);
+                   if (is_moxy) /* do nothing for now */ ; //XXX fixme when moxy arrives XXX
+                   else parent->setWheelRpm(rpm);
                    value2 = rpm;
+
                } else {
 
                     dualNullCount++;
-                    if (dualNullCount >= 12) {
+                    if (!is_moxy && dualNullCount >= 12) {
                         parent->setWheelRpm(0);
                         value2 = 0;
                     }
@@ -571,6 +658,35 @@ void ANTChannel::broadcastEvent(unsigned char *ant_message)
            }
            break;
 
+            //moxy
+            case CHANNEL_TYPE_MOXY:
+            {
+                value = antMessage.tHb;
+                value2 = antMessage.newsmo2;
+                parent->setHb(value2, value);
+            }
+            break;
+
+            // Tacx Vortex trainer
+            case CHANNEL_TYPE_TACX_VORTEX:
+            {
+               static int loadRefreshCounter = 1;
+
+               if (((loadRefreshCounter++) % 10) == 0)
+                   parent->refreshVortexLoad();
+
+                if (antMessage.vortexPage == TACX_VORTEX_DATA_CALIBRATION)
+                    parent->setVortexData(number, antMessage.vortexId);
+                else if (antMessage.vortexPage == TACX_VORTEX_DATA_SPEED)
+                {
+                    parent->setWatts(antMessage.vortexPower);
+                    // cadence is only supplied in some range, only set if valid value
+                    if (antMessage.vortexCadence)
+                        parent->setSecondaryCadence(antMessage.vortexCadence);
+                }
+            }
+            break;
+
            default:
              break; // unknown?
            }
@@ -607,11 +723,17 @@ void ANTChannel::channelId(unsigned char *ant_message) {
     // it is a kick, A0 gives the game away :)
     is_kickr = (device_id == ANT_SPORT_POWER_TYPE) && ((CHANNEL_ID_TRANSMISSION_TYPE(message)&0xF0) == 0xA0);
 
-    emit channelInfo(number, device_number, device_id);
+    if (is_kickr) {
+        qDebug()<<number<<"KICKR DETECTED VIA CHANNEL ID EVENT";
+    }
+
+    // tell controller we got a new channel id
     setId();
+    emit channelInfo(number, device_number, device_id);
 
     // if we were searching,
-    if (channel_type_flags & CHANNEL_TYPE_QUICK_SEARCH) {
+    if (channel_type_flags&CHANNEL_TYPE_QUICK_SEARCH) {
+        //qDebug()<<number<<"change timeout setting";
         parent->sendMessage(ANTMessage::setSearchTimeout(number, (int)(timeout_lost/2.5)));
     }
     channel_type_flags &= ~CHANNEL_TYPE_QUICK_SEARCH;
@@ -661,53 +783,63 @@ void ANTChannel::burstData(unsigned char *ant_message) {
     }
 }
 
-// choose this one..
-void ANTChannel::setChannelID(int device_number, int device_id, int /*txtype*/)
-{
-    parent->sendMessage(ANTMessage::setChannelID(number, device_number, device_id, 0)); // lets go back to allowing anything
-    parent->sendMessage(ANTMessage::open(number)); // lets go back to allowing anything
-}
-
+// TRANSITIONING FROM NOT OPEN TO OPEN AND BACK
+// this is basically a transition from an unassigned channel
+// to one with a device open on it and works from top to bottom
+// it is first called by open() to unassign channel and then
+// after each message response from the device we get called here
+// to perform the next action in the sequence
 void ANTChannel::attemptTransition(int message_id)
 {
+    // ignore all the nonsense if we are not trying to transition !
+    if (status != Closing && status != Opening) {
+        //qDebug()<<number<<"ignore transition"<<message_id;
+        return;
+    }
 
     const ant_sensor_type_t *st;
     int previous_state=state;
     st=&(parent->ant_sensor_types[channel_type]);
 
+    //qDebug()<<number<<"type="<<channel_type<<"device type="<<device_id<<"freq="<<st->frequency;
+
     // update state
     state=message_id;
 
     // do transitions from top status to bottom status !
     switch (state) {
 
-    case ANT_CLOSE_CHANNEL:
-        // next step is unassign and start over
-        // but we must wait until event_channel_closed
-        // which is its own channel event
-        state=MESSAGE_RECEIVED;
-        break;
+    case TRANSITION_START:
+        //qDebug()<<number<<"TRANSITION start";
 
-    case ANT_UNASSIGN_CHANNEL:
-        channel_assigned=0;
-
-        // lets make sure this channel is assigned to our network
-        // regardless of its current state.
+        // unassign regardless of status
         parent->sendMessage(ANTMessage::unassignChannel(number)); // unassign whatever we had before
 
-        // reassign to whatever we need!
-        parent->sendMessage(ANTMessage::assignChannel(number, 0, st->network)); // recieve channel on network 1
+        // drops through into assign channel because if the channel currently has no
+        // assignment the unassign channel message will generate an error response not
+        // an unassign channel response. But we don't really know what state the device
+        // might be in, so we unassign then assign at the beginning of a transition
+
+    case ANT_UNASSIGN_CHANNEL:
+        //qDebug()<<number<<"TRANSITION from unassigned";
+
+        // assign and set channel id all in one
+        parent->sendMessage(ANTMessage::assignChannel(number, CHANNEL_TYPE_RX, st->network)); // receive channel on network 1
+
         device_id=st->device_id;
-        parent->sendMessage(ANTMessage::setChannelID(number, 0, device_id, 0)); // lets go back to allowing anything
         setId();
         break;
 
     case ANT_ASSIGN_CHANNEL:
-        channel_assigned=1;
-        parent->sendMessage(ANTMessage::setChannelID(number, device_number, device_id, 0));
+
+        //qDebug()<<number<<"TRANSITION from assign channel";
+
+        parent->sendMessage(ANTMessage::setChannelID(number, device_number, device_id, 0)); // we need to be specific!
         break;
 
     case ANT_CHANNEL_ID:
+        //qDebug()<<number<<"TRANSITION from channel id";
+        //qDebug()<<number<<"**** adjust timeout";
         if (channel_type & CHANNEL_TYPE_QUICK_SEARCH) {
             parent->sendMessage(ANTMessage::setSearchTimeout(number, (int)(timeout_scan/2.5)));
         } else {
@@ -716,8 +848,10 @@ void ANTChannel::attemptTransition(int message_id)
         break;
 
     case ANT_SEARCH_TIMEOUT:
+        //qDebug()<<number<<"TRANSITION from search timeout";
         if (previous_state==ANT_CHANNEL_ID) {
-            // continue down the intialization chain
+            // continue down the initialization chain
+            //qDebug()<<"set channel period in timeout"<<st->period;
             parent->sendMessage(ANTMessage::setChannelPeriod(number, st->period));
         } else {
             // we are setting the ant_search timeout after connected
@@ -727,16 +861,32 @@ void ANTChannel::attemptTransition(int message_id)
         break;
 
     case ANT_CHANNEL_PERIOD:
+        //qDebug()<<number<<"TRANSITION from channel period";
+        //qDebug()<<"set channel freq in timeout"<<st->frequency;
         parent->sendMessage(ANTMessage::setChannelFreq(number, st->frequency));
         break;
 
     case ANT_CHANNEL_FREQUENCY:
+        //qDebug()<<number<<"TRANSITION from channel freq";
         parent->sendMessage(ANTMessage::open(number));
         mi.initialise();
         break;
 
     case ANT_OPEN_CHANNEL:
-        parent->sendMessage(ANTMessage::open(number));
+        //qDebug()<<number<<"TRANSITION from open (do nothing)";
+        status = Open;
+        //qDebug()<<"** CHANNEL"<<number<<"NOW OPEN **";
+        //parent->sendMessage(ANTMessage::open(number));
+        break;
+
+    case ANT_CLOSE_CHANNEL:
+        // next step is unassign and start over
+        // but we must wait until event_channel_closed
+        // which is its own channel event
+        state=MESSAGE_RECEIVED;
+        //qDebug()<<number<<"TRANSITION from closed";
+        status = Closed;
+        //qDebug()<<"** CHANNEL"<<number<<"NOW CLOSED **";
         break;
 
     default:
@@ -744,14 +894,8 @@ void ANTChannel::attemptTransition(int message_id)
     }
 }
 
-// set channel timeout
-void ANTChannel::setTimeout(int seconds)
-{
-    parent->sendMessage(ANTMessage::setSearchTimeout(number, seconds/2.5));
-}
-
 // Calibrate... needs fixing in version 3.1
-// request the device on this channel calibrates itselt
+// request the device on this channel calibrates itself
 void ANTChannel::requestCalibrate() {
   parent->sendMessage(ANTMessage::requestCalibrate(number));
 }
