@@ -20,6 +20,7 @@
 #define _GC_AllPlot_h 1
 #include "GoldenCheetah.h"
 #include "Colors.h"
+#include "AllPlotSlopeCurve.h"
 
 #include <qwt_plot.h>
 #include <qwt_axis_id.h>
@@ -28,10 +29,12 @@
 #include <qwt_plot_curve.h>
 #include <qwt_plot_dict.h>
 #include <qwt_plot_marker.h>
+#include <qwt_plot_intervalcurve.h>
 #include <qwt_point_3d.h>
 #include <qwt_scale_widget.h>
 #include <qwt_compat.h>
 #include <QtGui>
+#include <QFont>
 
 #include <QTableWidget>
 #include <QStackedWidget>
@@ -67,7 +70,7 @@ class CurveColors : public QObject
     Q_OBJECT;
 
     public:
-        CurveColors(QwtPlot *plot) : isolated(false), plot(plot) {
+        CurveColors(QwtPlot *plot, bool allplot=false) : isolated(false), allplot(allplot), plot(plot) {
 
             // span slider appears when curve isolated
             // to enable zooming in and out
@@ -102,19 +105,37 @@ class CurveColors : public QObject
 
         void restoreState() {
 
+            // if a curve is hidden, don't show its labels we hack this
+            // by hiding any labels with the same color, a bit of a
+            // hack but works largely and avoids having to integrate
+            // with the plot object to understand the relationship between
+            // labels and curves (which is only really known by LTMPlot)
+            QList<QColor> shown;
+
+            // make all the curves visible (that should be)
+            // and remember the colors so we can show/hide 
+            // the associated labels
+            QHashIterator<QwtPlotSeriesItem *, bool> c(state);
+            while (c.hasNext()) {
+                c.next();
+                c.key()->setVisible(c.value());
+
+                if (c.value()) {
+                    if (c.key()->rtti() == QwtPlotItem::Rtti_PlotIntervalCurve) {
+                        shown << static_cast<QwtPlotIntervalCurve*>(c.key())->pen().color();
+                    } else if (c.key()->rtti() == QwtPlotItem::Rtti_PlotCurve) {
+                        shown << static_cast<QwtPlotCurve*>(c.key())->pen().color();
+                    }
+                }
+            }
+
             // show all labels
             QHashIterator<QwtPlotMarker *, QwtScaleWidget*> l(labels);
             while (l.hasNext()) {
                 l.next();
 
-                l.key()->setVisible(true);
-            }
-
-            // make all the curves visible (that should be)
-            QHashIterator<QwtPlotSeriesItem *, bool> c(state);
-            while (c.hasNext()) {
-                c.next();
-                c.key()->setVisible(c.value());
+                // we cannot hide via legend on allplot so always hide/show labels
+                l.key()->setVisible(allplot || shown.contains(l.key()->label().color()));
             }
 
             // make all the axes have the right color
@@ -142,8 +163,9 @@ class CurveColors : public QObject
             // Labels
             foreach(QwtPlotItem *item, plot->itemList(QwtPlotItem::Rtti_PlotMarker)) {
 
-                // ignore event / interval markers
-                if (static_cast<QwtPlotMarker*>(item)->lineStyle() == QwtPlotMarker::VLine) continue;
+                // ignore event / interval markers / blanks
+                if (static_cast<QwtPlotMarker*>(item)->lineStyle() == QwtPlotMarker::VLine ||
+                    static_cast<QwtPlotMarker*>(item)->label().text() == "") continue;
 
                 QwtScaleWidget *x = plot->axisWidget(static_cast<QwtPlotMarker*>(item)->yAxis());
                 labels.insert(static_cast<QwtPlotMarker*>(item), x);
@@ -213,6 +235,13 @@ class CurveColors : public QObject
             isolatedAxis = plot->axisWidget(id);
             isolatedDiv = plot->axisScaleDiv(id);
 
+            // if a curve is hidden, don't show its labels we hack this
+            // by hiding any labels with the same color, a bit of a
+            // hack but works largely and avoids having to integrate
+            // with the plot object to understand the relationship between
+            // labels and curves (which is only really known by LTMPlot)
+            QList<QColor> shown;
+
             // hide curves that are not ours
             QHashIterator<QwtPlotSeriesItem *, bool> c(state);
             while (c.hasNext()) {
@@ -220,7 +249,16 @@ class CurveColors : public QObject
 
                 // isolate on axis hover (but leave huighlighters alone)
                 if (c.key()->yAxis() == id || c.key()->yAxis() == QwtAxisId(QwtAxis::yLeft,2)) {
+
+                    // show and remember color
                     c.key()->setVisible(c.value());
+                    if (c.value()) {
+                        if (c.key()->rtti() == QwtPlotItem::Rtti_PlotIntervalCurve) {
+                            shown << static_cast<QwtPlotIntervalCurve*>(c.key())->pen().color();
+                        } else if (c.key()->rtti() == QwtPlotItem::Rtti_PlotCurve) {
+                            shown << static_cast<QwtPlotCurve*>(c.key())->pen().color();
+                        }
+                    }
 
                     if (showslider && c.key()->yAxis() == id) {
 
@@ -266,14 +304,15 @@ class CurveColors : public QObject
                 if (l.value() != ours) {
                     l.key()->setVisible(false);
                 } else {
-                    l.key()->setVisible(true);
+                    // we cannot hide via legend on allplot so always hide/show labels
+                    l.key()->setVisible(allplot || shown.contains(l.key()->label().color()));
                 }
             }
 
             isolated = true;
         }
 
-        bool isolated;
+        bool isolated, allplot;
 
     public slots:
         void zoomChanged() {
@@ -303,6 +342,7 @@ class CurveColors : public QObject
 };
 
 class AllPlot;
+class MergeAdjust;
 class AllPlotObject : public QObject
 {
     Q_OBJECT;
@@ -332,12 +372,23 @@ class AllPlotObject : public QObject
     QVector<QwtPlotCurve*> tmpReferenceLines;
 
     QwtPlotCurve *wattsCurve;
+    QwtPlotCurve *slopeCurve;
+    AllPlotSlopeCurve *altSlopeCurve;
     QwtPlotCurve *atissCurve;
     QwtPlotCurve *antissCurve;
+    QwtPlotCurve *rvCurve;
+    QwtPlotCurve *rcadCurve;
+    QwtPlotCurve *rgctCurve;
+    QwtPlotCurve *gearCurve;
+    QwtPlotCurve *smo2Curve;
+    QwtPlotCurve *thbCurve;
+    QwtPlotCurve *o2hbCurve;
+    QwtPlotCurve *hhbCurve;
     QwtPlotCurve *npCurve;
     QwtPlotCurve *xpCurve;
     QwtPlotCurve *apCurve;
     QwtPlotCurve *hrCurve;
+    QwtPlotCurve *tcoreCurve;
     QwtPlotCurve *speedCurve;
     QwtPlotCurve *accelCurve;
     QwtPlotCurve *wattsDCurve;
@@ -357,19 +408,35 @@ class AllPlotObject : public QObject
     QwtPlotCurve *rteCurve;
     QwtPlotCurve *lpsCurve;
     QwtPlotCurve *rpsCurve;
+    QwtPlotCurve *lpcoCurve;
+    QwtPlotCurve *rpcoCurve;
+    QwtPlotIntervalCurve *lppCurve;
+    QwtPlotIntervalCurve *rppCurve;
+    QwtPlotIntervalCurve *lpppCurve;
+    QwtPlotIntervalCurve *rpppCurve;
 
     // source data
     QVector<double> match;
     QVector<double> matchTime;
     QVector<double> matchDist;
+    QVector<QwtPlotMarker*> matchLabels;
     QVector<double> wprime;
     QVector<double> wprimeTime;
     QVector<double> wprimeDist;
 
     QVector<double> hrArray;
+    QVector<double> tcoreArray;
     QVector<double> wattsArray;
     QVector<double> atissArray;
     QVector<double> antissArray;
+    QVector<double> rvArray;
+    QVector<double> rcadArray;
+    QVector<double> rgctArray;
+    QVector<double> gearArray;
+    QVector<double> smo2Array;
+    QVector<double> thbArray;
+    QVector<double> o2hbArray;
+    QVector<double> hhbArray;
     QVector<double> npArray;
     QVector<double> xpArray;
     QVector<double> apArray;
@@ -383,6 +450,7 @@ class AllPlotObject : public QObject
     QVector<double> timeArray;
     QVector<double> distanceArray;
     QVector<double> altArray;
+    QVector<double> slopeArray;
     QVector<double> tempArray;
     QVector<double> windArray;
     QVector<double> torqueArray;
@@ -391,15 +459,34 @@ class AllPlotObject : public QObject
     QVector<double> rteArray;
     QVector<double> lpsArray;
     QVector<double> rpsArray;
+    QVector<double> lpcoArray;
+    QVector<double> rpcoArray;
+    QVector<double> lppbArray;
+    QVector<double> rppbArray;
+    QVector<double> lppeArray;
+    QVector<double> rppeArray;
+    QVector<double> lpppbArray;
+    QVector<double> rpppbArray;
+    QVector<double> lpppeArray;
+    QVector<double> rpppeArray;
 
     // smoothed data
     QVector<double> smoothWatts;
     QVector<double> smoothAT;
     QVector<double> smoothANT;
+    QVector<double> smoothRV;
+    QVector<double> smoothRCad;
+    QVector<double> smoothRGCT;
+    QVector<double> smoothGear;
+    QVector<double> smoothSmO2;
+    QVector<double> smoothtHb;
+    QVector<double> smoothO2Hb;
+    QVector<double> smoothHHb;
     QVector<double> smoothNP;
     QVector<double> smoothAP;
     QVector<double> smoothXP;
     QVector<double> smoothHr;
+    QVector<double> smoothTcore;
     QVector<double> smoothSpeed;
     QVector<double> smoothAccel;
     QVector<double> smoothWattsD;
@@ -410,6 +497,7 @@ class AllPlotObject : public QObject
     QVector<double> smoothTime;
     QVector<double> smoothDistance;
     QVector<double> smoothAltitude;
+    QVector<double> smoothSlope;
     QVector<double> smoothTemp;
     QVector<double> smoothWind;
     QVector<double> smoothTorque;
@@ -419,6 +507,12 @@ class AllPlotObject : public QObject
     QVector<double> smoothRTE;
     QVector<double> smoothLPS;
     QVector<double> smoothRPS;
+    QVector<double> smoothLPCO;
+    QVector<double> smoothRPCO;
+    QVector<QwtIntervalSample> smoothLPP;
+    QVector<QwtIntervalSample> smoothRPP;
+    QVector<QwtIntervalSample> smoothLPPP;
+    QVector<QwtIntervalSample> smoothRPPP;
     QVector<QwtIntervalSample> smoothRelSpeed;
 
     // highlighting intervals
@@ -442,7 +536,7 @@ class AllPlot : public QwtPlot
 
         // you can declare which series to plot, none means do them all
         // wanttext is to say if plot markers should have text
-        AllPlot(AllPlotWindow *parent, Context *context, 
+        AllPlot(QWidget *parent, AllPlotWindow *window, Context *context, 
                 RideFile::SeriesType series = RideFile::none, RideFile::SeriesType secSeries = RideFile::none, bool wanttext = true);
         ~AllPlot();
 
@@ -456,6 +550,7 @@ class AllPlot : public QwtPlot
         void setDataFromPlots(QList<AllPlot*>); // user for single series comparing
         void setDataFromObject(AllPlotObject *object, AllPlot *reference); // for allplot when one per ride in a stack
                                                                            // reference is for settings et al
+        void setMatchLabels(AllPlotObject *object); // set labels from object
 
         // convert from time/distance to index in *smoothed* datapoints
         int timeIndex(double) const;
@@ -473,6 +568,8 @@ class AllPlot : public QwtPlot
         // refresh data / plot parameters
         void recalc(AllPlotObject *objects);
         void setYMax();
+        void setLeftOnePalette(); // color of yLeft,1 axis
+        void setRightPalette(); // color of yRight,0 axis
         void setXTitle();
         void setHighlightIntervals(bool);
 
@@ -480,67 +577,87 @@ class AllPlot : public QwtPlot
         void confirmTmpReference(double value, int axis, bool allowDelete);
         QwtPlotCurve* plotReferenceLine(const RideFilePoint *referencePoint);
 
+        virtual void replot();
+
         // remembering state etc
         CurveColors *curveColors;
 
     public slots:
 
+        void setShow(RideFile::SeriesType, bool);
         void setShowAccel(bool show);
         void setShowPowerD(bool show);
         void setShowCadD(bool show);
         void setShowTorqueD(bool show);
         void setShowHrD(bool show);
         void setShowPower(int id);
+        void setShowAltSlope(int id);
+        void setShowSlope(bool show);
         void setShowNP(bool show);
         void setShowATISS(bool show);
         void setShowANTISS(bool show);
         void setShowXP(bool show);
         void setShowAP(bool show);
         void setShowHr(bool show);
+        void setShowTcore(bool show);
         void setShowSpeed(bool show);
         void setShowCad(bool show);
         void setShowAlt(bool show);
         void setShowTemp(bool show);
         void setShowWind(bool show);
+        void setShowRV(bool show);
+        void setShowRGCT(bool show);
+        void setShowRCad(bool show);
+        void setShowSmO2(bool show);
+        void setShowtHb(bool show);
+        void setShowO2Hb(bool show);
+        void setShowHHb(bool show);
+        void setShowGear(bool show);
         void setShowW(bool show);
         void setShowTorque(bool show);
         void setShowBalance(bool show);
         void setShowTE(bool show);
         void setShowPS(bool show);
+        void setShowPCO(bool show);
+        void setShowDC(bool show);
+        void setShowPPP(bool show);
         void setShowGrid(bool show);
         void setPaintBrush(int state);
         void setShadeZones(bool x) { shade_zones=x; }
         void setSmoothing(int value);
         void setByDistance(int value);
-        void setWantAxis(bool x) { wantaxis = x;}
-        void configChanged();
+        void setWantAxis(bool x, bool y=false) { wantaxis = x; wantxaxis = y;}
+        void configChanged(qint32);
 
         // for tooltip
         void pointHover(QwtPlotCurve*, int);
-        void intervalHover(RideFileInterval h);
+        void intervalHover(IntervalItem *h);
 
     protected:
 
         friend class ::AllPlotBackground;
         friend class ::AllPlotZoneLabel;
         friend class ::AllPlotWindow;
+        friend class ::MergeAdjust;
         friend class ::IntervalPlotData;
 
         // cached state
         RideItem *rideItem;
         AllPlotBackground *bg;
         QSettings *settings;
-        RideFileInterval hovered;
+        IntervalItem *hovered;
 
         // controls
         bool shade_zones;
         int showPowerState;
+        int showAltSlopeState;
         bool showATISS;
         bool showANTISS;
         bool showNP;
         bool showXP;
         bool showAP;
         bool showHr;
+        bool showTcore;
         bool showSpeed;
         bool showAccel;
         bool showPowerD;
@@ -549,6 +666,7 @@ class AllPlot : public QwtPlot
         bool showHrD;
         bool showCad;
         bool showAlt;
+        bool showSlope;
         bool showTemp;
         bool showWind;
         bool showW;
@@ -556,6 +674,17 @@ class AllPlot : public QwtPlot
         bool showBalance;
         bool showTE;
         bool showPS;
+        bool showPCO;
+        bool showDC;
+        bool showPPP;
+        bool showRV;
+        bool showRGCT;
+        bool showRCad;
+        bool showSmO2;
+        bool showtHb;
+        bool showO2Hb;
+        bool showHHb;
+        bool showGear;
 
         // plot objects
         AllPlotObject *standard;
@@ -566,6 +695,7 @@ class AllPlot : public QwtPlot
         // array / smooth state
         int smooth;
         bool bydist;
+        bool fill;
 
         // scope of plot (none means all, or just for a specific series
         RideFile::SeriesType scope;
@@ -578,13 +708,17 @@ class AllPlot : public QwtPlot
     private:
 
         AllPlot *referencePlot;
-        AllPlotWindow *parent;
-        bool wanttext, wantaxis;
+        QWidget *parent;
+        AllPlotWindow *window;
+        bool wanttext, wantaxis, wantxaxis;
         bool isolation;
         LTMToolTip *tooltip;
         LTMCanvasPicker *_canvasPicker; // allow point selection/hover
+        QFont labelFont;
 
+        void setAltSlopePlotStyle (AllPlotSlopeCurve *curve);
         static void nextStep( int& step );
+
 };
 
 // Produce Labels for X-Axis
@@ -592,17 +726,19 @@ class ScaleScaleDraw: public QwtScaleDraw
 {
     public:
 
-        ScaleScaleDraw(double factor=1.0f) : factor(factor) {}
+        ScaleScaleDraw(double factor=1.0f) : factor(factor), decimals(0) {}
 
         void setFactor(double f) { factor=f; }
+        void setDecimals(int d) { decimals=d; }
 
         virtual QwtText label(double v) const {
-            return QString("%1").arg(v * factor, 0, 'f', 0);
+            return QString("%1").arg(v * factor, 0, 'f', decimals);
         }
 
     private:
 
         double factor;
+        int decimals;
 };
 #endif // _GC_AllPlot_h
 
