@@ -17,12 +17,14 @@
  */
 
 #include "Bin2RideFile.h"
-#include <math.h>
+#include <cmath>
 
 #define START 0x210
 #define UNIT_VERSION 0x2000
 #define SYSTEM_INFO 0x2003
 
+#define BIN2_DEBUG false // debug traces
+
 static int bin2FileReaderRegistered =
     RideFileFactory::instance().registerReader(
         "bin2", "Joule GPS File", new Bin2FileReader());
@@ -41,13 +43,15 @@ struct Bin2FileReaderState
     double last_interval_secs;
 
     bool jouleGPS;
+    bool jouleGPS_GPSPlus;
+
     bool stopped;
 
     QString deviceInfo;
 
     Bin2FileReaderState(QFile &file, QStringList &errors) :
         file(file), errors(errors), rideFile(NULL), secs(0), km(0),
-        interval(0), last_interval_secs(0.0),  jouleGPS(true), stopped(true)
+        interval(0), last_interval_secs(0.0),  jouleGPS(true), jouleGPS_GPSPlus(true), stopped(true)
     {
 
     }
@@ -88,7 +92,8 @@ struct Bin2FileReaderState
             if (count)
                 *count += 1;
 
-            res += c;
+            if (c != 0)
+                res += c;
         }
         return res;
     }
@@ -109,10 +114,16 @@ struct Bin2FileReaderState
     {
         QDateTime date = read_date(bytes_read, sum);
 
-        if (jouleGPS) {
+        if (jouleGPS_GPSPlus) {
             read_bytes(1, bytes_read, sum); // dummy
             read_bytes(4, bytes_read, sum); // time_moving
-            *secs = double(read_bytes(4, bytes_read, sum));
+
+            // time
+            double newsecs = double(read_bytes(4, bytes_read, sum));
+            if (BIN2_DEBUG)
+                qDebug() << "read_RTC_mark new ridetime" << newsecs << "secs (old" << *secs <<"secs)";
+
+            *secs = newsecs;
             read_bytes(16, bytes_read, sum);  // dummy
         } else {
             read_bytes(13, bytes_read, sum);  // dummy
@@ -126,8 +137,11 @@ struct Bin2FileReaderState
         int intervalNumber = read_bytes(1, bytes_read, sum);
 
         read_bytes(2, bytes_read, sum); // dummy
-        *secs = double(read_bytes(4, bytes_read, sum));
-        if (jouleGPS)
+        double rideTime = double(read_bytes(4, bytes_read, sum));
+        if (BIN2_DEBUG)
+            qDebug() << "read_interval_mark" << rideTime << "at" << *secs <<"secs";
+
+        if (jouleGPS_GPSPlus)
             read_bytes(24, bytes_read, sum); // dummy
         else
             read_bytes(12, bytes_read, sum); // dummy
@@ -149,7 +163,7 @@ struct Bin2FileReaderState
         double lng = 0.0;
         double km = 0.0;
 
-        if (jouleGPS) {
+        if (jouleGPS_GPSPlus) {
             cad = read_bytes(1, bytes_read, sum);
 
             read_bytes(1, bytes_read, sum); // pedal_smoothness
@@ -232,7 +246,7 @@ struct Bin2FileReaderState
             lng = lng/10000000.0;
 
         // 0.0 values at end are for garmin vector torque efficiency/pedal smoothness which are not available
-        rideFile->appendPoint(*secs, cad, hr, km, kph, nm, watts, alt, lng, lat, 0.0, 0, temp, lrbal, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, interval);
+        rideFile->appendPoint(*secs, cad, hr, km, kph, nm, watts, alt, lng, lat, 0.0, 0, temp, lrbal, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, interval);
         (*secs)++;
     }
 
@@ -252,7 +266,7 @@ struct Bin2FileReaderState
 
         rideFile->setStartTime(t);
 
-        if (jouleGPS)
+        if (jouleGPS_GPSPlus)
         {
             read_bytes(148, bytes_read, sum);
 
@@ -349,7 +363,7 @@ struct Bin2FileReaderState
 
             if (page_number == 0) {
                 // Page #0
-                if (jouleGPS) {
+                if (jouleGPS_GPSPlus) {
                     read_ride_summary(&bytes_read, &sum);
                     read_interval_summary(&bytes_read, &sum);
                     read_username(&bytes_read, &sum);
@@ -370,7 +384,7 @@ struct Bin2FileReaderState
                 read_bytes(1, &bytes_read, &sum); // checksum
 
             } else {
-                if (page_number == 1 && !jouleGPS)  {
+                if (page_number == 1 && !jouleGPS_GPSPlus)  {
                     //page still contains 48 interval_summary
                     int finish = length+6-bytes_read;
 
@@ -403,20 +417,21 @@ struct Bin2FileReaderState
             read_bytes(2, &bytes_read, &sum); // length
             uint16_t page_number = read_bytes(2, &bytes_read, &sum); // Page #
 
-            if (page_number == 2 && !jouleGPS) {
+            if (page_number == 2 && !jouleGPS_GPSPlus) {
                 //page still contains 5 interval_summary in 2 blocks of 256k
                 read_bytes(512, &bytes_read, &sum);
 
             }
 
             if (page_number > 0) {
-                //qDebug() << "page_number" << page_number;
+                if (BIN2_DEBUG)
+                    qDebug() << "page_number" << page_number;
                 int nb_record = 128;
 
                 // Page # >0
                 // Joule GPS : 128 x record (32k)
 
-                if (!jouleGPS) {
+                if (!jouleGPS_GPSPlus) {
                     // Page # = 2
                     // Joule  : 168 x record (20k)
                     // Page # > 1
@@ -435,19 +450,47 @@ struct Bin2FileReaderState
                     int flag = read_bytes(1, &bytes_read, &sum);
                     //b0..b1: "00" Detail Record
                     //b0..b1: "01" RTC Mark Record
-                    //b0..b1: "00" Interval Record
+                    //b0..b1: "10" Session Mark (Joule GPS)
+                    //b0..b1: "10" Interval Record (Joule 1.0)
+                    //b0..b1: "11" Interval Record (Joule GPS)
+                    //b0..b1: "11" Invalid ( 0xFF)
+
 
-                    //b2: reserved
                     //b3: Power was calculated
-                    //b4: HPR packet missing
+                    //b4: HR packet missing
                     //b5: CAD packet missing
                     //b6: PWR packet missing
                     //b7: Power data = old (No new power calculated)
 
+                    if (BIN2_DEBUG && (flag & 0x03) != 0) {
+                        qDebug() << "flag B0..B1" << (flag & 0x03) << "flag" << flag;
+                        if (flag == 0xff) {
+                            qDebug() << "Invalid record";
+                        }
+                        else {
+                            if ((flag & 0x08) == 0x08) {
+                                qDebug() << "Power was calculated";
+                            }
+                            if ((flag & 0x10) == 0x10) {
+                                qDebug() << "HPR packet missing";
+                            }
+                            if ((flag & 0x20) == 0x20) {
+                                qDebug() << "CAD packet missing";
+                            }
+                            if ((flag & 0x40) == 0x40) {
+                                qDebug() << "PWR packet missing";
+                            }
+                            if ((flag & 0x80) == 0x80) {
+                                qDebug() << "Power data = old";
+                            }
+                        }
+                    }
+
                     if (flag == 0xff) {
                         // means invalid entry
-                        //qDebug() << "invalid" << secs;
-                        if (jouleGPS)
+                        if (BIN2_DEBUG)
+                            qDebug() << "invalid at" << secs << "secs";
+                        if (jouleGPS_GPSPlus)
                             read_bytes(31, &bytes_read, &sum); // dummy
                         else
                             read_bytes(19, &bytes_read, &sum); // dummy
@@ -455,23 +498,29 @@ struct Bin2FileReaderState
                         (secs)++;
                     }
                     else if ((flag & 0x03) == 0x01){
+                        // The RTC Mark is written when there is a gap in the ride data,
+                        // or when the internal Real Time Clock is adjusted.
                         t= read_RTC_mark(&secs, &bytes_read, &sum);
                     }
-                    else if ((jouleGPS && (flag & 0x03) == 0x03) || (!jouleGPS && (flag & 0x03) == 0x02)) {
+                    else if ((jouleGPS_GPSPlus && (flag & 0x03) == 0x03) || (!jouleGPS_GPSPlus && (flag & 0x03) == 0x02)) {
+                        // The Interval Mark immediately precedes a Detail Record at the same RTC time.
                         int t = read_interval_mark(&secs, &bytes_read, &sum);
                         interval = t;
                     }
                     else if ((flag & 0x03) == 0x00 ){
+                        // The detail record contains current telemetry data.
                         read_detail_record(&secs, &bytes_read, &sum);
                     } else {
-                        //qDebug() << "unknow" << secs;
-                        if (jouleGPS)
+                        if (BIN2_DEBUG)
+                            qDebug() << "UNKNOW: flag" << flag << "at" << secs << "secs";
+
+                        if (jouleGPS_GPSPlus)
                             read_bytes(31, &bytes_read, &sum); // dummy
                         else
                             read_bytes(19, &bytes_read, &sum); // dummy
                     }
 
-                    if (!jouleGPS && (i+1)%12 == 0)
+                    if (!jouleGPS_GPSPlus && (i+1)%12 == 0)
                         read_bytes(16, &bytes_read, &sum); // unused
 
                 }
@@ -504,19 +553,24 @@ struct Bin2FileReaderState
 
             if (major_version == 18)
             {
+                jouleGPS_GPSPlus = false;
                 jouleGPS = false;
             } else
             {
-                jouleGPS = true;
+                jouleGPS_GPSPlus = true;
+                if (major_version == 22)
+                    jouleGPS = false;
             }
 
 
-            if (jouleGPS)
+            if (jouleGPS_GPSPlus)
                 data_version = read_bytes(2, &bytes_read, &sum);
 
             QString version = QString(minor_version<100?"%1.0%2 (%3)":"%1.%2 (%3)").arg(major_version).arg(minor_version).arg(data_version);
             if (jouleGPS)
                 deviceInfo += "Joule GPS";
+            else if (jouleGPS_GPSPlus)
+                deviceInfo += "Joule GPS+";
             else
                 deviceInfo += "Joule";
 
@@ -543,7 +597,7 @@ struct Bin2FileReaderState
         {
             read_bytes(2, &bytes_read, &sum); // length
 
-            if (jouleGPS)
+            if (jouleGPS_GPSPlus)
                 read_bytes(52, &bytes_read, &sum);
             else
                 read_bytes(50, &bytes_read, &sum);
@@ -551,7 +605,7 @@ struct Bin2FileReaderState
             uint16_t odometer = read_bytes(8, &bytes_read, &sum);
             deviceInfo += QString("Odometer %1km\n").arg(odometer/1000.0);
 
-            if (!jouleGPS)
+            if (!jouleGPS_GPSPlus)
                 read_bytes(34, &bytes_read, &sum);
 
             read_bytes(1, &bytes_read, &sum); // checksum
@@ -580,12 +634,14 @@ struct Bin2FileReaderState
 
         if (jouleGPS)
             rideFile->setDeviceType("Joule GPS");
+        else if (jouleGPS_GPSPlus)
+            rideFile->setDeviceType("Joule GPS+");
         else
             rideFile->setDeviceType("Joule");
 
         bytes_read += read_system_info();
         bytes_read += read_summary_page();
-        if (!jouleGPS)
+        if (!jouleGPS_GPSPlus)
             bytes_read += read_summary_page();
 
         while (!stop && (bytes_read < data_size)) {
