@@ -28,12 +28,14 @@
 #include "Colors.h"
 #include "Units.h"
 #include "TimeUtils.h"
+#include "HelpWhatsThis.h"
 
 #include <QDebug>
 
 BingMap::BingMap(Context *context) : GcChartWindow(context), context(context), range(-1), current(NULL)
 {
     setControls(NULL);
+
     setContentsMargins(0,0,0,0);
     layout = new QVBoxLayout();
     layout->setSpacing(0);
@@ -47,6 +49,9 @@ BingMap::BingMap(Context *context) : GcChartWindow(context), context(context), r
     view->setAcceptDrops(false);
     layout->addWidget(view);
 
+    HelpWhatsThis *help = new HelpWhatsThis(view);
+    view->setWhatsThis(help->getWhatsThisText(HelpWhatsThis::ChartRides_Map));
+
     webBridge = new BWebBridge(context, this);
 
     connect(this, SIGNAL(rideItemChanged(RideItem*)), this, SLOT(rideSelected()));
@@ -54,16 +59,16 @@ BingMap::BingMap(Context *context) : GcChartWindow(context), context(context), r
     connect(context, SIGNAL(intervalsChanged()), webBridge, SLOT(intervalsChanged()));
     connect(context, SIGNAL(intervalSelected()), webBridge, SLOT(intervalsChanged()));
     connect(context, SIGNAL(intervalZoom(IntervalItem*)), this, SLOT(zoomInterval(IntervalItem*)));
-    connect(context, SIGNAL(configChanged()), this, SLOT(configChanged()));
+    connect(context, SIGNAL(configChanged(qint32)), this, SLOT(configChanged(qint32)));
 
     first = true;
 
     // get the colors setup for first run
-    configChanged();
+    configChanged(CONFIG_APPEARANCE);
 }
 
 void
-BingMap::configChanged()
+BingMap::configChanged(qint32)
 {
     setProperty("color", GColor(CPLOTBACKGROUND));
     rideSelected();
@@ -81,9 +86,15 @@ BingMap::rideSelected()
     // Route metadata ...
     setSubTitle(ride->ride()->getTag("Route", tr("Route")));
 
-    range =ride->zoneRange();
-    if(range < 0) rideCP = 300;  // default cp to 300 watts
-    else rideCP = ride->zones->getCP(range);
+    range=-1;
+    rideCP=300;
+
+    if (context->athlete->zones()) {
+
+        // get the right range and CP
+        range = context->athlete->zones()->whichRange(ride->dateTime.date());
+        if (range >= 0) rideCP = context->athlete->zones()->getCP(range);
+    }
 
     loadRide();
 }
@@ -339,7 +350,7 @@ BingMap::drawShadedRoute()
 }
 
 //
-// Static helper - havervine formaula for calculating the distance
+// Static helper - havervine formula for calculating the distance
 //                 between 2 geo co-ordinates
 //
 static const double DEG_TO_RAD = 0.017453292519943295769236907684886;
@@ -455,12 +466,12 @@ BingMap::createMarkers()
     //
     // INTERVAL MARKERS
     //
-    if (context->athlete->allIntervalItems() == NULL) return; // none to do, we are all done then
+    if (myRideItem->intervals().count() == 0) return;
 
     int interval=0;
-    foreach (const RideFileInterval x, myRideItem->ride()->intervals()) {
+    foreach (IntervalItem *x, myRideItem->intervals()) {
 
-        int offset = myRideItem->ride()->intervalBegin(x);
+        int offset = myRideItem->ride()->intervalBeginSecs(x->start);
         code = QString("{ var latlng = new Microsoft.Maps.Location(%1,%2);\n" 
                    "var pushpinOptions = { };\n"
                    "var pushpin = new Microsoft.Maps.Pushpin(latlng, pushpinOptions);\n"
@@ -470,7 +481,7 @@ BingMap::createMarkers()
                    " }")
                    .arg(myRideItem->ride()->dataPoints()[offset]->lat,0,'g',GPS_COORD_TO_STRING)
                    .arg(myRideItem->ride()->dataPoints()[offset]->lon,0,'g',GPS_COORD_TO_STRING)
-                   .arg(x.name)
+                   .arg(x->name)
                    .arg(interval);
 
         view->page()->mainFrame()->evaluateJavaScript(code);
@@ -530,22 +541,8 @@ void BWebBridge::call(int count) { qDebug()<<"webBridge call:"<<count; }
 int
 BWebBridge::intervalCount()
 {
-    int highlighted;
-    highlighted = 0;
     RideItem *rideItem = gm->property("ride").value<RideItem*>();
-
-    if (context->athlete->allIntervalItems() == NULL ||
-        rideItem == NULL || rideItem->ride() == NULL) return 0; // not inited yet!
-
-    for (int i=0; i<context->athlete->allIntervalItems()->childCount(); i++) {
-        IntervalItem *current = dynamic_cast<IntervalItem *>(context->athlete->allIntervalItems()->child(i));
-        if (current != NULL) {
-            if (current->isSelected() == true) {
-                ++highlighted;
-            }
-        }
-    }
-    return highlighted;
+    if (rideItem) return rideItem->intervalsSelected().count();
 }
 
 // get a latlon array for the i'th selected interval
@@ -553,41 +550,29 @@ QVariantList
 BWebBridge::getLatLons(int i)
 {
     QVariantList latlons;
-    int highlighted=0;
     RideItem *rideItem = gm->property("ride").value<RideItem*>();
 
-    if (context->athlete->allIntervalItems() == NULL ||
-       rideItem ==NULL || rideItem->ride() == NULL) return latlons; // not inited yet!
+    if (rideItem == NULL) return latlons;
 
-    if (i) {
+    // valid highlighted interval ?
+    if (i < 0 || i > rideItem->intervalsSelected().count()) {
 
-        // get for specific interval
-        for (int j=0; j<context->athlete->allIntervalItems()->childCount(); j++) {
-            IntervalItem *current = dynamic_cast<IntervalItem *>(context->athlete->allIntervalItems()->child(j));
-            if (current != NULL) {
-                if (current->isSelected() == true) {
-                    ++highlighted;
+        IntervalItem *current = rideItem->intervalsSelected().at(i);
 
-                    // this one!
-                    if (highlighted==i) {
+        // so this one is the interval we need.. lets
+        // snaffle up the points in this section
+        foreach (RideFilePoint *p1, rideItem->ride()->dataPoints()) {
+            if (p1->secs+rideItem->ride()->recIntSecs() > current->start
+                && p1->secs< current->stop) {
 
-                        // so this one is the interval we need.. lets
-                        // snaffle up the points in this section
-                        foreach (RideFilePoint *p1, rideItem->ride()->dataPoints()) {
-                            if (p1->secs+rideItem->ride()->recIntSecs() > current->start
-                                && p1->secs< current->stop) {
-
-                                if (p1->lat || p1->lon) {
-                                    latlons << p1->lat;
-                                    latlons << p1->lon;
-                                }
-                            }
-                        }
-                        return latlons;
-                    }
+                if (p1->lat || p1->lon) {
+                    latlons << p1->lat;
+                    latlons << p1->lon;
                 }
             }
         }
+        return latlons;
+
     } else {
 
         // get latlons for entire route
@@ -616,6 +601,12 @@ BWebBridge::drawOverlays()
 void
 BWebBridge::toggleInterval(int x)
 {
+    RideItem *rideItem = gm->property("ride").value<RideItem*>();
+    if (x < 0 || rideItem->intervals().count() >= x) return;
+
+//XXX WHEN DECIDED HOW TO SELECT/UNSELECT INTERVALS
+#if 0
     IntervalItem *current = dynamic_cast<IntervalItem *>(context->athlete->allIntervalItems()->child(x));
     if (current) current->setSelected(!current->isSelected());
+#endif
 }
