@@ -79,7 +79,7 @@ DialWindow::DialWindow(Context *context) :
 
     // get updates..
     connect(context, SIGNAL(telemetryUpdate(RealtimeData)), this, SLOT(telemetryUpdate(RealtimeData)));
-    connect(context, SIGNAL(configChanged()), this, SLOT(seriesChanged()));
+    connect(context, SIGNAL(configChanged(qint32)), this, SLOT(seriesChanged()));
     connect(context, SIGNAL(stop()), this, SLOT(stop()));
     connect(context, SIGNAL(start()), this, SLOT(start()));
     connect(context, SIGNAL(newLap()), this, SLOT(onNewLap()));
@@ -195,11 +195,17 @@ DialWindow::telemetryUpdate(const RealtimeData &rtData)
 
     case RealtimeData::LRBalance:
         {
-            double tot = rtData.getWatts() + rtData.getAltWatts();
-            double left = rtData.getWatts() / tot * 100.00f;
-            double right = 100.00 - left;
-            if (tot < 0.1) left = right = 0;
-            valueLabel->setText(QString("%1 / %2").arg(left, 0, 'f', 0).arg(right, 0, 'f', 0));
+          double left = 0; double right = 0;
+          if (value == 0) { // no LR Balance provided - so use previous logic
+              double tot = rtData.getWatts() + rtData.getAltWatts();
+              left = rtData.getWatts() / tot * 100.00f;
+              right = 100.00 - left;
+              if (tot < 0.1) left = right = 0;
+          } else {
+              left = 100.00 - value; // value in ANT message is the "right" pedal portion
+              right = value;
+          }
+          valueLabel->setText(QString("%1 / %2").arg(left, 0, 'f', 0).arg(right, 0, 'f', 0));
         }
         break;
 
@@ -253,6 +259,10 @@ DialWindow::telemetryUpdate(const RealtimeData &rtData)
         valueLabel->setText(QString("%1").arg(round(sum/1000))); // kJoules
         break;
 
+    case RealtimeData::Wbal:
+        valueLabel->setText(QString("%1").arg(rtData.getWbal()/1000.00f, 0, 'f', 1)); // kJoules
+        break;
+
     // COGGAN Metrics
     case RealtimeData::NP:
     case RealtimeData::IF:
@@ -429,6 +439,25 @@ DialWindow::telemetryUpdate(const RealtimeData &rtData)
         }
         break;
 
+    case RealtimeData::SmO2:
+        valueLabel->setText(QString("%1%").arg(value, 0, 'f', 0));
+        break;
+    case RealtimeData::tHb:
+    case RealtimeData::O2Hb:
+    case RealtimeData::HHb:
+        valueLabel->setText(QString("%1").arg(value, 0, 'f', 1));
+        break;
+
+
+    case RealtimeData::LeftPedalSmoothness:
+    case RealtimeData::RightPedalSmoothness:
+    case RealtimeData::LeftTorqueEffectiveness:
+    case RealtimeData::RightTorqueEffectiveness:
+        {
+          valueLabel->setText(QString("%1").arg(value, 0, 'f', 0));
+        }
+        break;
+
     default:
         valueLabel->setText(QString("%1").arg(round(displayValue)));
         break;
@@ -513,6 +542,10 @@ void DialWindow::seriesChanged()
             foreground = GColor(CSPEED);
             break;
 
+    case RealtimeData::Wbal:
+            foreground = GColor(CWBAL);
+            break;
+
     case RealtimeData::Cadence:
     case RealtimeData::AvgCadence:
     case RealtimeData::AvgCadenceLap:
@@ -528,15 +561,49 @@ void DialWindow::seriesChanged()
     case RealtimeData::AltWatts:
             foreground = GColor(CALTPOWER);
             break;
+
+    case RealtimeData::SmO2:
+            foreground = GColor(CSMO2);
+            break;
+
+    case RealtimeData::tHb:
+            foreground = GColor(CTHB);
+            break;
+
+    case RealtimeData::O2Hb:
+            foreground = GColor(CO2HB);
+            break;
+
+    case RealtimeData::HHb:
+            foreground = GColor(CHHB);
+            break;
+
+    case RealtimeData::LeftPedalSmoothness:
+            foreground = GColor(CLPS);
+            break;
+
+    case RealtimeData::RightPedalSmoothness:
+            foreground = GColor(CRPS);
+            break;
+
+    case RealtimeData::LeftTorqueEffectiveness:
+            foreground = GColor(CLTE);
+            break;
+
+    case RealtimeData::RightTorqueEffectiveness:
+           foreground = GColor(CRTE);
+           break;
+
     }
 
-    // ugh. we use style sheets becuase palettes don't work on labels
+    // ugh. we use style sheets because palettes don't work on labels
     background = GColor(CTRAINPLOTBACKGROUND);
     setProperty("color", background);
     QString sh = QString("QLabel { background: %1; color: %2; }")
                  .arg(background.name())
                  .arg(foreground.name());
     valueLabel->setStyleSheet(sh);
+    repaint();
 }
 
 void
