@@ -20,7 +20,7 @@
 #include "RideMetric.h"
 #include "BestIntervalDialog.h"
 #include "HrZones.h"
-#include <math.h>
+#include <cmath>
 #include <QApplication>
 
 class HrZoneTime : public RideMetric {
@@ -41,13 +41,14 @@ public:
         setPrecision(0);
         setConversion(1.0);
     }
+    bool isTime() const { return true; }
     void setLevel(int level) { this->level=level-1; } // zones start from zero not 1
     void compute(const RideFile *ride, const Zones *, int, const HrZones *hrZone, int hrZoneRange,
                  const QHash<QString,RideMetric*> &, const Context *)
     {
         seconds = 0;
         // get zone ranges
-        if (hrZone && hrZoneRange >= 0) {
+        if (hrZone && hrZoneRange >= 0 && ride->areDataPresent()->hr) {
             // iterate and compute
             foreach(const RideFilePoint *point, ride->dataPoints()) {
                 if (hrZone->whichZone(hrZoneRange, point->hr) == level)
