@@ -22,6 +22,7 @@
 #include "Settings.h"
 #include "LibraryParser.h"
 #include "TrainDB.h"
+#include "HelpWhatsThis.h"
 #include <QVBoxLayout>
 #include <QHeaderView>
 #include <QLabel>
@@ -64,7 +65,7 @@ Library::initialise(QDir home)
 
         // lets read library.xml, if not there then add workout config
         // if thats not set then add home
-        QFile libraryXML(home.absolutePath() + "/library.xml");
+        QFile libraryXML(home.canonicalPath() + "/library.xml");
         if (libraryXML.exists() == true) {
 
             // parse it!
@@ -154,7 +155,7 @@ Library::importFiles(Context *context, QStringList files)
             // set target directory
             QString workoutDir = appsettings->value(NULL, GC_WORKOUTDIR).toString();
             if (workoutDir == "") {
-                QDir root = context->athlete->home;
+                QDir root = context->athlete->home->root();
                 root.cdUp();
                 workoutDir = root.absolutePath();
             }
@@ -178,7 +179,7 @@ Library::importFiles(Context *context, QStringList files)
         trainDB->endLUW();
 
         // now write to disk.. any refs we added
-        LibraryParser::serialize(context->athlete->home);
+        LibraryParser::serialize(context->athlete->home->root());
 
         // Tell traintool to select what was imported
         if (videos.count()) context->notifySelectVideo(videos[0]);
@@ -199,7 +200,7 @@ Library::removeRef(Context *context, QString ref)
     int index = refs.indexOf(ref);
     if (index >= 0) {
         refs.removeAt(index);
-        LibraryParser::serialize(context->athlete->home);
+        LibraryParser::serialize(context->athlete->home->root());
     }
 }
 
@@ -210,6 +211,8 @@ LibrarySearchDialog::LibrarySearchDialog(Context *context) : context(context)
 {
     setAttribute(Qt::WA_DeleteOnClose);
     setWindowTitle(tr("Search for Workouts and Media"));
+    HelpWhatsThis *help = new HelpWhatsThis(this);
+    this->setWhatsThis(help->getWhatsThisText(HelpWhatsThis::MenuBar_Tools_ScanDisk_WorkoutVideo));
     setMinimumWidth(600);
 
     searcher = NULL;
@@ -402,10 +405,10 @@ LibrarySearchDialog::search()
 
 
             // now write to disk..
-            LibraryParser::serialize(context->athlete->home);
+            LibraryParser::serialize(context->athlete->home->root());
         }
 
-        // ok, we;ve completed a search without aborting
+        // ok, we've completed a search without aborting
         // so lets rebuild the database of workouts and videos
         // using what we found...
         updateDB();
@@ -702,7 +705,7 @@ WorkoutImportDialog::WorkoutImportDialog(Context *context, QStringList files) :
     okButton = new QPushButton(tr("OK"), this);
     okButton->setDefault(true);
 
-    // if importing workoouts..
+    // if importing workouts..
     overwrite = new QCheckBox(tr("Overwite existing files"),this);
     if (!workouts.count()) overwrite->hide();
 
@@ -746,12 +749,12 @@ WorkoutImportDialog::import()
     }
 
     // now write to disk..
-    LibraryParser::serialize(context->athlete->home);
+    LibraryParser::serialize(context->athlete->home->root());
 
     // set target directory
     QString workoutDir = appsettings->value(NULL, GC_WORKOUTDIR).toString();
     if (workoutDir == "") {
-        QDir root = context->athlete->home;
+        QDir root = context->athlete->home->root();
         root.cdUp();
         workoutDir = root.absolutePath();
     }
