@@ -73,7 +73,9 @@ class ModelDataColor : public Color
             } if (iszones == true) { // zone 0 is set to zone 1 to distinguish from no value
                 cHSV = zonecolor.value(val-1, QColor(0,0,0));
             } else if (val) {
-                cHSV.setHsv((double)255 * (double)((val-min)/(max-min)), 255, 255);
+                int red = (double)255 * (double)((val-min)/(max-min));
+                if (red < 0 || red > 255) red = 127;
+                cHSV.setHsv(red, 255, 255);
             }
             cRGB = cHSV.convertTo(QColor::Rgb);
             colour.r = (double) cRGB.red()/(double)255;
@@ -115,7 +117,9 @@ class ModelDataColor : public Color
                 // just add 100 colors graded from min to max
                 for (i=0, val=min; i<100; i++, val += ((max-min)/100)) {
 
-                    cHSV.setHsv((double)255 * (double)((val-min)/(max-min)), 255, 255);
+                    int red = (double)255 * (double)((val-min)/(max-min));
+                    if (red < 0 || red > 255) red = 127;
+                    cHSV.setHsv(red, 255, 255);
                     cRGB = cHSV.convertTo(QColor::Rgb);
                     colour.r = (double) cRGB.red()/(double)255;
                     colour.g = (double) cRGB.green()/(double)255;
@@ -237,6 +241,14 @@ ModelDataProvider::pointType(const RideFilePoint *point, int type)
         // logic needed and I'm just lookup table!
         case MODEL_XYTIME : return 1;
         case MODEL_POWERZONE : return point->watts;
+        case MODEL_LRBALANCE : return point->lrbalance;
+        case MODEL_RV : return point->rvert;
+        case MODEL_RGCT : return point->rcontact;
+        case MODEL_RCAD : return point->rcad;
+        case MODEL_GEAR : return point->gear;
+        case MODEL_SMO2 : return point->smo2;
+        case MODEL_THB : return point->thb;
+        case MODEL_SLOPE : return point->slope;
     }
     return 0; // ? unknown channel ?
 }
@@ -281,6 +293,14 @@ ModelDataProvider::describeType(int type, bool longer)
             // logic needed and I'm just lookup table!
             case MODEL_XYTIME : return tr("Time at X/Y (%)");
             case MODEL_POWERZONE : return tr("Power Zone");
+            case MODEL_LRBALANCE : return (tr("L/R Balance"));
+            case MODEL_RV :  return (tr("Running Vertical Oscillation"));
+            case MODEL_RGCT : return (tr("Running Ground Contact Time"));
+            case MODEL_RCAD :  return (tr("Running Cadence"));
+            case MODEL_GEAR :  return (tr("Gear Ratio"));
+            case MODEL_SMO2 :  return (tr("Muscle Oxygen"));
+            case MODEL_THB :  return (tr("Haemoglobin Mass"));
+            case MODEL_SLOPE : return (tr("Slope (gradient)"));
         }
         return tr("Unknown");; // ? unknown channel ?
     } else {
@@ -301,6 +321,14 @@ ModelDataProvider::describeType(int type, bool longer)
             case MODEL_POWERZONE : return tr("Zone");
             case MODEL_CPV : return tr("CPV");
             case MODEL_AEPF : return tr("AEPF");
+            case MODEL_LRBALANCE : return (tr("Balance"));
+            case MODEL_RV :  return (tr("RV"));
+            case MODEL_RGCT : return (tr("GCT"));
+            case MODEL_RCAD :  return (tr("Run Cad"));
+            case MODEL_GEAR :  return (tr("Gear"));
+            case MODEL_SMO2 :  return (tr("SmO2"));
+            case MODEL_THB :  return (tr("tHb"));
+            case MODEL_SLOPE : return (tr("Slope"));
         }
         return tr("None");; // ? unknown channel ?
     }
@@ -337,7 +365,7 @@ ModelDataProvider::ModelDataProvider (ModelPlot &plot, ModelSettings *settings)
     if (cranklength == 0.0) cranklength = 0.175;
 
     // Run through the ridefile points putting the selected
-    // values into the approprate bins
+    // values into the appropriate bins
     settings->colorProvider->color.clear();
     settings->colorProvider->num.clear();
     settings->colorProvider->zonecolor.clear();
@@ -487,8 +515,9 @@ ModelDataProvider::ModelDataProvider (ModelPlot &plot, ModelSettings *settings)
     // COLOR
     // if needed convert average power to the related power zone
     // but only if ranges are defined i.e. user has set CP
-	const Zones *zones = settings->ride->zones;
-	int zone_range = settings->ride->zoneRange();
+	const Zones *zones = plot.context->athlete->zones();
+    int zone_range = -1;
+    if (zones) zone_range = zones->whichRange(settings->ride->dateTime.date());
     if (settings->color == MODEL_POWERZONE && zone_range >= 0) {
         // if we need to color by power zones
         // zones are setup and we want to color by them
@@ -655,7 +684,7 @@ ModelDataProvider::ModelDataProvider (ModelPlot &plot, ModelSettings *settings)
     setDomain(maxbinx,minbinx,minbiny,maxbiny); // max x/y vals
 
     // set the barsize to a sensible radius (20% space)
-    // the +settting->xbin bit is to offset the additional mx/my bug
+    // the +setting->xbin bit is to offset the additional mx/my bug
     double xr = 0.8 * (((double)settings->xbin/((double)(maxbinx-minbinx)+(settings->xbin))) / 2.0);
     double yr = 0.8 * (((double)settings->ybin/((double)(maxbiny-minbiny)+(settings->ybin))) / 2.0);
 
@@ -712,7 +741,7 @@ ModelDataProvider::ModelDataProvider (ModelPlot &plot, ModelSettings *settings)
     plot.setOrtho(false);
     plot.setSmoothMesh(true);
 
-    // cordinates - labels, gridlines, tic markers etc
+    // coordinates - labels, gridlines, tic markers etc
     if (settings->gridlines == true)
         plot.coordinates()->setGridLines(true, true, Qwt3D::BACK | Qwt3D::LEFT | Qwt3D::FLOOR);
     else
@@ -872,14 +901,14 @@ ModelPlot::ModelPlot(Context *context, QWidget *parent, ModelSettings *settings)
     resetViewPoint();
 
     // set colors
-    configChanged();
+    configChanged(CONFIG_APPEARANCE);
 
     updateData();
     updateGL();
 }
 
 void
-ModelPlot::configChanged()
+ModelPlot::configChanged(qint32)
 {
     // setColors bg
     QColor rgba = GColor(CPLOTBACKGROUND);
