@@ -97,8 +97,20 @@ PDModel::setIntervals(double sanI1, double sanI2, double anI1, double anI2,
 // cp, tau and t0 values needed for the model
 // this is the function originally found in CPPlot
 void
-PDModel::deriveCPParameters(bool three)
+PDModel::deriveCPParameters(int model)
 {
+    // bit of a hack, but the model deriving code is shared
+    // basically because it does pretty much the same thing
+    // for all the models and I didn't want to abstract it 
+    // any further, so we pass the subclass as a model number
+    // to control which intervals and formula to use
+    // where model is
+    // 0 = CP 2 parameter
+    // 1 = CP 3 parameter
+    // 2 = Extended Model  (Damien)
+    // 3 = Veloclinic (Mike P)
+    // 4 = Ward Smith
+
     // bounds on anaerobic interval in minutes
     const double t1 = anI1;
     const double t2 = anI2;
@@ -107,7 +119,7 @@ PDModel::deriveCPParameters(bool three)
     const double t3 = aeI1;
     const double t4 = aeI2;
 
-    // bounds of these time valus in the data
+    // bounds of these time values in the data
     int i1, i2, i3, i4;
 
     // find the indexes associated with the bounds
@@ -183,7 +195,8 @@ PDModel::deriveCPParameters(bool three)
         }
 
         // estimate t0 - but only for veloclinic/3parm cp
-        if (three) t0 = tau / (data[1] / cp - 1) - 1 / 60.0;
+        // where model is non-zero (CP2 is nonzero)
+        if (model) t0 = tau / (data[1] / cp - 1) - 1 / 60.0;
 
     } while ((fabs(tau - tau_prev) > tau_delta_max) || (fabs(t0 - t0_prev) > t0_delta_max));
 }
@@ -209,6 +222,9 @@ CP2Model::CP2Model(Context *context) :
 double 
 CP2Model::y(double t) const
 {
+    // don't start at zero !
+    t += (!minutes?1.00f:1/60.00f);
+
     // adjust to seconds
     if (minutes) t *= 60.00f;
 
@@ -235,15 +251,15 @@ CP2Model::CP()
 void CP2Model::onDataChanged() 
 { 
     // calc tau etc and make sure the interval is
-    // set corretly - i.e. 'domain of validity'
-    deriveCPParameters(); 
+    // set correctly - i.e. 'domain of validity'
+    deriveCPParameters(0); 
     setInterval(QwtInterval(tau, PDMODEL_MAXT));
 
 }
 
 void CP2Model::onIntervalsChanged() 
 { 
-    deriveCPParameters(); 
+    deriveCPParameters(0); 
     setInterval(QwtInterval(tau, PDMODEL_MAXT));
 }
 
@@ -267,6 +283,9 @@ CP3Model::CP3Model(Context *context) :
 double 
 CP3Model::y(double t) const
 {
+    // don't start at zero !
+    t += (!minutes?1.00f:1/60.00f);
+
     // adjust to seconds
     if (minutes) t *= 60.00f;
 
@@ -303,15 +322,101 @@ CP3Model::PMax()
 void CP3Model::onDataChanged() 
 { 
     // calc tau etc and make sure the interval is
-    // set corretly - i.e. 'domain of validity'
-    deriveCPParameters(true); 
+    // set correctly - i.e. 'domain of validity'
+    deriveCPParameters(1); 
     setInterval(QwtInterval(tau, PDMODEL_MAXT));
 
 }
 
 void CP3Model::onIntervalsChanged() 
 { 
-    deriveCPParameters(true); 
+    deriveCPParameters(1); 
+    setInterval(QwtInterval(tau, PDMODEL_MAXT));
+}
+
+//
+// Ward Smith Model
+//
+WSModel::WSModel(Context *context) : PDModel(context)
+{
+    // set default intervals to search CP 30-60
+    anI1=1800;
+    anI2=2400;
+    aeI1=2400;
+    aeI2=3600;
+
+    connect (this, SIGNAL(dataChanged()), this, SLOT(onDataChanged()));
+    connect (this, SIGNAL(intervalsChanged()), this, SLOT(onIntervalsChanged()));
+}
+
+// P(t) - return y for t in 2 parameter model
+double 
+WSModel::y(double t) const
+{
+    // don't start at zero !
+    t += (!minutes?1.00f:1/60.00f);
+
+    // adjust to seconds
+    if (minutes) t *= 60.00f;
+
+    //WPrime and PMax
+    double WPrime = cp * tau * 60;
+    double PMax = cp * (double(1.00f)+tau /(((double(1)/double(60))+t0)));
+    double wPMax = PMax-cp;
+
+    // WS Model P(t) = W'/t * (1- exp(-t/( W'/(wPmax)))) + CP
+    return ((WPrime / (double(t)))  * (1- exp(-t/(WPrime/wPMax)))) + cp;
+}
+
+// 2 parameter model can calculate these
+double 
+WSModel::WPrime()
+{
+    // kjoules
+    return (cp * tau * 60);
+}
+
+double
+WSModel::CP()
+{
+    return cp;
+}
+
+double
+WSModel::FTP()
+{
+    return y(45 * 60);
+}
+
+double
+WSModel::PMax()
+{
+    // casting to double across to ensure we don't lose precision
+    // but basically its the max value of the curve at time t of 1s
+    // which is cp * 1 + tau / ((t/60) + t0)
+    double WPrime = cp * tau * 60;
+    double PMax = cp * (double(1.00f)+tau /(((double(1)/double(60))+t0)));
+    double wPMax = PMax-cp;
+
+    // WS Model P(t) = W'/t * (1- exp(-t/( W'/(wPmax)))) + CP
+    return ((WPrime / (double(1)))  * (1- exp(-1/(WPrime/wPMax)))) + cp;
+}
+
+
+// could have just connected signal to slot
+// but might want to be more sophisticated in future
+void WSModel::onDataChanged() 
+{ 
+    // calc tau etc and make sure the interval is
+    // set correctly - i.e. 'domain of validity'
+    deriveCPParameters(4); 
+    setInterval(QwtInterval(tau, PDMODEL_MAXT));
+
+}
+
+void WSModel::onIntervalsChanged() 
+{ 
+    deriveCPParameters(4); 
     setInterval(QwtInterval(tau, PDMODEL_MAXT));
 }
 
@@ -373,6 +478,9 @@ MultiModel::setVariant(int variant)
 double 
 MultiModel::y(double t) const
 {
+    // don't start at zero !
+    t += (!minutes?1.00f:1/60.00f);
+
     // adjust to seconds
     if (minutes) t *= 60.00f;
 
@@ -440,10 +548,10 @@ MultiModel::PMax()
 void MultiModel::onDataChanged() 
 { 
     // calc tau etc and make sure the interval is
-    // set corretly - i.e. 'domain of validity'
-    deriveCPParameters(true); 
+    // set correctly - i.e. 'domain of validity'
+    deriveCPParameters(3); 
 
-    // and veloclinic paramters too;
+    // and veloclinic parameters too;
     w1 = cp*tau*60; // initial estimate from classic cp model
     p1 = PMax() - cp;
     p2 = cp;
@@ -462,9 +570,9 @@ void MultiModel::onDataChanged()
 
 void MultiModel::onIntervalsChanged() 
 { 
-    deriveCPParameters(true); 
+    deriveCPParameters(3); 
 
-    // and veloclinic paramters too;
+    // and veloclinic parameters too;
     w1 = cp*tau*60; // initial estimate from classic model
     p1 = PMax() - cp;
     p2 = cp;
@@ -506,6 +614,10 @@ ExtendedModel::ExtendedModel(Context *context) :
 double
 ExtendedModel::y(double t) const
 {
+    // don't start at zero !
+    if (t == 0)
+        qDebug() << "ExtendedModel t=0 !!";
+
     if (!minutes) t /= 60.00f;
     return paa*(1.20-0.20*exp(-1*double(t)))*exp(paa_dec*(double(t))) + ecp * (1-exp(tau_del*double(t))) * (1-exp(ecp_del*double(t))) * (1+ecp_dec*exp(ecp_dec_del/double(t))) * ( 1 + etau/(double(t)));
 }
@@ -549,7 +661,7 @@ void
 ExtendedModel::onDataChanged()
 {
     // calc tau etc and make sure the interval is
-    // set corretly - i.e. 'domain of validity'
+    // set correctly - i.e. 'domain of validity'
     deriveExtCPParameters();
     setInterval(QwtInterval(etau, PDMODEL_MAXT));
 
@@ -581,7 +693,7 @@ ExtendedModel::deriveExtCPParameters()
     const double t7 = laeI1;
     const double t8 = laeI2;
 
-    // bounds of these time valus in the data
+    // bounds of these time values in the data
     int i1, i2, i3, i4, i5, i6, i7, i8;
 
     // find the indexes associated with the bounds
@@ -849,3 +961,16 @@ void CP3Model::saveParameters(QList<double>&here)
     here << tau;
     here << t0;
 }
+void WSModel::loadParameters(QList<double>&here)
+{
+    cp = here[0];
+    tau = here[1];
+    t0 = here[2];
+}
+void WSModel::saveParameters(QList<double>&here)
+{
+    here.clear();
+    here << cp;
+    here << tau;
+    here << t0;
+}
