@@ -14,8 +14,10 @@
 #include <QCheckBox>
 #include <QListWidget>
 #include <QList>
+#include <QFileDialog>
 #include "Zones.h"
 #include "HrZones.h"
+#include "PaceZones.h"
 #include <QLabel>
 #include <QDateEdit>
 #include <QCheckBox>
@@ -29,12 +31,7 @@
 #include "DataProcessor.h"
 #include "Season.h"
 #include "SeasonParser.h"
-
-#ifdef GC_HAVE_LIBOAUTH
-extern "C" {
-#include <oauth.h>
-}
-#endif
+#include "RideAutoImportConfig.h"
 
 class QGroupBox;
 class QHBoxLayout;
@@ -43,11 +40,11 @@ class ColorsPage;
 class IntervalMetricsPage;
 class ZonePage;
 class HrZonePage;
+class PaceZonePage;
 class SummaryMetricsPage;
 class MetadataPage;
 class KeywordsPage;
 class FieldsPage;
-class MeasuresPage;
 class Colors;
 class RiderPage;
 class SeasonsPage;
@@ -59,7 +56,7 @@ class GeneralPage : public QWidget
 
     public:
         GeneralPage(Context *context);
-        void saveClicked();
+        qint32 saveClicked();
 
         QString athleteWAS; // remember what we started with !
 
@@ -71,11 +68,13 @@ class GeneralPage : public QWidget
         Context *context;
 
         QComboBox *langCombo;
-        QComboBox *unitCombo;
         QComboBox *crankLengthCombo;
-        QComboBox *wheelSizeCombo;
+        QComboBox *rimSizeCombo;
+        QComboBox *tireSizeCombo;
         QComboBox *wbalForm;
         QCheckBox *garminSmartRecord;
+        QCheckBox *warnOnExit;
+        QLineEdit *wheelSizeEdit;
         QLineEdit *garminHWMarkedit;
         QLineEdit *hystedit;
         QLineEdit *athleteDirectory;
@@ -84,7 +83,6 @@ class GeneralPage : public QWidget
         QPushButton *athleteBrowseButton;
 
         QLabel *langLabel;
-        QLabel *unitLabel;
         QLabel *warningLabel;
         QLabel *workoutLabel;
         QLabel *athleteLabel;
@@ -96,6 +94,19 @@ class GeneralPage : public QWidget
         QCheckBox *showSBToday;
         QIntValidator *perfManSTSavgValidator;
         QIntValidator *perfManLTSavgValidator;
+
+        struct {
+            int wheel;
+            int crank;
+            float hyst;
+            int wbal;
+            int lts,sts;
+            bool warn;
+        } b4;
+
+    private slots:
+        void calcWheelSize();
+        void resetWheelSize();
 };
 
 class RiderPage : public QWidget
@@ -106,7 +117,7 @@ class RiderPage : public QWidget
 
     public:
         RiderPage(QWidget *parent, Context *context);
-        void saveClicked();
+        qint32 saveClicked();
 
     public slots:
         void chooseAvatar();
@@ -119,11 +130,22 @@ class RiderPage : public QWidget
         QDateEdit *dob;
         QComboBox *sex;
         QLabel *weightlabel;
+        QLabel *heightlabel;
+        QLabel *wbaltaulabel;
         QComboBox *unitCombo;
         QDoubleSpinBox *weight;
+        QDoubleSpinBox *height;
+        QSpinBox *wbaltau;
         QTextEdit  *bio;
         QPushButton *avatarButton;
         QPixmap     avatar;
+
+    struct {
+        int unit;
+        double weight;
+        double height;
+    } b4;
+
 };
 
 class CredentialsPage : public QScrollArea
@@ -134,12 +156,16 @@ class CredentialsPage : public QScrollArea
 
     public:
         CredentialsPage(QWidget *parent, Context *context);
-        void saveClicked();
+        qint32 saveClicked();
 
     public slots:
+#ifdef GC_HAVE_KQOAUTH
         void authoriseTwitter();
+#endif
         void authoriseStrava();
         void authoriseCyclingAnalytics();
+        void authoriseGoogleCalendar();
+        void dvCALDAVTypeChanged(int);
 
     private:
         Context *context;
@@ -150,20 +176,15 @@ class CredentialsPage : public QScrollArea
         QComboBox *tpType;
         QPushButton *tpTest;
 
-        QLineEdit *gcURL; // url for gc racing (not available yet)
-        QLineEdit *gcUser;
-        QLineEdit *gcPass;
-
+#ifdef GC_HAVE_KQOAUTH
         QLineEdit *twitterURL; // url for twitter.com
         QPushButton *twitterAuthorise;
-        QLineEdit *twitterPIN;
-        char *t_key, *t_secret;
+#endif
 
+        QComboBox *dvCALDAVType;
         QPushButton *stravaAuthorise, *stravaAuthorised, *twitterAuthorised;
-        char *s_id, *s_secret;
-
         QPushButton *cyclingAnalyticsAuthorise, *cyclingAnalyticsAuthorised;
-
+        QPushButton *googleCalendarAuthorise, *googleCalendarAuthorised;
 
         QLineEdit *rideWithGPSUser;
         QLineEdit *rideWithGPSPass;
@@ -171,6 +192,9 @@ class CredentialsPage : public QScrollArea
         QLineEdit *ttbUser;
         QLineEdit *ttbPass;
 
+        QLineEdit *veloHeroUser;
+        QLineEdit *veloHeroPass;
+
         QLineEdit *selUser;
         QLineEdit *selPass;
 
@@ -178,15 +202,12 @@ class CredentialsPage : public QScrollArea
         QLineEdit *wiUser;
         QLineEdit *wiPass;
 
-        QLineEdit *zeoURL; // url for myzeo
-        QLineEdit *zeoUser;
-        QLineEdit *zeoPass;
-
         QLineEdit *webcalURL; // url for webcal calendar (read only, TP.com, Google Calendar)
 
-        QLineEdit *dvURL; // url for calDAV calendar (read/write, e.g. Google, Hotmail)
+        QLineEdit *dvURL; // url for calDAV calendar (read/write, e.g. Hotmail)
         QLineEdit *dvUser;
         QLineEdit *dvPass;
+        QLineEdit *dvGoogleCalid;
 };
 
 class deviceModel : public QAbstractTableModel
@@ -216,8 +237,12 @@ class deviceModel : public QAbstractTableModel
         bool insertRows(int position, int rows, const QModelIndex &index=QModelIndex());
         bool removeRows(int position, int rows, const QModelIndex &index=QModelIndex());
 
-
         QList<DeviceConfiguration> Configuration;  // the actual data
+
+    public slots:
+        // config changed by wizard so reset
+        void doReset();
+
  };
 
 class DevicePage : public QWidget
@@ -227,7 +252,7 @@ class DevicePage : public QWidget
 
     public:
         DevicePage(QWidget *, Context *);
-        void saveClicked();
+        qint32 saveClicked();
 
         QTableView *deviceList;
 
@@ -272,7 +297,7 @@ class BestsMetricsPage : public QWidget
         void rightClicked();
         void availChanged();
         void selectedChanged();
-        void saveClicked();
+        qint32 saveClicked();
 
     protected:
 
@@ -310,7 +335,7 @@ class IntervalMetricsPage : public QWidget
         void rightClicked();
         void availChanged();
         void selectedChanged();
-        void saveClicked();
+        qint32 saveClicked();
 
     protected:
 
@@ -346,7 +371,7 @@ class SummaryMetricsPage : public QWidget
         void rightClicked();
         void availChanged();
         void selectedChanged();
-        void saveClicked();
+        qint32 saveClicked();
 
     protected:
 
@@ -412,7 +437,7 @@ class ColorsPage : public QWidget
 
     public:
         ColorsPage(QWidget *parent);
-        void saveClicked();
+        qint32 saveClicked();
 
     public slots:
         void applyThemeClicked();
@@ -452,6 +477,13 @@ class ColorsPage : public QWidget
         QTreeWidget *themes;
         const Colors *colorSet;
         QPushButton *applyTheme;
+
+        struct {
+            bool alias, scroll, head;
+            double line;
+            int chrome;
+            unsigned long fingerprint;
+        } b4;
 };
 
 class FieldsPage : public QWidget
@@ -493,7 +525,7 @@ class ProcessorPage : public QWidget
     public:
 
         ProcessorPage(Context *context);
-        void saveClicked();
+        qint32 saveClicked();
 
     public slots:
 
@@ -510,6 +542,36 @@ class ProcessorPage : public QWidget
 
 };
 
+class DefaultsPage : public QWidget
+{
+    Q_OBJECT
+    G_OBJECT
+
+
+    public:
+
+        DefaultsPage(QWidget *parent, QList<DefaultDefinition>);
+        void getDefinitions(QList<DefaultDefinition>&);
+
+    public slots:
+        void addClicked();
+        void upClicked();
+        void downClicked();
+        void deleteClicked();
+
+    protected:
+
+        QTreeWidget *defaults;
+
+#ifndef Q_OS_MAC
+        QToolButton *upButton, *downButton;
+#else
+        QPushButton *upButton, *downButton;
+#endif
+        QPushButton *addButton, *deleteButton;
+
+};
+
 class MetadataPage : public QWidget
 {
     Q_OBJECT
@@ -520,7 +582,7 @@ class MetadataPage : public QWidget
     public:
 
         MetadataPage(Context *);
-        void saveClicked();
+        qint32 saveClicked();
 
     public slots:
 
@@ -534,13 +596,25 @@ class MetadataPage : public QWidget
         KeywordsPage *keywordsPage;
         FieldsPage *fieldsPage;
         ProcessorPage *processorPage;
+        DefaultsPage *defaultsPage;
 
         // local versions for modification
         QList<KeywordDefinition> keywordDefinitions;
         QList<FieldDefinition>   fieldDefinitions;
+        QList<DefaultDefinition>  defaultDefinitions;
         QString colorfield;
+
+        // initial values
+        struct {
+            unsigned long fieldFingerprint;
+            unsigned long keywordFingerprint;
+            QString colorfield;
+        } b4;
 };
 
+//
+// Power Zones
+//
 class SchemePage : public QWidget
 {
     Q_OBJECT
@@ -550,7 +624,7 @@ class SchemePage : public QWidget
     public:
         SchemePage(ZonePage *parent);
         ZoneScheme getScheme();
-        void saveClicked();
+        qint32 saveClicked();
 
     public slots:
         void addClicked();
@@ -574,8 +648,10 @@ class CPPage : public QWidget
 
     public slots:
         void addClicked();
+        void editClicked();
         void deleteClicked();
         void defaultClicked();
+        void rangeEdited();
         void rangeSelectionChanged();
         void addZoneClicked();
         void deleteZoneClicked();
@@ -587,12 +663,13 @@ class CPPage : public QWidget
         QDateEdit *dateEdit;
         QDoubleSpinBox *cpEdit;
         QDoubleSpinBox *wEdit;
+        QDoubleSpinBox *pmaxEdit;
 
         ZonePage *zonePage;
         QTreeWidget *ranges;
         QTreeWidget *zones;
-        QPushButton *addButton, *deleteButton, *defaultButton;
-        QPushButton *addZoneButton, *deleteZoneButton;
+        QPushButton *addButton, *updateButton, *deleteButton;
+        QPushButton *addZoneButton, *deleteZoneButton, *defaultButton;
 };
 
 class ZonePage : public QWidget
@@ -604,10 +681,11 @@ class ZonePage : public QWidget
     public:
 
         ZonePage(Context *);
-        void saveClicked();
+        qint32 saveClicked();
 
         //ZoneScheme scheme;
         Zones zones;
+        quint16 b4Fingerprint; // how did it start ?
 
         // Children talk to each other
         SchemePage *schemePage;
@@ -626,6 +704,9 @@ class ZonePage : public QWidget
         // local versions for modification
 };
 
+//
+// Heartrate Zones
+//
 class HrSchemePage : public QWidget
 {
     Q_OBJECT
@@ -635,7 +716,7 @@ class HrSchemePage : public QWidget
 public:
     HrSchemePage(HrZonePage *parent);
     HrZoneScheme getScheme();
-    void saveClicked();
+    qint32 saveClicked();
 
     public slots:
     void addClicked();
@@ -660,8 +741,10 @@ public:
 
     public slots:
     void addClicked();
+    void editClicked();
     void deleteClicked();
     void defaultClicked();
+    void rangeEdited();
     void rangeSelectionChanged();
     void addZoneClicked();
     void deleteZoneClicked();
@@ -678,8 +761,8 @@ private:
     HrZonePage  *zonePage;
     QTreeWidget *ranges;
     QTreeWidget *zones;
-    QPushButton *addButton, *deleteButton, *defaultButton;
-    QPushButton *addZoneButton, *deleteZoneButton;
+    QPushButton *addButton, *updateButton, *deleteButton;
+    QPushButton *addZoneButton, *deleteZoneButton, *defaultButton;
 };
 
 class HrZonePage : public QWidget
@@ -691,10 +774,11 @@ class HrZonePage : public QWidget
 public:
 
     HrZonePage(Context *);
-    void saveClicked();
+    qint32 saveClicked();
 
     //ZoneScheme scheme;
     HrZones zones;
+    quint16 b4Fingerprint; // how did it start ?
 
     // Children talk to each other
     HrSchemePage *schemePage;
@@ -713,6 +797,107 @@ protected:
     // local versions for modification
 };
 
+//
+// Pace Zones
+//
+class PaceSchemePage : public QWidget
+{
+    Q_OBJECT
+    G_OBJECT
+
+
+    public:
+        PaceSchemePage(PaceZonePage *parent);
+        PaceZoneScheme getScheme();
+        qint32 saveClicked();
+
+    public slots:
+        void addClicked();
+        void deleteClicked();
+        void renameClicked();
+
+    private:
+        PaceZonePage *zonePage;
+        QTreeWidget *scheme;
+        QPushButton *addButton, *renameButton, *deleteButton;
+};
+
+class CVPage : public QWidget
+{
+    Q_OBJECT
+    G_OBJECT
+
+
+    public:
+        CVPage(PaceZonePage *parent);
+        QCheckBox *metric;
+
+    public slots:
+        void addClicked();
+        void deleteClicked();
+        void defaultClicked();
+        void rangeSelectionChanged();
+        void addZoneClicked();
+        void deleteZoneClicked();
+        void zonesChanged();
+
+        void metricChanged();
+
+    private:
+        bool active;
+
+        QDateEdit *dateEdit;
+        QTimeEdit *cvEdit;
+
+        PaceZonePage *zonePage;
+        QTreeWidget *ranges;
+        QTreeWidget *zones;
+        QPushButton *addButton, *deleteButton, *defaultButton;
+        QPushButton *addZoneButton, *deleteZoneButton;
+        QLabel *per;
+};
+
+class PaceZonePage : public QWidget
+{
+    Q_OBJECT
+    G_OBJECT
+
+
+    public:
+
+        PaceZonePage(Context *);
+        ~PaceZonePage() { delete zones; }
+        qint32 saveClicked();
+
+        PaceZones* zones;
+        quint16 b4Fingerprint; // how did it start ?
+
+        // Children talk to each other
+        PaceSchemePage *schemePage;
+        CVPage *cvPage;
+
+    public slots:
+
+
+    protected:
+
+        Context *context;
+        bool changed;
+
+        QTabWidget *tabs;
+
+    private:
+
+        QLabel *sportLabel;
+        QComboBox *sportCombo;
+
+    private slots:
+        void changeSport();
+
+};
+
+// Seasons
+
 class SeasonsPage : public QWidget
 {
     Q_OBJECT
@@ -730,7 +915,7 @@ class SeasonsPage : public QWidget
         void renameClicked();
         void deleteClicked();
         void clearEdit();
-        void saveClicked();
+        qint32 saveClicked();
 
     private:
 
@@ -750,34 +935,68 @@ class SeasonsPage : public QWidget
         QList<Season> array;
 };
 
-class MeasuresPage : public QWidget
+class AutoImportPage : public QWidget
 {
     Q_OBJECT
     G_OBJECT
 
 
     public:
-        MeasuresPage(Context *context);
-        void getDefinitions(QList<FieldDefinition>&);
+
+        AutoImportPage(Context *);
+        qint32 saveClicked();
+        void addRuleTypes(QComboBox *p);
 
     public slots:
+
         void addClicked();
         void upClicked();
         void downClicked();
-        void renameClicked();
         void deleteClicked();
-        void saveClicked();
+        void browseImportDir();
+
 
     private:
 
         Context *context;
+        QList<RideAutoImportRule> rules;
+
         QTreeWidget *fields;
+
 #ifndef Q_OS_MAC
         QToolButton *upButton, *downButton;
 #else
         QPushButton *upButton, *downButton;
 #endif
-        QPushButton *addButton, *renameButton, *deleteButton;
+        QPushButton *addButton, *renameButton, *deleteButton, *browseButton;
+
 };
 
+class IntervalsPage : public QWidget
+{
+    Q_OBJECT
+    G_OBJECT
+
+    public:
+        IntervalsPage(Context *context);
+        qint32 saveClicked();
+
+        unsigned int discoveryWAS;
+
+    public slots:
+
+    private:
+        Context *context;
+        int user; //index of user interval type
+
+        QList <QCheckBox*> checkBoxes;
+
+        struct {
+            int discovery;
+        } b4;
+
+    private slots:
+};
+
+
 #endif
