@@ -27,7 +27,7 @@
 #include "Zones.h"
 #include "Colors.h"
 
-#include <math.h>
+#include <cmath>
 #include <qwt_series_data.h>
 #include <qwt_legend.h>
 #include <qwt_plot_canvas.h>
@@ -78,10 +78,10 @@ public:
             if (zone_range == -1)
                 zone_range = zones->whichRange(QDate::currentDate());
 
-        } else if (rideItem) {
+        } else if (rideItem && parent->context->athlete->zones()) {
 
-            zones = rideItem->zones;
-            zone_range = rideItem->zoneRange();
+            zones = parent->context->athlete->zones();
+            zone_range = zones->whichRange(rideItem->dateTime.date());
 
         } else {
 
@@ -200,18 +200,19 @@ PfPvPlot::PfPvPlot(Context *context)
 
     merge_intervals = false;
     frame_intervals = true;
+    gear_ratio_display = false;
 
     // only default on first time through, after this the user may have adjusted
     if (appsettings->value(this, GC_SHADEZONES, true).toBool()==false) shade_zones = false;
     else shade_zones = true;
 
-    configChanged();
+    configChanged(CONFIG_APPEARANCE);
 
     recalc();
 }
 
 void
-PfPvPlot::configChanged()
+PfPvPlot::configChanged(qint32)
 {
     setCanvasBackground(GColor(CPLOTBACKGROUND));
 
@@ -289,14 +290,12 @@ PfPvPlot::refreshZoneItems()
 
     // set zones from ride or athlete and date of
     // first item in the compare set
-    const Zones *zones;
+    const Zones *zones = context->athlete->zones();
     int zone_range = -1;
 
     // comparing does zones for items selected not current ride
     if (context->isCompareIntervals) {
 
-        zones = context->athlete->zones();
-
         // no athlete zones anyway!
         if (!zones) return;
 
@@ -310,10 +309,9 @@ PfPvPlot::refreshZoneItems()
             zone_range = zones->whichRange(QDate::currentDate());
         }
 
-    } else if (rideItem) {
+    } else if (rideItem && zones) {
 
-        zones = rideItem->zones;
-        zone_range = rideItem->zoneRange();
+        zone_range = zones->whichRange(rideItem->dateTime.date());
 
     } else {
 
@@ -390,19 +388,7 @@ PfPvPlot::refreshZoneItems()
 // how many intervals selected?
 int PfPvPlot::intervalCount() const
 {
-    int highlighted;
-    highlighted = 0;
-    if (context->athlete->allIntervalItems() == NULL) return 0; // not inited yet!
-
-    for (int i=0; i<context->athlete->allIntervalItems()->childCount(); i++) {
-        IntervalItem *current = dynamic_cast<IntervalItem *>(context->athlete->allIntervalItems()->child(i));
-        if (current != NULL) {
-            if (current->isSelected() == true) {
-                ++highlighted;
-            }
-        }
-    }
-    return highlighted;
+    return rideItem->intervalsSelected().count();
 }
 
 void 
@@ -413,7 +399,7 @@ PfPvPlot::mouseTrack(double cad, double watts)
     double aepf = (watts * 60.0) / (cad * cl_ * 2.0 * PI);
     double cpv = (cad * cl_ * 2.0 * PI) / 60.0;
 
-    if (rideItem && rideItem->ride() && rideItem->ride()->intervals().count() >= intervalMarkers.count()) {
+    if (rideItem && rideItem->ride() && rideItem->intervals().count() >= intervalMarkers.count()) {
         // are we hovering "close" to an interval marker ?
         int index = 0;
         foreach (QwtPlotMarker *is, intervalMarkers) {
@@ -422,7 +408,7 @@ PfPvPlot::mouseTrack(double cad, double watts)
             double y = is->yValue() - aepf;
 
             if ((x > -0.05f && x < 0.05f) && (y > -3 && y < 3)) {
-                context->notifyIntervalHover(rideItem->ride()->intervals()[index]);
+                context->notifyIntervalHover(rideItem->intervals()[index]);
             }
     
             index++;
@@ -445,7 +431,7 @@ PfPvPlot::refreshIntervalMarkers()
 
     // do we have a ride with intervals to refresh ?
     int count=0;
-    if (rideItem && rideItem->ride() && rideItem->ride()->dataPoints().count() && (count = rideItem->ride()->intervals().count())) {
+    if (rideItem && rideItem->ride() && rideItem->ride()->dataPoints().count() && (count = rideItem->intervals().count())) {
 
         // accumulating...
         QVector<accum> intervalAccumulator(count);
@@ -461,10 +447,10 @@ PfPvPlot::refreshIntervalMarkers()
                 // accumulate values for each interval here ....
                 for(int i=0; i < count; i++) {
 
-                    RideFileInterval v = rideItem->ride()->intervals()[i];
+                    IntervalItem *v = rideItem->intervals()[i];
 
                     // in our interval ?
-                    if (p1->secs >= v.start && p1->secs <= v.stop) {
+                    if (p1->secs >= v->start && p1->secs <= v->stop) {
                         intervalAccumulator[i].aepf += aepf;
                         intervalAccumulator[i].cpv += cpv;
                         intervalAccumulator[i].count++;
@@ -529,7 +515,7 @@ PfPvPlot::setData(RideItem *_rideItem)
     if (ride) {
 
         // quickly erase old data
-        curve->setVisible(false);
+        mainCurvesSetVisible(false);
 
 
         // due to the discrete power and cadence values returned by the
@@ -537,11 +523,13 @@ PfPvPlot::setData(RideItem *_rideItem)
         // Rather than pass them all to the curve, use a set to strip
         // out duplicates.
         std::set<std::pair<double, double> > dataSet;
-        std::set<std::pair<double, double> > dataSetSelected;
 
+        const int num_gearRatio_Plots = 4;
+        QVector<std::set<std::pair<double, double> > > gearSet(num_gearRatio_Plots);
         long tot_cad = 0;
         long tot_cad_points = 0;
 
+
         foreach(const RideFilePoint *p1, ride->dataPoints()) {
 
             if (p1->watts != 0 && p1->cad != 0) {
@@ -553,7 +541,20 @@ PfPvPlot::setData(RideItem *_rideItem)
                     dataSet.insert(std::make_pair<double, double>(aepf, cpv));
                     tot_cad += p1->cad;
                     tot_cad_points++;
-                }
+
+                    // fill the dataSets for gearRatio display
+                    if (p1->gear != 0) {
+                        if (p1->gear > 0.0f && p1->gear <= 1.00f) {
+                            gearSet[0].insert(std::make_pair<double, double>(aepf, cpv));
+                        } else if (p1->gear > 1.00f && p1->gear <= 2.50f ) {
+                            gearSet[1].insert(std::make_pair<double, double>(aepf, cpv));
+                        } else if (p1->gear > 2.50f && p1->gear <= 4.00 ) {
+                            gearSet[2].insert(std::make_pair<double, double>(aepf, cpv));
+                        } else {  // > 4.0
+                            gearSet[3].insert(std::make_pair<double, double>(aepf, cpv));
+                        }
+                    }
+                 }
             }
         }
 
@@ -562,7 +563,7 @@ PfPvPlot::setData(RideItem *_rideItem)
         if (tot_cad_points == 0) {
             //setTitle(tr("no cadence"));
             refreshZoneItems();
-            curve->setVisible(false);
+            mainCurvesSetVisible(false);
 
         } else {
             // Now that we have the set of points, transform them into the
@@ -597,21 +598,75 @@ PfPvPlot::setData(RideItem *_rideItem)
             // averages for intervals
             refreshIntervalMarkers();
 
-            curve->setSymbol(sym);
-            curve->setVisible(true);
+            // now the curves for gearRatio
+            if (gearRatioCurves.size()) {
+               QListIterator<QwtPlotCurve *> g(gearRatioCurves);
+               while (g.hasNext()) {
+                   QwtPlotCurve *curve = g.next();
+                   curve->detach();
+                   delete curve;
+               }
+            }
+            gearRatioCurves.clear();
+
+            // get the detailed data
+            QVector<QwtArray<double> > aepfArrayGearRatio(num_gearRatio_Plots);
+            QVector<QwtArray<double> > cpvArrayGearRatio(num_gearRatio_Plots);
+
+            for (int i=0;i<num_gearRatio_Plots;i++) {
+                std::set<std::pair<double, double> >::const_iterator m(gearSet[i].begin());
+                while (m != gearSet[i].end()) {
+                    const std::pair<double, double>& dataPoint = *m;
+
+                    aepfArrayGearRatio[i].push_back(dataPoint.first);
+                    cpvArrayGearRatio[i].push_back(dataPoint.second);
+
+                    ++m;
+                }
+            }
+
+            // create the plots
+            for (int i=0;i<num_gearRatio_Plots;i++) {
+                QwtPlotCurve *gearCurve;
+                gearCurve = new QwtPlotCurve();
+                QwtSymbol *sym = new QwtSymbol;
+                sym->setStyle(QwtSymbol::Diamond);
+                sym->setSize(3);
+                sym->setBrush(QBrush(Qt::NoBrush));
+                sym->setPen(QPen(Qt::red));
+                switch (i) {
+                  case 0: sym->setPen(QPen(Qt::red));  sym->setBrush(QBrush(Qt::red));  break;
+                  case 1: sym->setPen(QPen(Qt::yellow)); sym->setBrush(QBrush(Qt::yellow));break;
+                  case 2: sym->setPen(QPen(Qt::green));  sym->setBrush(QBrush(Qt::green));break;
+                  case 3: sym->setPen(QPen(Qt::blue));  sym->setBrush(QBrush(Qt::blue)); break;
+                    // default means something is wrong, but no err
+                  default: sym->setPen(QPen(Qt::white)); sym->setBrush(QBrush(Qt::white));
+                }
+                gearCurve->setSymbol(sym);
+                gearCurve->setStyle(QwtPlotCurve::Dots);
+                gearCurve->setRenderHint(QwtPlotItem::RenderAntialiased);
+                gearCurve->setSamples(cpvArrayGearRatio[i], aepfArrayGearRatio[i]);
+                gearCurve->attach(this);
+
+                gearRatioCurves.append(gearCurve);
+
+            }
+
         }
+        mainCurvesSetVisible(true);
+
     } else {
 
         //setTitle("no data");
         refreshZoneItems();
-        curve->setVisible(false);
+        mainCurvesSetVisible(false);
     }
 
     replot();
 }
 
 void
-PfPvPlot::intervalHover(RideFileInterval x)
+PfPvPlot::intervalHover(IntervalItem *x)
 {
     if (!isVisible()) return;
     if (context->isCompareIntervals) return;
@@ -625,46 +680,50 @@ PfPvPlot::intervalHover(RideFileInterval x)
         hover = NULL;
     }
 
-    // collect the data
-    QVector<double> aepfArray, cpvArray;
-    foreach(const RideFilePoint *p1, rideItem->ride()->dataPoints()) {
+    // if hovering AND no intervals are selected then lets do it
+    if (x && rideItem->intervalsSelected().count() == 0) {
 
-        if (p1->secs < x.start || p1->secs > x.stop) continue;
+        // collect the data
+        QVector<double> aepfArray, cpvArray;
+        foreach(const RideFilePoint *p1, rideItem->ride()->dataPoints()) {
 
-        if (p1->watts != 0 && p1->cad != 0) {
-            double aepf = (p1->watts * 60.0) / (p1->cad * cl_ * 2.0 * PI);
-            double cpv = (p1->cad * cl_ * 2.0 * PI) / 60.0;
+            if (p1->secs < x->start || p1->secs > x->stop) continue;
 
-            aepfArray << aepf;
-            cpvArray << cpv;
+            if (p1->watts != 0 && p1->cad != 0) {
+                double aepf = (p1->watts * 60.0) / (p1->cad * cl_ * 2.0 * PI);
+                double cpv = (p1->cad * cl_ * 2.0 * PI) / 60.0;
+
+                aepfArray << aepf;
+                cpvArray << cpv;
+            }
         }
-    }
 
-    // which interval is it or how many ?
-    int count = 0;
-    int ours = 0;
-    foreach(RideFileInterval p, rideItem->ride()->intervals()) {
-        if (p.start == x.start && p.stop == x.stop) ours = count;
-        count++;
-    }
+        // which interval is it or how many ?
+        int count = 0;
+        int ours = 0;
+        foreach(IntervalItem *p, rideItem->intervals()) {
+            if (p->start == x->start && p->stop == x->stop) ours = count;
+            count++;
+        }
 
-    // any data ?
-    if (aepfArray.size()) {
-        QwtSymbol *sym = new QwtSymbol;
-        sym->setStyle(QwtSymbol::Ellipse);
-        sym->setSize(4);
-        QColor color;
-        color.setHsv(ours * 255/count, 255,255);
-        color.setAlpha(128);
-        sym->setPen(QPen(color));
-        sym->setBrush(QBrush(color));
+        // any data ?
+        if (aepfArray.size()) {
+            QwtSymbol *sym = new QwtSymbol;
+            sym->setStyle(QwtSymbol::Ellipse);
+            sym->setSize(4);
+            QColor color;
+            color.setHsv(ours * 255/count, 255,255);
+            color.setAlpha(128);
+            sym->setPen(QPen(color));
+            sym->setBrush(QBrush(color));
 
-        hover = new QwtPlotCurve();
-        hover->setSymbol(sym);
-        hover->setStyle(QwtPlotCurve::Dots);
-        hover->setRenderHint(QwtPlotItem::RenderAntialiased);
-        hover->setSamples(cpvArray, aepfArray);
-        hover->attach(this);
+            hover = new QwtPlotCurve();
+            hover->setSymbol(sym);
+            hover->setStyle(QwtPlotCurve::Dots);
+            hover->setRenderHint(QwtPlotItem::RenderAntialiased);
+            hover->setSamples(cpvArray, aepfArray);
+            hover->attach(this);
+        }
     }
 
     replot(); // refresh
@@ -695,8 +754,8 @@ PfPvPlot::showIntervals(RideItem *_rideItem)
        int num_intervals=intervalCount();
 
        if (mergeIntervals()) num_intervals = 1;
-       if (frameIntervals() || num_intervals==0) curve->setVisible(true);
-       if (frameIntervals()==false && num_intervals) curve->setVisible(false);
+       if (frameIntervals() || num_intervals==0) mainCurvesSetVisible(true);
+       if (frameIntervals()==false && num_intervals) mainCurvesSetVisible(false);
        QVector<std::set<std::pair<double, double> > > dataSetInterval(num_intervals);
 
        long tot_cad = 0;
@@ -708,11 +767,11 @@ PfPvPlot::showIntervals(RideItem *_rideItem)
                 double aepf = (p1->watts * 60.0) / (p1->cad * cl_ * 2.0 * PI);
                 double cpv = (p1->cad * cl_ * 2.0 * PI) / 60.0;
 
-                for (int high=-1, t=0; t<context->athlete->allIntervalItems()->childCount(); t++) {
+                for (int high=-1, t=0; t<rideItem->intervals().count(); t++) {
 
-                    IntervalItem *current = dynamic_cast<IntervalItem *>(context->athlete->allIntervalItems()->child(t));
+                    IntervalItem *current = dynamic_cast<IntervalItem *>(rideItem->intervals().at(t));
 
-                    if ((current != NULL) && current->isSelected()) {
+                    if ((current != NULL) && current->selected) {
                         ++high;
                         if (p1->secs+ride->recIntSecs() > current->start && p1->secs< current->stop) {
                             if (mergeIntervals())
@@ -751,15 +810,13 @@ PfPvPlot::showIntervals(RideItem *_rideItem)
            // ensure same colors are used for each interval selected
            int num_intervals_defined=0;
            QVector<int> intervalmap;
-           if (context->athlete->allIntervalItems() != NULL) {
 
-                num_intervals_defined = context->athlete->allIntervalItems()->childCount();
+            num_intervals_defined = rideItem->intervals().count();
 
-                for (int g=0; g<context->athlete->allIntervalItems()->childCount(); g++) {
-                    IntervalItem *curr = dynamic_cast<IntervalItem *>(context->athlete->allIntervalItems()->child(g));
-                    if (curr->isSelected()) intervalmap.append(g);
-                }
-           }
+            for (int g=0; g<rideItem->intervals().count(); g++) {
+                IntervalItem *curr = dynamic_cast<IntervalItem *>(rideItem->intervals().at(g));
+                if (curr->selected) intervalmap.append(g);
+            }
 
             // honor display sequencing
             QMap<int, int> intervalOrder;
@@ -767,12 +824,12 @@ PfPvPlot::showIntervals(RideItem *_rideItem)
 
             if (mergeIntervals()) intervalOrder.insert(1,0);
             else {
-                for (int i=0; i<context->athlete->allIntervalItems()->childCount(); i++) {
+                for (int i=0; i<rideItem->intervals().count(); i++) {
 
-                    IntervalItem *current = dynamic_cast<IntervalItem *>(context->athlete->allIntervalItems()->child(i));
+                    IntervalItem *current = dynamic_cast<IntervalItem *>(rideItem->intervals().at(i));
 
-                    if (current != NULL && current->isSelected() == true) {
-                            intervalOrder.insert(current->displaySequence, count++);
+                    if (current != NULL && current->selected == true) {
+                        intervalOrder.insert(current->displaySequence, count++);
                     }
                 }
             }
@@ -819,7 +876,7 @@ PfPvPlot::showIntervals(RideItem *_rideItem)
 void
 PfPvPlot::recalcCompare()
 {
-    // do all the calcuations for compare mode
+    // do all the calculations for compare mode
     // we do as a separate method to 'isolate' compare mode
     // updates to ensure we don't break all the charts !!
     maxAEPF = 600;
@@ -1164,6 +1221,15 @@ PfPvPlot::setFrameIntervals(bool value)
     showIntervals(rideItem);
 }
 
+void
+PfPvPlot::setGearRatioDisplay(bool value)
+{
+    gear_ratio_display = value;
+    if (context->isCompareIntervals) return;
+    mainCurvesSetVisible(true);
+    replot();
+}
+
 void
 PfPvPlot::showCompareIntervals()
 {
@@ -1179,7 +1245,7 @@ PfPvPlot::showCompareIntervals()
     intervalCurves.clear();
 
     // quickly erase old data
-    curve->setVisible(false);
+    mainCurvesSetVisible(false);
 
     int num_intervals = context->compareIntervals.count();
 
@@ -1286,3 +1352,17 @@ PfPvPlot::showCompareIntervals()
     replot();
 }
 
+void
+PfPvPlot::mainCurvesSetVisible(bool visible) {
+
+    if (gearRatioCurves.size()) {
+        QListIterator<QwtPlotCurve *> i(gearRatioCurves);
+        while (i.hasNext()) {
+            QwtPlotCurve *curve = i.next();
+            curve->setVisible(visible ? gear_ratio_display : false);
+        }
+    }
+
+    curve->setVisible(visible ? !gear_ratio_display : false);
+
+}
