@@ -16,46 +16,162 @@
  * Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <QDebug>
+#include "Context.h"
+#include "Athlete.h"
 #include "MainWindow.h"
+
 #include "RideItem.h"
 #include "RideFile.h"
 #include "RideImportWizard.h"
-#include "Context.h"
-#include "Athlete.h"
-#include "QuarqRideFile.h"
-#include <QWaitCondition>
+
+#include "RideAutoImportConfig.h"
+#include "HelpWhatsThis.h"
 #include "Settings.h"
 #include "Units.h"
+
 #include "GcRideFile.h"
 #include "JsonRideFile.h"
-#include "TcxRideFile.h"
-#include "MetricAggregator.h"
+#include "TcxRideFile.h" // for opening multi-ride file
 
+#include <QDebug>
+#include <QWaitCondition>
+#include <QMessageBox>
 
 // drag and drop passes urls ... convert to a list of files and call main constructor
-RideImportWizard::RideImportWizard(QList<QUrl> *urls, QDir &home, Context *context, QWidget *parent) : QDialog(parent), context(context)
+RideImportWizard::RideImportWizard(QList<QUrl> *urls, Context *context, QWidget *parent) : QDialog(parent), context(context)
 {
     setAttribute(Qt::WA_DeleteOnClose);
     setWindowFlags(windowFlags() | Qt::WindowStaysOnTopHint);
     QList<QString> filenames;
     for (int i=0; i<urls->count(); i++)
         filenames.append(QFileInfo(urls->value(i).toLocalFile()).absoluteFilePath());
-    init(filenames, home, context);
+    autoImportMode = false;
+    init(filenames, context);
     filenames.clear();
 }
 
-RideImportWizard::RideImportWizard(QList<QString> files, QDir &home, Context *context, QWidget *parent) : QDialog(parent), context(context)
+RideImportWizard::RideImportWizard(QList<QString> files, Context *context, QWidget *parent) : QDialog(parent), context(context)
 {
-    init(files, home, context);
+    autoImportMode = false;
+    init(files, context);
 }
 
-void
-RideImportWizard::init(QList<QString> files, QDir &home, Context * /*mainWindow*/)
+
+RideImportWizard::RideImportWizard(RideAutoImportConfig *dirs, Context *context, QWidget *parent) : QDialog(parent), context(context), importConfig(dirs)
 {
+    autoImportMode = true;
+    QList<QString> files;
+
+    // get the directories
+    QList<RideAutoImportRule> rules = importConfig->getConfig();
+
+    // prepare the widget to show the status of the directory
+    directoryWidget = new QTableWidget(rules.count(), 3, this);
+
+    directoryWidget->verticalHeader()->setDefaultSectionSize(20);
+
+    QTableWidgetItem *directoryHeading = new QTableWidgetItem;
+    directoryHeading->setText(tr("Directory"));
+    directoryWidget->setHorizontalHeaderItem(0, directoryHeading);
+
+    QTableWidgetItem *importRuleHeading = new QTableWidgetItem;
+    importRuleHeading->setText(tr("Import Rule"));
+    directoryWidget->setHorizontalHeaderItem(1, importRuleHeading);
+
+    QTableWidgetItem *statusHeading = new QTableWidgetItem;
+    statusHeading->setText(tr("Directory Status"));
+    directoryWidget->setHorizontalHeaderItem(2, statusHeading);
+
+    // and get the allowed files formats
+    const RideFileFactory &rff = RideFileFactory::instance();
+    QStringList suffixList = rff.suffixes();
+    suffixList.replaceInStrings(QRegExp("^"), "*.");
+    QStringList allFormats;
+    foreach(QString suffix, rff.suffixes())
+        allFormats << QString("*.%1").arg(suffix);
+
+    // Fill in the directory names and importRuleStatus
+    int i=-1;
+    foreach (RideAutoImportRule rule, rules){
+        i++; // do it here to allow "continue" - and start with "0"
+        QTableWidgetItem *t;
+
+        // Directory
+        t = new QTableWidgetItem();
+        t->setText(rule.getDirectory());
+        t->setFlags(t->flags() & (~Qt::ItemIsEditable));
+        directoryWidget->setItem(i,0,t);
+
+        // Import Rule
+        QList<QString> descriptions = rule.getRuleDescriptions();
+        t = new QTableWidgetItem();
+        t->setText(descriptions.at(rule.getImportRule()));
+        t->setFlags(t->flags() & (~Qt::ItemIsEditable));
+        directoryWidget->setItem(i,1,t);
+
+        // Import Status
+        t = new QTableWidgetItem();
+        t->setText(tr(""));
+        t->setFlags(t->flags() & (~Qt::ItemIsEditable));
+        directoryWidget->setItem(i,2,t);
+
+        // only add files if configured to do so
+        if (rule.getImportRule() == 0) {
+            directoryWidget->item(i,2)->setText(tr("No import"));
+            continue;
+        }
+
+        // do some checks on the directory first
+        QString currentImportDirectory = rule.getDirectory();
+        if (currentImportDirectory == "") {
+            directoryWidget->item(i,2)->setText(tr("No directory"));
+            continue;
+        }
+        QDir *importDir = new QDir (currentImportDirectory);
+        if (!importDir->exists()) {    // directory might not be available (USB,..)
+            directoryWidget->item(i,2)->setText(tr("Directory not available"));
+            continue;
+        }
+        if (!importDir->isReadable()) {
+            directoryWidget->item(i,2)->setText(tr("Directory not readable"));
+            continue;
+        }
+
+        // now get the files with their full names
+        QFileInfoList fileInfos = importDir->entryInfoList(allFormats, QDir::Files, QDir::NoSort);
+        if (!fileInfos.isEmpty()) {
+            int j = 0;
+            foreach(QFileInfo f, fileInfos) {
+                files.append(f.absoluteFilePath());
+                j++;
+            }
+            directoryWidget->item(i,2)->setText(tr("%1 files for import selected").arg(QString::number(j)));
+          } else {
+            directoryWidget->item(i,2)->setText(tr("No activity files found"));
+            continue;
+        }
+    }
+
+    directoryWidget->setColumnWidth(0, 480);
+    directoryWidget->setColumnWidth(1, 150);
+    directoryWidget->setColumnWidth(2, 250);
+
+    init(files, context);
+
+}
+
+
+void
+RideImportWizard::init(QList<QString> files, Context * /*mainWindow*/)
+{
+
+    // setup Help
+    HelpWhatsThis *help = new HelpWhatsThis(this);
+    this->setWhatsThis(help->getWhatsThisText(HelpWhatsThis::MenuBar_Activity_Import));
 
     // initialise dialog box
     tableWidget = new QTableWidget(files.count(), 6, this);
+
     tableWidget->setItemDelegate(new RideDelegate(1)); // use a delegate for column 1 date
     tableWidget->verticalHeader()->setDefaultSectionSize(20);
     phaseLabel = new QLabel;
@@ -73,13 +189,13 @@ RideImportWizard::init(QList<QString> files, QDir &home, Context * /*mainWindow*
     todayButton->addItem(tr("Choose Date"));
     cancelButton = new QPushButton(tr("Cancel"));
     abortButton = new QPushButton(tr("Abort"));
-    overFiles = new QCheckBox(tr("Overwrite Existing Files"));
+    //overFiles = new QCheckBox(tr("Overwrite Existing Files"));  // deprecate for this release... XXX
     // initially the cancel, overwrite and today widgets are hidden
     // they only appear whilst we are asking the user for dates
     cancelButton->setHidden(true);
     todayButton->setHidden(true);
-    overFiles->setHidden(true);
-    overwriteFiles = false;
+    //overFiles->setHidden(true);  // deprecate for this release... XXX
+    //overwriteFiles = false;
 
     aborted = false;
 
@@ -89,10 +205,10 @@ RideImportWizard::init(QList<QString> files, QDir &home, Context * /*mainWindow*
     // only used when editing dates
     connect(todayButton, SIGNAL(activated(int)), this, SLOT(todayClicked(int)));
     connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelClicked()));
-    connect(overFiles, SIGNAL(clicked()), this, SLOT(overClicked()));
+    // connect(overFiles, SIGNAL(clicked()), this, SLOT(overClicked()));  // deprecate for this release... XXX
 
     // title & headings
-    setWindowTitle(tr("Import Ride Files"));
+    setWindowTitle(tr("Import Files"));
     QTableWidgetItem *filenameHeading = new QTableWidgetItem;
     filenameHeading->setText(tr("Filename"));
     tableWidget->setHorizontalHeaderItem(0, filenameHeading);
@@ -117,19 +233,24 @@ RideImportWizard::init(QList<QString> files, QDir &home, Context * /*mainWindow*
     statusHeading->setText(tr("Import Status"));
     tableWidget->setHorizontalHeaderItem(5, statusHeading);
 
-    // save target dir
-    this->home = home;
+    // save target dir for the file import
+    this->homeImports = context->athlete->home->imports();
+    this->homeActivities = context->athlete->home->activities();
+    this->tmpActivities = context->athlete->home->tmpActivities();
 
     // Fill in the filenames and all the textItems
     for (int i=0; i < files.count(); i++) {
         QTableWidgetItem *t;
 
-        filenames.append(QFileInfo(files[i]).absoluteFilePath());
+        filenames.append(QFileInfo(files[i]).canonicalFilePath());
         blanks.append(true); // by default editable
 
         // Filename
         t = new QTableWidgetItem();
-        t->setText(QFileInfo(files[i]).fileName());
+        if (autoImportMode)
+            t->setText(QFileInfo(files[i]).canonicalFilePath());
+        else
+            t->setText(QFileInfo(files[i]).fileName());
         t->setFlags(t->flags() & (~Qt::ItemIsEditable));
         tableWidget->setItem(i,0,t);
 
@@ -172,11 +293,19 @@ RideImportWizard::init(QList<QString> files, QDir &home, Context * /*mainWindow*
     buttons->addStretch();
     buttons->addWidget(todayButton);
     buttons->addStretch();
-    buttons->addWidget(overFiles);
+    // buttons->addWidget(overFiles); // deprecate for this release... XXX
     buttons->addWidget(cancelButton);
     buttons->addWidget(abortButton);
 
     QVBoxLayout *contents = new QVBoxLayout(this);
+    if (autoImportMode) {
+        contents->addWidget(directoryWidget);
+        // only show table if files are available in autoimportMode
+        if (files.count() == 0) {
+            tableWidget->setVisible(false);
+            progressBar->setVisible(false);
+        }
+    }
     contents->addWidget(tableWidget);
     contents->addWidget(progressBar);
     contents->addLayout(buttons);
@@ -199,21 +328,32 @@ RideImportWizard::init(QList<QString> files, QDir &home, Context * /*mainWindow*
     resize(920 +
            ((files.count() > 16 ? 24 : 0) +
            ((files.count() > 9 && files.count() < 17) ? 8 : 0)),
-           118 + (files.count() > 16 ? 17*20 : (files.count()+1) * 20));
+           118 + ((files.count() > 16 ? 17*20 : (files.count()+1) * 20)
+           + ((autoImportMode) ? 100 : 0))); // assume not more the 5 directory in average
 
+    if (autoImportMode) directoryWidget->adjustSize();
     tableWidget->adjustSize();
 
-    // Refresh prior to running down the list & processing...
-    this->show();
+    // set number of files / so that a caller of the constructor can decide what to do
+    numberOfFiles = files.count();
+}
+
+int
+RideImportWizard::getNumberOfFiles() {
+    return numberOfFiles;
 }
 
 int
 RideImportWizard::process()
 {
 
+    // Make visible and put in front prior to running down the list & processing...
+    if (!isActiveWindow()) activateWindow();
+    this->show();
+
     // set progress bar limits - for each file we
     // will make 5 passes over the files
-    //         1. checking it is a file ane readable
+    //         1. checking if a file is readable
     //         2. parsing it with the RideFileReader
     //         3. [optional] collect date/time information from user
     //         4. copy file into Library
@@ -222,6 +362,8 @@ RideImportWizard::process()
     // So, therefore the progress bar runs from 0 to files*4. (since step 5 is not implemented yet)
     progressBar->setMinimum(0);
     progressBar->setMaximum(filenames.count()*4);
+    if (!isActiveWindow()) activateWindow();
+
 
     // Pass one - Is it valid?
     phaseLabel->setText(tr("Step 1 of 4: Check file permissions"));
@@ -256,6 +398,7 @@ RideImportWizard::process()
     }
 
     if (aborted) { done(0); }
+    if (!isActiveWindow()) activateWindow();
     repaint();
     QApplication::processEvents();
 
@@ -276,6 +419,7 @@ RideImportWizard::process()
               QApplication::processEvents();
 
               if (aborted) { done(0); }
+              if (!isActiveWindow()) activateWindow();
               this->repaint();
               QApplication::processEvents();
 
@@ -380,8 +524,9 @@ RideImportWizard::process()
                    // ride != NULL but !errors.isEmpty() means they're just warnings
                    if (errors.isEmpty())
                        tableWidget->item(i,5)->setText(tr("Validated"));
-                   else
+                   else {
                        tableWidget->item(i,5)->setText(tr("Warning - ") + errors.join(tr(" ")));
+                   }
 
                    // Set Date and Time
                    if (ride->startTime().isNull()) {
@@ -393,7 +538,7 @@ RideImportWizard::process()
 
                    } else {
 
-                       // Cool, the date and time was extrcted from the source file
+                       // Cool, the date and time was extracted from the source file
                        blanks[i] = false;
                        tableWidget->item(i,1)->setText(ride->startTime().toString(tr("dd MMM yyyy")));
                        tableWidget->item(i,2)->setText(ride->startTime().toString("hh:mm:ss"));
@@ -421,14 +566,14 @@ RideImportWizard::process()
                        .arg(secs%3600/60,2,10,zero)
                        .arg(secs%60,2,10,zero);
                    tableWidget->item(i,3)->setText(time);
-                   tableWidget->item(i,3)->setTextAlignment(Qt::AlignHCenter); // put in the middle
+                   tableWidget->item(i,3)->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter); // put in the middle
 
                    // show distance by looking at last data point
                    QString dist = context->athlete->useMetricUnits
                        ? QString ("%1 km").arg(km, 0, 'f', 1)
                        : QString ("%1 mi").arg(km * MILES_PER_KM, 0, 'f', 1);
                    tableWidget->item(i,4)->setText(dist);
-                   tableWidget->item(i,4)->setTextAlignment(Qt::AlignRight); // put in the middle
+                   tableWidget->item(i,4)->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
 
                    delete ride;
                } else {
@@ -439,6 +584,7 @@ RideImportWizard::process()
         progressBar->setValue(progressBar->value()+1);
         QApplication::processEvents();
         if (aborted) { done(0); }
+        if (!isActiveWindow()) activateWindow();
         this->repaint();
 
         next:;
@@ -461,8 +607,11 @@ RideImportWizard::process()
 
         // does nothing for the moment
         progressBar->setValue(progressBar->value()+1);
+        if (!isActiveWindow()) activateWindow();
         progressBar->repaint();
    }
+   // get it on top
+   activateWindow();
 
    // Wait for user to press save
    abortButton->setText(tr("Save"));
@@ -474,7 +623,7 @@ RideImportWizard::process()
 
    if (needdates == 0) {
       // no need to wait for the user to input dates
-      // nd press save if all the dates are set
+      // and press save if all the dates are set
       // (i.e. they got set by the file importers already)
       // do nothing for now since we need to confirm dates
       // and confirm overwrite files rather than importing
@@ -490,14 +639,24 @@ RideImportWizard::process()
    }
    connect(tableWidget, SIGNAL(itemChanged(QTableWidgetItem *)), this, SLOT(activateSave()));
 
+   // in autoimport mode / no files, skip "Save",... and goto "Finish"
+   if (autoImportMode && filenames.count()== 0) {
+       cancelButton->setHidden(true);
+       todayButton->setHidden(true);
+       abortButton->setDisabled(false);
+       phaseLabel->setText(tr("No files for automatic import selected."));
+       abortButton->setText(tr("Finish"));
+       aborted = false;
+   }
+
    return 0;
 }
 
-void
-RideImportWizard::overClicked()
-{
+//void
+//RideImportWizard::overClicked()
+//{
     //overwriteFiles = overFiles->isChecked(); //deprecate in this release XXX
-}
+//}
 
 void
 RideImportWizard::activateSave()
@@ -583,7 +742,7 @@ RideImportWizard::todayClicked(int index)
 
     // More than a days worth of rides so can't squeeze into a single day!
     if (totalduration > (24 * 3600)) {
-        QMessageBox::warning ( this, tr ( "Invalid Selection" ), tr ( "More than 24hrs of rides to fit into a day" ));
+        QMessageBox::warning ( this, tr ( "Invalid Selection" ), tr ( "More than 24hrs of activities to fit into a day" ));
         return;
     }
 
@@ -636,7 +795,9 @@ RideImportWizard::todayClicked(int index)
     }
     // phew! - repaint!
     QApplication::processEvents();
+    if (!isActiveWindow()) activateWindow();
     tableWidget->repaint();
+
 }
 
 void
@@ -650,76 +811,39 @@ struct cpi_file_info {
     QString file, inname, outname;
 };
 
-static QStringList
-findDuplicates(QString filename)
-{
-    // does this ride already exist?
-    // either the full name is a match
-    // or the same name but different
-    // filetype: e.g. xxx.gc matches xxx.tcx
-    QStringList duplicates;
-
-    // get a list of possible duplicates (files we support)
-    QString basename = QFileInfo(filename).baseName();
-    QStringList filters;
-    foreach (QString ext, RideFileFactory::instance().suffixes()) {
-        QString check = basename + "." + ext;
-        filters << check;
-    }
-
-    // check if any matched (case insensitive)
-    QFlags<QDir::Filter> spec = QDir::Files;
-#ifdef Q_OS_WIN32
-    spec |= QDir::Hidden;
-#endif
-    // get list and convert to full path
-    foreach(QString name, QFileInfo(filename).dir().entryList(filters, spec, QDir::Name)) {
-        duplicates << QFileInfo(filename).dir().path() + "/" + name;
-    }
-
-    return duplicates;
-}
-
-static void
-removeDuplicate(QString filename)
-{
-    // rename to .bak, if that already exists
-    // then wipe it first
-    QString backup = filename + ".bak";
-    QFile(backup).remove(); // wipe it, if it is there
-    QFile(filename).rename(backup);
-}
 
 void
 RideImportWizard::abortClicked()
 {
+    // NOTE: abort button morphs into save and finish button later - so all 3 variants are processed here
 
     // if done when labelled abort we kill off this dialog
     QString label = abortButton->text();
 
+    // Process "ABORT"
     if (label == tr("Abort")) {
         hide();
-        context->athlete->isclean = false;
-        context->athlete->metricDB->refreshMetrics();
         aborted=true; // terminated. I'll be back.
         return;
     }
 
+    // Process "FINISH"
     if (label == tr("Finish")) {
        // phew. our work is done. -- lets force an update stats...
        hide();
-       context->athlete->isclean = false;
-       context->athlete->metricDB->refreshMetrics();
        done(0);
        return;
     }
 
-   int needdates=0;
-   for (int i=0; i<filenames.count(); i++) {
+    // Process "SAVE"
+
+    // SAVE STEP 1 - Check if there are any dates needed and not yet defined
+    int needdates=0;
+    for (int i=0; i<filenames.count(); i++) {
 
         QTableWidgetItem *t = tableWidget->item(i,5);
         if (t->text().startsWith(tr("Error"))) continue;
-       // date needed?
+        // date needed?
         t = tableWidget->item(i,1);
         if (t->text() == "") {
             needdates++;
@@ -732,11 +856,12 @@ RideImportWizard::abortClicked()
             needdates++;
         }
         t->setFlags(t->flags() | (Qt::ItemIsEditable));
-   }
+    }
 
-   if (needdates) return; // no dice dude, we need those dates filled in!
+    if (needdates) return; // no dice dude, we need those dates filled in!
 
-    // if done when labelled save we copy the files and run the cpi calculator
+
+    // SAVE STEP 2 - set the labels and make the text un-editable
     phaseLabel->setText(tr("Step 4 of 4: Save to Library"));
 
     abortButton->setText(tr("Abort"));
@@ -747,145 +872,128 @@ RideImportWizard::abortClicked()
 
     // now set this fields uneditable again ... yeesh.
     for (int i=0; i <filenames.count(); i++) {
-            QTableWidgetItem *t = tableWidget->item(i,1);
-            t->setFlags(t->flags() & (~Qt::ItemIsEditable));
-            t = tableWidget->item(i,2);
-            t->setFlags(t->flags() & (~Qt::ItemIsEditable));
+        QTableWidgetItem *t = tableWidget->item(i,1);
+        t->setFlags(t->flags() & (~Qt::ItemIsEditable));
+        t = tableWidget->item(i,2);
+        t->setFlags(t->flags() & (~Qt::ItemIsEditable));
     }
 
     QChar zero = QLatin1Char ( '0' );
 
+
+    // Saving now - process the files one-by-one
     for (int i=0; i< filenames.count(); i++) {
 
-        if (tableWidget->item(i,5)->text().startsWith(tr("Error"))) continue; // skip error
+        if (tableWidget->item(i,5)->text().startsWith(tr("Error"))) continue; // skip errors
 
         tableWidget->item(i,5)->setText(tr("Saving..."));
         tableWidget->setCurrentCell(i,5);
         QApplication::processEvents();
         if (aborted) { done(0); }
+        if (!isActiveWindow()) activateWindow();
         this->repaint();
 
-        // Setup the ridetime as a QDateTime
+
+        // SAVE STEP 3 - prepare the new file names for the next steps - basic name and .JSON in GC format
+
         QDateTime ridedatetime = QDateTime(QDate().fromString(tableWidget->item(i,1)->text(), tr("dd MMM yyyy")),
-                                 QTime().fromString(tableWidget->item(i,2)->text(), "hh:mm:ss"));
-        QString suffix = QFileInfo(filenames[i]).suffix();
+                                           QTime().fromString(tableWidget->item(i,2)->text(), "hh:mm:ss"));
         QString targetnosuffix = QString ( "%1_%2_%3_%4_%5_%6" )
-                               .arg ( ridedatetime.date().year(), 4, 10, zero )
-                               .arg ( ridedatetime.date().month(), 2, 10, zero )
-                               .arg ( ridedatetime.date().day(), 2, 10, zero )
-                               .arg ( ridedatetime.time().hour(), 2, 10, zero )
-                               .arg ( ridedatetime.time().minute(), 2, 10, zero )
-                               .arg ( ridedatetime.time().second(), 2, 10, zero );
-        QString target = QString ("%1.%2" )
-                               .arg ( targetnosuffix )
-                               .arg ( suffix );
-        QString fulltarget = home.absolutePath() + "/" + target;
+                .arg ( ridedatetime.date().year(), 4, 10, zero )
+                .arg ( ridedatetime.date().month(), 2, 10, zero )
+                .arg ( ridedatetime.date().day(), 2, 10, zero )
+                .arg ( ridedatetime.time().hour(), 2, 10, zero )
+                .arg ( ridedatetime.time().minute(), 2, 10, zero )
+                .arg ( ridedatetime.time().second(), 2, 10, zero );
+        QString activitiesTarget = QString ("%1.%2" ).arg ( targetnosuffix ).arg ( "json" );
 
-        // if its a gc file we need to parse and serialize
-        // using the ridedatetime and target filename
-        if (filenames[i].endsWith(".gc", Qt::CaseInsensitive) ||
-            filenames[i].endsWith(".json", Qt::CaseInsensitive)) {
+        // create filenames incl. directory path for GC .JSON for both /tmpActivities and /activities directory
+        QString tmpActivitiesFulltarget = tmpActivities.canonicalPath() + "/" + activitiesTarget;
+        QString finalActivitiesFulltarget = homeActivities.canonicalPath() + "/" + activitiesTarget;
 
-            QStringList duplicates;
+        // check if a ride at this point of time already exists in /activities - if yes, skip import
+        if (QFileInfo(finalActivitiesFulltarget).exists()) { tableWidget->item(i,5)->setText(tr("Error - Activity file exists")); continue; }
 
-            // CHECK FOR DUPLICATE
-            duplicates = findDuplicates(fulltarget);
-            if (duplicates.count() && !overwriteFiles) {
-                tableWidget->item(i,5)->setText(tr("Error - File exists"));
-            } else {
 
-                // wipe away the duplicate
-                foreach(QString duplicate, duplicates) {
-                    removeDuplicate(duplicate); // we do not use removeRide coz it clashes
-                }
+        // SAVE STEP 4 - copy the source file to "/imports" directory (if it's not taken from there as source)
+        // add the date/time of the target to the source file name (for identification)
 
-                // read the file (again)
-                QStringList errors;
-                QFile thisfile(filenames[i]);
-                RideFile *ride(RideFileFactory::instance().openRideFile(context, thisfile, errors));
+        // copy the sourceFile to /imports ONLY if the source is NOT coming from /imports itself
+        QFileInfo sourceFileInfo (filenames[i]);
+        QString importsTarget;
+        if (sourceFileInfo.canonicalPath() != homeImports.canonicalPath()) {
 
-                // update ridedatetime
-                ride->setStartTime(ridedatetime);
-
-                // serialize
-                if (filenames[i].endsWith(".gc")) {
-                    GcFileReader reader;
-                    QFile target(fulltarget);
-                    reader.writeRideFile(context, ride, target);
-                } else {
-                    JsonFileReader reader;
-                    QFile target(fulltarget);
-                    reader.writeRideFile(context, ride, target);
-                }
-
-                // clear
-                delete ride;
-
-                if (duplicates.count()) {
-                    tableWidget->item(i,5)->setText(tr("File Overwritten"));
-                } else {
-                    tableWidget->item(i,5)->setText(tr("File Saved"));
-                    context->athlete->addRide(QFileInfo(fulltarget).fileName(), 
-                                              tableWidget->rowCount() < 20 ? true : false); // don't signal if mass importing
-                }
+            // add the GC file base name to create unique file names during import
+            // there should not be 2 ride files with exactly the same time stamp (as this is also not foreseen for the .json)
+            importsTarget = sourceFileInfo.baseName() + "_" + targetnosuffix + "." + sourceFileInfo.suffix();
+            QString importsFulltarget = homeImports.canonicalPath() + "/" + importsTarget;
+            // copy the source file to /imports with adjusted name
+            QFile source(filenames[i]);
+            if (!source.copy(importsFulltarget)) {
+                tableWidget->item(i,5)->setText(tr("Error - copy of %1 to import directory failed").arg(importsTarget));
             }
-
         } else {
-            // for native file formats the filename IS the ride date time so
-            // no need to write -- we just copy
-
-            // so now we have sourcefile in 'filenames[i]' and target file name in 'target'
-            if (!fulltarget.compare(filenames[i])) { // they are the same file! so skip copy
-                tableWidget->item(i,5)->setText(tr("Error - Source is Target"));
-
-            // CHECK FOR DUPLICATE
-            } else if (findDuplicates(fulltarget).count()) {
-                if (overwriteFiles) {
-
-                    // wipe away that duplicate
-                    foreach(QString duplicate, findDuplicates(fulltarget)) {
-                        removeDuplicate(duplicate);
-                    }
-
-                    tableWidget->item(i,5)->setText(tr("Overwriting file..."));
-                    QFile source(filenames[i]);
-                    QString fulltargettmp(home.absolutePath() + tr("/") + targetnosuffix + tr(".tmp"));
-
-                    if (source.copy(fulltargettmp)) {
-
-                        // mv tmp to target
-                        QFile temp(fulltargettmp);
-                        if (temp.rename(fulltarget)) {
-                            tableWidget->item(i,5)->setText(tr("File Overwritten"));
-                            //no need to add since its already there!
-                        } else
-                            tableWidget->item(i,5)->setText(tr("Error - overwrite failed"));
-                    } else {
-                        tableWidget->item(i,5)->setText(tr("Error - overwrite failed"));
-                    }
-                } else {
-                    tableWidget->item(i,5)->setText(tr("Error - File exists"));
-                }
-            } else {
-                    tableWidget->item(i,5)->setText(tr("Saving file..."));
-                    QFile source(filenames[i]);
-                    if (source.copy(fulltarget)) {
-                        tableWidget->item(i,5)->setText(tr("File Saved"));
-                        context->athlete->addRide(QFileInfo(fulltarget).fileName(), 
-                                                  tableWidget->rowCount() < 20 ? true : false); // don't signal if mass importing
-
-                        // free immediately otherwise all imported rides are cached
-                        // and with large imports this can lead to memory exhaustion
-                        // BUT! Some charts/windows will hava snaffled away the ridefile
-                        // pointer which is now invalid so once all the rides have been imported
-                        // we need to select the last one... see below
-                    } else
-                        tableWidget->item(i,5)->setText(tr("Error - copy failed"));
-            }
+            // file is re-imported from /imports - keep the name for .JSON Source File Tag
+            importsTarget = sourceFileInfo.fileName();
         }
+
+
+        // SAVE STEP 5 - open the file with the respective format reader and export as .JSON
+        // to track if addRideCache() has caused an error due to bad data we work with a interim directory for the activities
+        // -- first   export to /tmpactivities
+        // -- second  create RideCache() entry
+        // -- third   move file from /tmpactivities to /activities
+
+        tableWidget->item(i,5)->setText(tr("Saving file..."));
+
+        // serialize the file to .JSON
+        QStringList errors;
+        QFile thisfile(filenames[i]);
+        RideFile *ride(RideFileFactory::instance().openRideFile(context, thisfile, errors));
+
+        // did the input file parse ok ? (should be fine here - since it was alrady checked before - but just in case)
+        if (ride) {
+
+            // update ridedatetime and set the Source File name
+            ride->setStartTime(ridedatetime);
+            ride->setTag("Source Filename", importsTarget);
+            ride->setTag("Filename", activitiesTarget);
+
+            // serialize
+            JsonFileReader reader;
+            QFile target(tmpActivitiesFulltarget);
+            if (reader.writeRideFile(context, ride, target)) {
+
+                // now try adding the Ride to the RideCache - since this may fail due to various reason, the activity file
+                // is stored in tmpActivities during this process to understand which file has create the problem when restarting GC
+                // - only after the step was successful the file is moved
+                // to the "clean" activities folder
+                context->athlete->addRide(QFileInfo(tmpActivitiesFulltarget).fileName(),
+                                          tableWidget->rowCount() < 20 ? true : false, // don't signal if mass importing
+                                          true);                                       // file is available only in /tmpActivities, so use this one please
+                // rideCache is successfully updated, let's move the file to the real /activities
+                if (moveFile(tmpActivitiesFulltarget, finalActivitiesFulltarget)) {
+                    tableWidget->item(i,5)->setText(tr("File Saved"));
+                    // and correct the path locally stored in Ride Item
+                    context->ride->setFileName(homeActivities.canonicalPath(), activitiesTarget);
+                }  else {
+                    tableWidget->item(i,5)->setText(tr("Error - Moving %1 to activities folder").arg(activitiesTarget));
+                }
+
+            }  else {
+                tableWidget->item(i,5)->setText(tr("Error - .JSON creation failed"));
+            }
+        } else {
+            tableWidget->item(i,5)->setText(tr("Error - Import of activitiy file failed"));
+        }
+
+        // clear
+        delete ride;
+
         QApplication::processEvents();
         if (aborted) { done(0); }
         progressBar->setValue(progressBar->value()+1);
+        if (!isActiveWindow()) activateWindow();
         this->repaint();
     }
 
@@ -894,18 +1002,42 @@ RideImportWizard::abortClicked()
     for (int i=0; i< filenames.count(); i++)
         if (!tableWidget->item(i,5)->text().startsWith(tr("Error"))) {
             completed++;
-    }
+        }
 
     tableWidget->setSortingEnabled(true); // so you can browse through errors etc
     QString donemessage = QString(tr("Import Complete. %1 of %2 successful."))
-                                  .arg(completed, 1, 10, zero)
-                                  .arg(filenames.count(), 1, 10, zero);
+            .arg(completed, 1, 10, zero)
+            .arg(filenames.count(), 1, 10, zero);
     progressBar->setValue(progressBar->maximum());
     phaseLabel->setText(donemessage);
     abortButton->setText(tr("Finish"));
     aborted = false;
 }
 
+
+bool
+RideImportWizard::moveFile(const QString &source, const QString &target) {
+
+    QFile r(source);
+
+    // first try it with a rename
+    if (r.rename(target)) return true; // job is done
+
+    // now the harder variant (copy & delete)
+    if (r.copy(target))
+      {
+        // try to remove - but if this fails, no problem, file has been copied at least
+        r.remove();
+        // even if remove failed, the copy was successful - so GC is fine
+        return true;
+      }
+
+    // more required ?
+
+    return false;
+
+}
+
 // clean up files
 RideImportWizard::~RideImportWizard()
 {
