@@ -18,6 +18,7 @@
 
 #include "ScatterPlot.h"
 #include "ScatterWindow.h"
+#include "Statistic.h"
 #include "IntervalItem.h"
 #include "Context.h"
 #include "Context.h"
@@ -90,6 +91,15 @@ pointType(const RideFilePoint *point, int type, int side, bool metric, double cr
         case MODEL_LRBALANCE : return !side ? point->lrbalance : (100 - point->lrbalance);
         case MODEL_TE : return side ? point->rte : point->lte;
         case MODEL_PS : return side ? point->rps : point->lps;
+
+        case MODEL_RV : return point->rvert;
+        case MODEL_RGCT : return point->rcontact;
+        case MODEL_RCAD : return point->rcad;
+        case MODEL_GEAR : return point->gear;
+        case MODEL_SMO2 : return point->smo2;
+        case MODEL_THB : return point->thb;
+        case MODEL_HHB : return point->hhb;
+        case MODEL_O2HB : return point->o2hb;
     }
     return 0; // ? unknown channel ?
 }
@@ -139,6 +149,14 @@ QString ScatterPlot::describeType(int type, bool longer, bool metric)
             case MODEL_LRBALANCE : return (tr("L/R Balance"));
             case MODEL_TE : return (tr("Torque Efficiency"));
             case MODEL_PS : return (tr("Pedal Smoothness"));
+            case MODEL_RV :  return (tr("Running Vertical Oscillation"));
+            case MODEL_RGCT : return (tr("Running Ground Contact Time"));
+            case MODEL_RCAD :  return (tr("Running Cadence"));
+            case MODEL_GEAR :  return (tr("Gear Ratio"));
+            case MODEL_SMO2 :  return (tr("Muscle Oxygen"));
+            case MODEL_THB :  return (tr("Haemoglobin Mass"));
+            case MODEL_HHB :  return (tr("Deoxygenated Haemoglobin"));
+            case MODEL_O2HB :  return (tr("Oxygenated Haemoglobin"));
         }
         return (tr("Unknown"));; // ? unknown channel ?
     } else {
@@ -165,6 +183,14 @@ QString ScatterPlot::describeType(int type, bool longer, bool metric)
             case MODEL_LRBALANCE : return (tr("Balance"));
             case MODEL_TE : return (tr("TE"));
             case MODEL_PS : return (tr("PS"));
+            case MODEL_RV :  return (tr("RV"));
+            case MODEL_RGCT : return (tr("GCT"));
+            case MODEL_RCAD :  return (tr("Run Cad"));
+            case MODEL_GEAR :  return (tr("Gear"));
+            case MODEL_SMO2 :  return (tr("SmO2"));
+            case MODEL_THB :  return (tr("tHb"));
+            case MODEL_O2HB :  return (tr("O2Hb"));
+            case MODEL_HHB :  return (tr("HHb"));
         }
         return (tr("None")); // ? unknown channel ?
     }
@@ -194,13 +220,13 @@ ScatterPlot::ScatterPlot(Context *context) : context(context)
     sd->enableComponent(QwtScaleDraw::Backbone, false);
     setAxisScaleDraw(QwtPlot::yLeft, sd);
 
-    connect(context, SIGNAL(configChanged()), this, SLOT(configChanged()));
-    connect(context, SIGNAL(intervalHover(RideFileInterval)), this, SLOT(intervalHover(RideFileInterval)));
+    connect(context, SIGNAL(configChanged(qint32)), this, SLOT(configChanged(qint32)));
+    connect(context, SIGNAL(intervalHover(IntervalItem*)), this, SLOT(intervalHover(IntervalItem*)));
 
     // lets watch the mouse move...
     new mouseTracker(this);
 
-    configChanged(); // use latest colors etc
+    configChanged(CONFIG_APPEARANCE | CONFIG_GENERAL); // use latest wheelsize/cranklength and colors
 }
 
 void ScatterPlot::setData (ScatterSettings *settings)
@@ -240,15 +266,22 @@ void ScatterPlot::setData (ScatterSettings *settings)
     }
     intervalCurves.clear();
 
+    // clear out any label
+    foreach(QwtPlotMarker *c, labels) {
+        c->detach();
+        delete c;
+    }
+    labels.clear();
+
     // count the currently selected intervals
     QVector<int> intervals;
     QMap<int,int> displaySequence;
 
-    for (int child=0; child<context->athlete->allIntervalItems()->childCount(); child++) {
-        IntervalItem *current = dynamic_cast<IntervalItem *>(context->athlete->allIntervalItems()->child(child));
-        if ((current != NULL) && current->isSelected()) {
+    for (int child=0; child<settings->ride->intervals().count(); child++) {
+        IntervalItem *current = settings->ride->intervals().at(child);
+        if (current->selected) {
             intervals.append(child);
-            displaySequence.insert(current->displaySequence, intervals.count()-1);
+            displaySequence.insert(current->displaySequence, child);
         }
     }
 
@@ -279,127 +312,235 @@ void ScatterPlot::setData (ScatterSettings *settings)
 
     for (int side = 0; side < curves; side++) {
 
-        //
-        // GET ALL DATA AND MIN / MAX
-        //
-        int points=0;
-        QVector<double> x;
-        QVector<double> y;
+        if (context->isCompareIntervals == false) {
+            //
+            // GET ALL DATA AND MIN / MAX
+            //
+            int points=0;
+            QVector<double> x;
+            QVector<double> y;
 
-        foreach(const RideFilePoint *point, settings->ride->ride()->dataPoints()) {
-
-            double xv = pointType(point, settings->x, side, context->athlete->useMetricUnits, cranklength);
-            double yv = pointType(point, settings->y, side, context->athlete->useMetricUnits, cranklength);
-
-            // skip zeroes?
-            if (settings->ignore && (int(xv) == 0 || int(yv) == 0)) continue;
-
-            // add it 
-            x <<xv;
-            y <<yv;
-            points++;
-
-            if (yv > maxY) maxY = yv;
-            if (yv < minY) minY = yv;
-            if (xv > maxX) maxX = xv;
-            if (xv < minX) minX = xv;
-        }
-
-        //
-        // CREATE INTERVAL CURVES (and use framing data from above if needed)
-        //
-        if (intervals.count() > 0) {
-
-            // interval data in here
-            QVector<QVector<double> > xvals(intervals.count()); // array of curve x arrays
-            QVector<QVector<double> > yvals(intervals.count()); // array of curve x arrays
-            QVector<int> points(intervals.count());             // points in eac curve
-
-            // extract interval data
             foreach(const RideFilePoint *point, settings->ride->ride()->dataPoints()) {
 
-                double x = pointType(point, settings->x, side, context->athlete->useMetricUnits, cranklength);
-                double y = pointType(point, settings->y, side, context->athlete->useMetricUnits, cranklength);
+                double xv = pointType(point, settings->x, side, context->athlete->useMetricUnits, cranklength);
+                double yv = pointType(point, settings->y, side, context->athlete->useMetricUnits, cranklength);
 
-                if (!(settings->ignore && (x == 0 && y ==0))) {
+                // skip values ? Like zeroes...
+                if (skipValues(xv, yv, settings))
+                    continue;
 
-                    // which interval is it in?
-                    for (int idx=0; idx<intervals.count(); idx++) {
+                // add it
+                x <<xv;
+                y <<yv;
+                points++;
 
-                        IntervalItem *current = dynamic_cast<IntervalItem *>(context->athlete->allIntervalItems()->child(intervals[idx]));
+                if (yv > maxY) maxY = yv;
+                if (yv < minY) minY = yv;
+                if (xv > maxX) maxX = xv;
+                if (xv < minX) minX = xv;
+            }
 
-                        if (point->secs+settings->ride->ride()->recIntSecs() > current->start && point->secs< current->stop) {
-                            xvals[idx].append(x);
-                            yvals[idx].append(y);
-                            points[idx]++;
+            //
+            // CREATE INTERVAL CURVES (and use framing data from above if needed)
+            //
+            if (intervals.count() > 0) {
+
+                // interval data in here
+                QVector<QVector<double> > xvals(settings->ride->intervals().count()); // array of curve x arrays
+                QVector<QVector<double> > yvals(settings->ride->intervals().count()); // array of curve x arrays
+                QVector<int> points(settings->ride->intervals().count());             // points in eac curve
+
+                // extract interval data
+                foreach(const RideFilePoint *point, settings->ride->ride()->dataPoints()) {
+
+                    double x = pointType(point, settings->x, side, context->athlete->useMetricUnits, cranklength);
+                    double y = pointType(point, settings->y, side, context->athlete->useMetricUnits, cranklength);
+
+                    if (!(settings->ignore && (x == 0 && y ==0))) {
+
+                        // which interval is it in?
+                        for (int idx=0; idx<settings->ride->intervals().count(); idx++) {
+
+                            IntervalItem *current = settings->ride->intervals().at(idx);
+                            if (current->selected && point->secs+settings->ride->ride()->recIntSecs() > current->start && point->secs< current->stop) {
+                                xvals[idx].append(x);
+                                yvals[idx].append(y);
+                                points[idx]++;
+                            }
                         }
                     }
                 }
+
+                // now we have the interval data lets create the curves
+                QMapIterator<int, int> order(displaySequence);
+                while (order.hasNext()) {
+                    order.next();
+                    int idx = order.value();
+
+                    QColor intervalColor;
+                    intervalColor.setHsv((255/settings->ride->intervals().count()) * idx, 255,255);
+                    // left / right are darker lighter
+                    if (side) intervalColor = intervalColor.lighter(50);
+
+                    QwtSymbol *sym = new QwtSymbol;
+                    sym->setStyle(QwtSymbol::Ellipse);
+                    sym->setSize(4);
+                    sym->setBrush(QBrush(intervalColor));
+                    sym->setPen(QPen(intervalColor));
+
+                    QwtPlotCurve *ic = new QwtPlotCurve();
+
+                    ic->setSymbol(sym);
+                    ic->setStyle(QwtPlotCurve::Dots);
+                    ic->setRenderHint(QwtPlotItem::RenderAntialiased);
+                    ic->setSamples(xvals[idx].constData(), yvals[idx].constData(), points[idx]);
+                    ic->attach(this);
+                    intervalCurves.append(ic);
+
+                    // show as a line too, if not framed
+#if 0 // a bit ugly tbh
+                    if (!settings->frame) {
+                        QwtPlotCurve *icl = new QwtPlotCurve();
+                        icl->setStyle(QwtPlotCurve::Lines);
+                        icl->setPen(QPen(intervalColor, Qt::SolidLine));
+                        icl->setRenderHint(QwtPlotItem::RenderAntialiased);
+                        icl->setSamples(xvals[idx].constData(), yvals[idx].constData(), points[idx]);
+                        icl->attach(this);
+                        intervalCurves.append(icl);
+                    }
+#endif
+                }
             }
 
-            // now we have the interval data lets create the curves
-            QMapIterator<int, int> order(displaySequence);
-            while (order.hasNext()) {
-                order.next();
-                int idx = order.value();
+            // setup the framing curve
+            if (intervals.count() == 0 || settings->frame) {
+                smooth(x, y, points, settings->smoothing);
 
-                QColor intervalColor;
-                intervalColor.setHsv((255/context->athlete->allIntervalItems()->childCount()) * (intervals[idx]), 255,255);
-                // left / right are darker lighter
-                if (side) intervalColor = intervalColor.lighter(50);
+                if (side) {
 
-                QwtSymbol *sym = new QwtSymbol;
-                sym->setStyle(QwtSymbol::Ellipse);
-                sym->setSize(4);
-                sym->setBrush(QBrush(intervalColor));
-                sym->setPen(QPen(intervalColor));
+                    QwtSymbol *sym = new QwtSymbol;
+                    sym->setStyle(QwtSymbol::Ellipse);
+                    sym->setSize(4);
+                    sym->setPen(QPen(Qt::cyan));
+                    sym->setBrush(QBrush(Qt::cyan));
 
-                QwtPlotCurve *ic = new QwtPlotCurve();
+                    curve2 = new QwtPlotCurve();
+                    curve2->setSymbol(sym);
+                    curve2->setStyle(QwtPlotCurve::Dots);
+                    curve2->setRenderHint(QwtPlotItem::RenderAntialiased);
+                    curve2->setSamples(x.constData(), y.constData(), points);
+                    curve2->setZ(-1);
+                    curve2->attach(this);
 
-                ic->setSymbol(sym);
-                ic->setStyle(QwtPlotCurve::Dots);
-                ic->setRenderHint(QwtPlotItem::RenderAntialiased);
-                ic->setSamples(xvals[idx].constData(), yvals[idx].constData(), points[idx]);
-                ic->attach(this);
+                    if (settings->trendLine>0)  {
+                        addTrendLine(x, y, points, Qt::cyan);
+                    }
 
-                intervalCurves.append(ic);
+                } else {
+
+                    QwtSymbol *sym = new QwtSymbol;
+                    sym->setStyle(QwtSymbol::Ellipse);
+                    sym->setSize(4);
+                    sym->setPen(QPen(Qt::red));
+                    sym->setBrush(QBrush(Qt::red));
+
+                    curve = new QwtPlotCurve();
+                    curve->setSymbol(sym);
+                    curve->setStyle(QwtPlotCurve::Dots);
+                    curve->setRenderHint(QwtPlotItem::RenderAntialiased);
+                    curve->setSamples(x.constData(), y.constData(), points);
+                    curve->setZ(-1);
+                    curve->attach(this);
+
+                    if (settings->trendLine>0)  {
+                        addTrendLine(x, y, points, Qt::red);
+                    }
+                }
             }
         }
+        else {
 
-        // setup the framing curve
-        if (intervals.count() == 0 || settings->frame) {
+            if (context->compareIntervals.count() > 0) {
 
-            if (side) {
+                //
+                // CREATE COMPARE CURVES
+                //
 
-                QwtSymbol *sym = new QwtSymbol;
-                sym->setStyle(QwtSymbol::Ellipse);
-                sym->setSize(4);
-                sym->setPen(QPen(Qt::cyan));
-                sym->setBrush(QBrush(Qt::cyan));
+                int i=0;
+                if (settings->compareMode > 0)
+                    i++;
+                for (; i< context->compareIntervals.count(); i++) {
+                    CompareInterval interval = context->compareIntervals.at(i);
 
-                curve2 = new QwtPlotCurve();
-                curve2->setSymbol(sym);
-                curve2->setStyle(QwtPlotCurve::Dots);
-                curve2->setRenderHint(QwtPlotItem::RenderAntialiased);
-	            curve2->setSamples(x.constData(), y.constData(), points);
-                curve2->setZ(-1);
-                curve2->attach(this);
+                    if (interval.checked == false)
+                        continue;
 
-            } else {
+                    // interval data in here
+                    QVector<double> xval;
+                    QVector<double> yval;
+                    int nbPoints = 0;
 
-                QwtSymbol *sym = new QwtSymbol;
-                sym->setStyle(QwtSymbol::Ellipse);
-                sym->setSize(4);
-                sym->setPen(QPen(Qt::red));
-                sym->setBrush(QBrush(Qt::red));
+                    // extract interval data
+                    foreach(const RideFilePoint *point, interval.data->dataPoints()) {
 
-                curve = new QwtPlotCurve();
-                curve->setSymbol(sym);
-                curve->setStyle(QwtPlotCurve::Dots);
-                curve->setRenderHint(QwtPlotItem::RenderAntialiased);
-	            curve->setSamples(x.constData(), y.constData(), points);
-                curve->setZ(-1);
-                curve->attach(this);
+                        double x;
+                        double y;
+                        if (settings->compareMode == 1) {
+                            const RideFilePoint *refPoint = context->compareIntervals.at(0).data->dataPoints().at(context->compareIntervals.at(0).data->timeIndex(point->secs));
+                            x = pointType(refPoint, settings->x, side, context->athlete->useMetricUnits, cranklength);
+                            y = pointType(point, settings->y, side, context->athlete->useMetricUnits, cranklength);
+                        }
+                        else if (settings->compareMode == 2) {
+                            const RideFilePoint *refPoint = context->compareIntervals.at(0).data->dataPoints().at(context->compareIntervals.at(0).data->timeIndex(point->secs));
+                            x = pointType(point, settings->x, side, context->athlete->useMetricUnits, cranklength);
+                            y = pointType(refPoint, settings->y, side, context->athlete->useMetricUnits, cranklength);
+                        }
+                        else {
+                            x = pointType(point, settings->x, side, context->athlete->useMetricUnits, cranklength);
+                            y = pointType(point, settings->y, side, context->athlete->useMetricUnits, cranklength);
+                        }
+
+
+                        // skip values ?
+                        if (skipValues(x, y, settings))
+                            continue;
+
+                        if (y > maxY) maxY = y;
+                        if (y < minY) minY = y;
+                        if (x > maxX) maxX = x;
+                        if (x < minX) minX = x;
+
+                        xval.append(x);
+                        yval.append(y);
+                        nbPoints++;
+                    }
+
+                    smooth(xval, yval, nbPoints, settings->smoothing);
+
+                    QColor intervalColor = interval.color;
+                    // left / right are darker lighter
+                    if (side) intervalColor = intervalColor.lighter(50);
+
+                    QwtSymbol *sym = new QwtSymbol;
+                    sym->setStyle(QwtSymbol::Ellipse);
+                    sym->setSize(4);
+                    sym->setBrush(QBrush(intervalColor));
+                    sym->setPen(QPen(intervalColor));
+
+                    QwtPlotCurve *ic = new QwtPlotCurve();
+
+                    ic->setSymbol(sym);
+                    ic->setStyle(QwtPlotCurve::Dots);
+                    ic->setRenderHint(QwtPlotItem::RenderAntialiased);
+                    ic->setSamples(xval.constData(), yval.constData(), nbPoints);
+                    ic->attach(this);
+
+                    intervalCurves.append(ic);
+
+                    if (settings->trendLine>0)  {
+                        addTrendLine(xval, yval, nbPoints, intervalColor);
+                    }
+                }
             }
         }
     }
@@ -433,6 +574,10 @@ void ScatterPlot::setData (ScatterSettings *settings)
     if (settings->x == MODEL_CPV) setAxisScale(xBottom, 0, 3);
     else setAxisScale(xBottom, minX, maxX);
 
+    // gear
+    if (settings->x == MODEL_GEAR) setAxisScale(xBottom, 0, 7);
+    if (settings->y == MODEL_GEAR) setAxisScale(yLeft, 0, 7);
+
 
     // and those interval markers
     refreshIntervalMarkers(settings);
@@ -445,7 +590,7 @@ void ScatterPlot::mouseMoved()
 {
     if (!isVisible()) return;
 
-    if (ride && ride->ride() && ride->ride()->intervals().count() >= intervalMarkers.count()) {
+    if (ride && ride->ride() && ride->intervals().count() >= intervalMarkers.count()) {
 
         // where is the mouse ?
         QPoint pos = QCursor::pos();
@@ -460,7 +605,7 @@ void ScatterPlot::mouseMoved()
             int dy = mpos.y() - pos.y();
 
             if ((dx > -6 && dx < 6) && (dy > -6 && dy < 6))
-                context->notifyIntervalHover(ride->ride()->intervals()[index]);
+                context->notifyIntervalHover(ride->intervals()[index]);
 
             index++;
         }
@@ -469,7 +614,7 @@ void ScatterPlot::mouseMoved()
 }
 
 void
-ScatterPlot::intervalHover(RideFileInterval ri)
+ScatterPlot::intervalHover(IntervalItem *ri)
 {
     if (!isVisible()) return;
     if (context->isCompareIntervals) return;
@@ -488,6 +633,9 @@ ScatterPlot::intervalHover(RideFileInterval ri)
         hover2 = NULL;
     }
 
+    // null so just clear hover
+    if (!ri) return;
+
     // how many curves do we need ?
     if (xseries == MODEL_LRBALANCE || xseries == MODEL_TE || xseries == MODEL_PS ||
         yseries == MODEL_LRBALANCE || yseries == MODEL_TE || yseries == MODEL_PS) {
@@ -509,7 +657,7 @@ ScatterPlot::intervalHover(RideFileInterval ri)
             double y = pointType(p1, xseries, side, context->athlete->useMetricUnits, cranklength);
             double x = pointType(p1, yseries, side, context->athlete->useMetricUnits, cranklength);
 
-            if (p1->secs < ri.start || p1->secs > ri.stop) continue;
+            if (p1->secs < ri->start || p1->secs > ri->stop) continue;
 
             xArray << x;
             yArray << y;
@@ -518,8 +666,8 @@ ScatterPlot::intervalHover(RideFileInterval ri)
         // which interval is it or how many ?
         int count = 0;
         int ours = 0;
-        foreach(RideFileInterval p, ride->ride()->intervals()) {
-            if (p.start == ri.start && p.stop == ri.stop) ours = count;
+        foreach(IntervalItem *p, ride->intervals()) {
+            if (p->start == ri->start && p->stop == ri->stop) ours = count;
             count++;
         }
 
@@ -572,10 +720,12 @@ ScatterPlot::refreshIntervalMarkers(ScatterSettings *settings)
     }
     intervalMarkers.clear();
 
+    if (context->isCompareIntervals) return;
+
     // do we have a ride with intervals to refresh ?
     int count=0;
 
-    if (settings->ride && settings->ride->ride() && settings->ride->ride()->dataPoints().count() && (count = settings->ride->ride()->intervals().count())) {
+    if (settings->ride && settings->ride->ride() && settings->ride->ride()->dataPoints().count() && (count = settings->ride->intervals().count())) {
 
         // accumulating...
         QVector<saccum> intervalAccumulator(count);
@@ -594,10 +744,10 @@ ScatterPlot::refreshIntervalMarkers(ScatterSettings *settings)
             // accumulate values for each interval here ....
             for(int i=0; i < count; i++) {
 
-                RideFileInterval v = settings->ride->ride()->intervals()[i];
+                IntervalItem *v = settings->ride->intervals()[i];
 
                 // in our interval ?
-                if (point->secs >= v.start && point->secs <= v.stop) {
+                if (point->secs >= v->start && point->secs <= v->stop) {
                     intervalAccumulator[i].x += x;
                     intervalAccumulator[i].y += y;
                     intervalAccumulator[i].count++;
@@ -634,7 +784,7 @@ ScatterPlot::refreshIntervalMarkers(ScatterSettings *settings)
 }
 
 void
-ScatterPlot::configChanged()
+ScatterPlot::configChanged(qint32)
 {
     // setColors bg
     setCanvasBackground(GColor(CPLOTBACKGROUND));
@@ -663,3 +813,171 @@ ScatterPlot::setAxisTitle(int axis, QString label)
     QwtPlot::setAxisFont(axis, stGiles);
     QwtPlot::setAxisTitle(axis, title);
 }
+
+void
+ScatterPlot::addTrendLine(QVector<double> xval, QVector<double> yval, int nbPoints, QColor intervalColor)
+{
+    QwtPlotCurve *trend = new QwtPlotCurve();
+
+    // cosmetics
+    QPen cpen = QPen(intervalColor.darker(200));
+    cpen.setWidth(2); // double thickness for trend lines
+    cpen.setStyle(Qt::SolidLine);
+    trend->setPen(cpen);
+    if (appsettings->value(this, GC_ANTIALIAS, true).toBool()==true)
+        trend->setRenderHint(QwtPlotItem::RenderAntialiased);
+    trend->setBaseline(0);
+    trend->setYAxis(yLeft);
+    trend->setStyle(QwtPlotCurve::Lines);
+
+    // perform linear regression
+    Statistic regress(xval.data(), yval.data(), nbPoints);
+    double xtrend[2], ytrend[2];
+    xtrend[0] = 0.0;
+    ytrend[0] = regress.getYforX(0.0);
+    // point 2 is at far right of chart, not the last point
+    // since we may be forecasting...
+    xtrend[1] = regress.maxX;
+    ytrend[1] = regress.getYforX(regress.maxX);
+    trend->setSamples(xtrend,ytrend, 2);
+
+    trend->attach(this);
+
+    intervalCurves.append(trend);
+
+    // make that mark -- always above with topN
+    QwtPlotMarker *label = new QwtPlotMarker();
+    QString labelString = regress.label();
+    QwtText text(labelString);
+    text.setColor(intervalColor.darker(200));
+    label->setLabel(text);
+    label->setValue(xtrend[1]*0.8, ytrend[1]*0.9);
+    label->setYAxis(yLeft);
+    label->setSpacing(6); // not px but by yaxis value !? mad.
+    label->setLabelAlignment(Qt::AlignTop | Qt::AlignCenter);
+
+    // and attach
+    label->attach(this);
+    labels << label;
+}
+
+void
+ScatterPlot::smooth(QVector<double> &xval, QVector<double> &yval, int count, int applySmooth)
+{
+    if (applySmooth<2 || count < applySmooth)
+        return;
+
+    QVector<double> smoothX(count);
+    QVector<double> smoothY(count);
+
+    for (int i=0; i<count; i++) {
+        smoothX[i] = xval.value(i);
+        smoothY[i] = yval.value(i);
+    }
+
+    // initialise rolling average
+    double rXtot = 0;
+    double rYtot = 0;
+
+    for (int i=applySmooth; i>0 && count-i >=0; i--) {
+        rXtot += smoothX[count-i];
+        rYtot += smoothY[count-i];
+    }
+
+    // now run backwards setting the rolling average
+    for (int i=count-1; i>=applySmooth; i--) {
+        int hereX = smoothX[i];
+        smoothX[i] = rXtot / applySmooth;
+        rXtot -= hereX;
+        rXtot += smoothX[i-applySmooth];
+
+        int hereY = smoothY[i];
+        smoothY[i] = rYtot / applySmooth;
+        rYtot -= hereY;
+        rYtot += smoothY[i-applySmooth];
+    }
+
+    // Finish with smaller rolling average
+    for (int i=applySmooth-1; i>0; i--) {
+        int hereX = smoothX[i];
+        smoothX[i] = rXtot / (i+2);
+        rXtot -= hereX;
+
+        int hereY = smoothY[i];
+        smoothY[i] = rYtot / (i+2);
+        rYtot -= hereY;
+    }
+
+    xval = smoothX;
+    yval = smoothY;
+}
+
+void
+ScatterPlot::resample(QVector<double> &xval, QVector<double> &yval, int &count, double recInterval, int applySmooth)
+{
+    if (recInterval >= applySmooth)
+        return;
+
+
+    int newcount=0;
+    QVector<double> newxval;
+    QVector<double> newyval;
+
+    if (applySmooth > recInterval) {
+        double totalX = 0.0;
+        double totalY = 0.0;
+
+        int j=0;
+
+        for (int i = 0; i < count; i++) {
+            j++;
+
+            if (j<applySmooth) {
+                totalX += xval.at(i)*recInterval;
+                totalY += yval.at(i)*recInterval;
+            }
+            else {
+                double part = recInterval-j+applySmooth;
+                totalX += xval.at(i)*part;
+                totalY += yval.at(i)*part;
+
+                double smoothX = totalX/applySmooth;
+                double smoothY = totalY/applySmooth;
+
+                newxval.append(smoothX);
+                newyval.append(smoothY);
+                newcount++;
+
+                j=0;
+                totalX = xval.at(i)*(recInterval-part);
+                totalY = yval.at(i)*(recInterval-part);
+            }
+        }
+    }
+
+    count = newcount;
+    xval = newxval;
+    yval = newyval;
+}
+
+bool
+ScatterPlot::skipValues(double xv, double yv, ScatterSettings *settings) {
+
+    // skip zeroes? - special logic for Model Gear, since there value between 0.01 and 1 happen and are relevant
+    if ((settings->x != MODEL_GEAR && settings->y != MODEL_GEAR)
+         && settings->ignore && (int(xv) == 0 || int(yv) == 0)) return true;
+
+    // Model Gear
+    if ((settings->x == MODEL_GEAR)
+         && settings->ignore && (xv == 0.0f || int(yv) == 0)) return true;
+    if ((settings->y == MODEL_GEAR)
+         && settings->ignore && (int(xv) == 0 || yv == 0.0f)) return true;
+
+    // Temp 0 values are relevant
+    if ((settings->x == MODEL_TEMP && xv == RideFile::NoTemp) || (settings->y == MODEL_TEMP && yv == RideFile::NoTemp)) return true;
+
+    // LR Balance : if skip 0% skip also 100%
+    if ((settings->x == MODEL_LRBALANCE && settings->ignore && xv == 100) || (settings->y == MODEL_LRBALANCE && settings->ignore && yv == 100)) return true;
+
+    return false;
+}
