@@ -24,15 +24,30 @@
 #include "mvjson.h"
 #include "TimeUtils.h"
 #include "Units.h"
+#include "VeloHeroUploader.h"
+#include "TrainingstagebuchUploader.h"
+#include "HelpWhatsThis.h"
 
-// acccess to metrics
-#include "MetricAggregator.h"
+// access to metrics
 #include "RideMetric.h"
-#include "DBAccess.h"
 #include "TcxRideFile.h"
 
 #include <zlib.h>
 
+bool
+ShareDialogUploader::canUpload( QString &err )
+{
+    (void)err;
+
+    return true;
+}
+
+bool
+ShareDialogUploader::wasUploaded()
+{
+    return false;
+}
+
 //
 // Utility function to create a QByteArray of data in GZIP format
 // This is essentially the same as qCompress but creates it in
@@ -72,7 +87,9 @@ static QByteArray zCompress(const QByteArray &source)
 ShareDialog::ShareDialog(Context *context, RideItem *item) :
     context(context)
 {
-    setWindowTitle(tr("Share your ride"));
+    setWindowTitle(tr("Share your activity"));
+    HelpWhatsThis *help = new HelpWhatsThis(this);
+    this->setWhatsThis(help->getWhatsThisText(HelpWhatsThis::MenuBar_Activity_Share));
 
     // make the dialog a resonable size
     setMinimumWidth(550);
@@ -80,44 +97,115 @@ ShareDialog::ShareDialog(Context *context, RideItem *item) :
 
     ride = item;
 
-    // uploaders
-    stravaUploader = new StravaUploader(context, ride, this);
-    rideWithGpsUploader = new RideWithGpsUploader(context, ride, this);
-    cyclingAnalyticsUploader = new CyclingAnalyticsUploader(context, ride, this);
-    selfLoopsUploader = new SelfLoopsUploader(context, ride, this);
-    //garminUploader = new GarminUploader(context, ride, this); // not in 3.1
-
     QVBoxLayout *mainLayout = new QVBoxLayout(this);
     QGroupBox *groupBox1 = new QGroupBox(tr("Choose which sites you wish to share on: "));
 
+    QGridLayout *vbox1 = new QGridLayout();
+    unsigned int col = 0;
+
+    QString err;
+
+    stravaUploader = new StravaUploader(context, ride, this);
     stravaChk = new QCheckBox(tr("Strava"));
-#ifndef GC_STRAVA_CLIENT_SECRET
-    stravaChk->setEnabled(false);
-#endif
+    if( ! stravaUploader->canUpload( err ) ){
+        stravaChk->setEnabled( false );
+    } else if( ! stravaUploader->wasUploaded() ){
+        stravaChk->setChecked( true );
+    }
+    vbox1->addWidget(stravaChk,0,col++);
+
+    rideWithGpsUploader = new RideWithGpsUploader(context, ride, this);
     rideWithGPSChk = new QCheckBox(tr("Ride With GPS"));
+    if( ! rideWithGpsUploader->canUpload( err ) ){
+        rideWithGPSChk->setEnabled( false );
+    } else if( ! rideWithGpsUploader->wasUploaded() ){
+        rideWithGPSChk->setChecked( true );
+    }
+    vbox1->addWidget(rideWithGPSChk,0,col++);
+
+    cyclingAnalyticsUploader = new CyclingAnalyticsUploader(context, ride, this);
     cyclingAnalyticsChk = new QCheckBox(tr("Cycling Analytics"));
-#ifndef GC_CYCLINGANALYTICS_CLIENT_SECRET
-    cyclingAnalyticsChk->setEnabled(false);
-#endif
+    if( ! cyclingAnalyticsUploader->canUpload( err ) ){
+        cyclingAnalyticsChk->setEnabled( false );
+    } else if( ! cyclingAnalyticsUploader->wasUploaded() ){
+        cyclingAnalyticsChk->setChecked( true );
+    }
+    vbox1->addWidget(cyclingAnalyticsChk,0,col++);
+
+    selfLoopsUploader = new SelfLoopsUploader(context, ride, this);
     selfLoopsChk = new QCheckBox(tr("Selfloops"));
+    if( ! selfLoopsUploader->canUpload( err ) ){
+        selfLoopsChk->setEnabled( false );
+    } else if( ! selfLoopsUploader->wasUploaded() ){
+        selfLoopsChk->setChecked( true );
+    }
+    vbox1->addWidget(selfLoopsChk,0,col++);
+
+    veloHeroUploader = new VeloHeroUploader(context, ride, this);
+    veloHeroChk = new QCheckBox(tr("VeloHero"));
+    if( ! veloHeroUploader->canUpload( err ) ){
+        veloHeroChk->setEnabled( false );
+    } else if( ! veloHeroUploader->wasUploaded() ){
+        veloHeroChk->setChecked( true );
+    }
+    vbox1->addWidget(veloHeroChk,0,col++);
+
+    trainingstagebuchUploader = new TrainingstagebuchUploader(context, ride, this);
+    trainingstagebuchChk = new QCheckBox(tr("Trainingstagebuch.org"));
+    if( ! trainingstagebuchUploader->canUpload( err ) ){
+        trainingstagebuchChk->setEnabled( false );
+    } else if( ! trainingstagebuchUploader->wasUploaded() ){
+        trainingstagebuchChk->setChecked( true );
+    }
+    vbox1->addWidget(trainingstagebuchChk,0,col++);
+
+    //garminUploader = new GarminUploader(context, ride, this); // not in 3.1
     //garminChk = new QCheckBox(tr("Garmin Connect"));
     //garminChk->setVisible(false);
-
-    QGridLayout *vbox1 = new QGridLayout();
-    vbox1->addWidget(stravaChk,0,0);
-    vbox1->addWidget(rideWithGPSChk,0,1);
-    vbox1->addWidget(cyclingAnalyticsChk,0,2);
-    vbox1->addWidget(selfLoopsChk,0,3);
-    //vbox1->addWidget(garminChk,0,4);
+    //if( ! garminUploader->canUpload( err ) ){
+    //    garminChk->setEnabled( false );
+    //}
+    //if( ! garminUploader->wasUploaded() ){
+    //    garminChk->setChecked( true );
+    //}
+    //vbox1->addWidget(garminChk,0,col++);
 
 
     groupBox1->setLayout(vbox1);
     mainLayout->addWidget(groupBox1);
 
-    QGroupBox *groupBox2 = new QGroupBox(tr("Choose a name for your ride: "));
+    QGroupBox *groupBox2 = new QGroupBox(tr("Choose a name for your activity: "));
 
     titleEdit = new QLineEdit();
 
+    // If we have a ride, and it can be opened then lets set the default for the title
+    // We try metadata fields; Title, then Name then Route then Workout Code
+    if (ride && ride->ride()) {
+
+        // is "Title" set?
+        if (!ride->ride()->getTag("Title", "").isEmpty()) {
+            titleEdit->setText(ride->ride()->getTag("Title", ""));
+        } else {
+
+            // is "Name" set?
+            if (!ride->ride()->getTag("Name", "").isEmpty()) {
+                titleEdit->setText(ride->ride()->getTag("Name", ""));
+            } else {
+
+                // is "Route" set?
+                if (!ride->ride()->getTag("Route", "").isEmpty()) {
+                    titleEdit->setText(ride->ride()->getTag("Route", ""));
+                } else {
+
+                    //  is Workout Code set?
+                    if (!ride->ride()->getTag("Workout Code", "").isEmpty()) {
+                        titleEdit->setText(ride->ride()->getTag("Workout Code", ""));
+                    }
+                }
+            }
+        }
+    }
+
     QGridLayout *vbox2 = new QGridLayout();
     vbox2->addWidget(titleEdit,0,0);
 
@@ -164,7 +252,7 @@ ShareDialog::ShareDialog(Context *context, RideItem *item) :
 
     QHBoxLayout *buttonLayout = new QHBoxLayout;
 
-    uploadButton = new QPushButton(tr("&Upload Ride"), this);
+    uploadButton = new QPushButton(tr("&Upload Activity"), this);
     buttonLayout->addWidget(uploadButton);
     closeButton = new QPushButton(tr("&Close"), this);
     buttonLayout->addStretch();
@@ -183,8 +271,9 @@ ShareDialog::upload()
 {
     show();
 
-    if (!stravaChk->isChecked() && !rideWithGPSChk->isChecked() &&
-        !cyclingAnalyticsChk->isChecked() && !selfLoopsChk->isChecked() 
+    if ( !rideWithGPSChk->isChecked() && !selfLoopsChk->isChecked()
+        && !veloHeroChk->isChecked() && !trainingstagebuchChk->isChecked()
+        && !stravaChk->isChecked() && !cyclingAnalyticsChk->isChecked()
         //&& !garminChk->isChecked()
         ) {
         QMessageBox aMsgBox;
@@ -193,6 +282,8 @@ ShareDialog::upload()
         return;
     }
 
+    uploadButton->setEnabled(false);
+
     shareSiteCount = 0;
     progressBar->setValue(0);
     progressLabel->setText("");
@@ -210,59 +301,69 @@ ShareDialog::upload()
     if (selfLoopsChk->isChecked()) {
         shareSiteCount ++;
     }
+    if (veloHeroChk->isChecked()) {
+        shareSiteCount ++;
+    }
+    if (trainingstagebuchChk->isChecked()) {
+        shareSiteCount ++;
+    }
     //if (garminChk->isChecked()) {
     //    shareSiteCount ++;
     //}
 
-    if (stravaChk->isChecked()) {
-        stravaUploader->upload();
+    if (stravaChk->isEnabled() && stravaChk->isChecked()) {
+        doUploader( stravaUploader );
     }
-    if (rideWithGPSChk->isChecked()) {
-        rideWithGpsUploader->upload();
+    if (rideWithGPSChk->isEnabled() && rideWithGPSChk->isChecked()) {
+        doUploader( rideWithGpsUploader );
     }
-    if (cyclingAnalyticsChk->isChecked()) {
-        cyclingAnalyticsUploader->upload();
+    if (cyclingAnalyticsChk->isEnabled() && cyclingAnalyticsChk->isChecked()) {
+        doUploader( cyclingAnalyticsUploader );
     }
-    if (selfLoopsChk->isChecked()) {
-        selfLoopsUploader->upload();
+    if (selfLoopsChk->isEnabled() && selfLoopsChk->isChecked()) {
+        doUploader( selfLoopsUploader );
     }
-    //if (garminChk->isChecked()) {
-    //    garminUploader->upload();
+    if (veloHeroChk->isEnabled() && veloHeroChk->isChecked()) {
+        doUploader( veloHeroUploader );
+    }
+    if (trainingstagebuchChk->isEnabled() && trainingstagebuchChk->isChecked()) {
+        doUploader( trainingstagebuchUploader );
+    }
+    //if (garminChk->isEnabled() && garminChk->isChecked()) {
+    //    doUploader( garminUploader );
     //}
-}
 
-StravaUploader::StravaUploader(Context *context, RideItem *ride, ShareDialog *parent) :
-    context(context), ride(ride), parent(parent)
-{
-    stravaUploadId = ride->ride()->getTag("Strava uploadId", "0").toInt();
-    eventLoop = new QEventLoop(this);
-    networkManager = new QNetworkAccessManager(this);
+    uploadButton->setEnabled(true);
 }
 
 void
-StravaUploader::upload()
+ShareDialog::okClicked()
 {
-    // OAuth no more login
-    token = appsettings->cvalue(context->athlete->cyclist, GC_STRAVA_TOKEN, "").toString();
-    if (token=="")
-    {
-        QMessageBox aMsgBox;
-        aMsgBox.setText(tr("Cannot login to Strava. Check permission"));
-        aMsgBox.exec();
-        return;
-    }
+    dialog->accept();
+    return;
+}
 
-    // already shared ?
-    if(stravaUploadId>0)
-    {
-        overwrite = false;
+void
+ShareDialog::closeClicked()
+{
+    dialog->reject();
+    return;
+}
 
+
+void
+ShareDialog::doUploader( ShareDialogUploader *uploader )
+{
+    assert(uploader);
+
+    if( uploader->wasUploaded() ){
         dialog = new QDialog();
         QVBoxLayout *layout = new QVBoxLayout;
 
         QVBoxLayout *layoutLabel = new QVBoxLayout();
         QLabel *label = new QLabel();
-        label->setText(tr("This Ride is marked as already on Strava. Are you sure you want to upload it?"));
+        label->setText(tr("This activity is marked as already on %1. Are you sure you want to upload it?")
+            .arg(uploader->name()) );
         layoutLabel->addWidget(label);
 
         QPushButton *ok = new QPushButton(tr("OK"), dialog);
@@ -283,6 +384,46 @@ StravaUploader::upload()
         if (!dialog->exec()) return;
     }
 
+    uploader->upload();
+}
+
+StravaUploader::StravaUploader(Context *context, RideItem *ride, ShareDialog *parent) :
+    ShareDialogUploader( tr("Strava"), context, ride, parent)
+{
+    stravaUploadId = ride->ride()->getTag("Strava uploadId", "0").toInt();
+    eventLoop = new QEventLoop(this);
+    networkManager = new QNetworkAccessManager(this);
+}
+
+bool
+StravaUploader::canUpload( QString &err )
+{
+#ifdef GC_STRAVA_CLIENT_SECRET
+   token = appsettings->cvalue(context->athlete->cyclist, GC_STRAVA_TOKEN, "").toString();
+   if( token!="" )
+        return true;
+
+    err = tr("no Strava token set. Please authorize in Settings.");
+#else
+    err = tr("Strava support isn't enabled in this build");
+#endif
+    return false;
+}
+
+bool
+StravaUploader::wasUploaded()
+{
+    return stravaUploadId>0;
+}
+
+void
+StravaUploader::upload()
+{
+    // OAuth no more login
+    token = appsettings->cvalue(context->athlete->cyclist, GC_STRAVA_TOKEN, "").toString();
+    if (token=="")
+        return;
+
     requestUploadStrava();
 
     if(!uploadSuccessful)
@@ -301,7 +442,7 @@ StravaUploader::upload()
 void
 StravaUploader::requestUploadStrava()
 {
-    parent->progressLabel->setText(tr("Upload ride to Strava..."));
+    parent->progressLabel->setText(tr("Upload activity to Strava..."));
     parent->progressBar->setValue(parent->progressBar->value()+10/parent->shareSiteCount);
 
     int year = ride->fileName.left(4).toInt();
@@ -376,7 +517,7 @@ StravaUploader::requestUploadStrava()
     networkManager->post(request, multiPart);
 
     parent->progressBar->setValue(parent->progressBar->value()+30/parent->shareSiteCount);
-    parent->progressLabel->setText(tr("Upload ride... Sending to Strava"));
+    parent->progressLabel->setText(tr("Upload... Sending to Strava"));
 
     eventLoop->exec();
 }
@@ -392,8 +533,8 @@ StravaUploader::requestUploadStravaFinished(QNetworkReply *reply)
     QString response = reply->readLine();
     //qDebug() << response;
 
-    // use a lightweigth json parser to do this
-    QString uploadError="invalid reponse or parser error";
+    // use a lightweight json parser to do this
+    QString uploadError="invalid response or parser error";
     try {
 
         // parse !
@@ -444,7 +585,7 @@ void
 StravaUploader::requestVerifyUpload()
 {
     parent->progressBar->setValue(0);
-    parent->progressLabel->setText(tr("Ride processing..."));
+    parent->progressLabel->setText(tr("Processing..."));
 
     // reconnect for verify
     networkManager->disconnect();
@@ -505,59 +646,32 @@ StravaUploader::requestVerifyUploadFinished(QNetworkReply *reply)
     }
 }
 
-void
-StravaUploader::okClicked()
-{
-    dialog->accept();
-    return;
-}
-
-void
-StravaUploader::closeClicked()
-{
-    dialog->reject();
-    return;
-}
-
 RideWithGpsUploader::RideWithGpsUploader(Context *context, RideItem *ride, ShareDialog *parent) :
-    context(context), ride(ride), parent(parent)
+    ShareDialogUploader( tr("Ride With GPS"), context, ride, parent)
 {
     rideWithGpsActivityId = ride->ride()->getTag("RideWithGPS tripid", "");
 }
 
+bool
+RideWithGpsUploader::canUpload( QString &err )
+{
+    QString username = appsettings->cvalue(context->athlete->cyclist, GC_RWGPSUSER).toString();
+    if( username.length() > 0 )
+        return true;
+
+    err = tr("no credentials set for RideWithGps. Please check Settings.");
+    return false;
+}
+
+bool
+RideWithGpsUploader::wasUploaded()
+{
+    return rideWithGpsActivityId.length()>0;
+}
+
 void
 RideWithGpsUploader::upload()
 {
-    if(rideWithGpsActivityId.length()>0)
-    {
-        overwrite = false;
-
-        dialog = new QDialog();
-        QVBoxLayout *layout = new QVBoxLayout;
-
-        QVBoxLayout *layoutLabel = new QVBoxLayout();
-        QLabel *label = new QLabel();
-        label->setText(tr("This Ride is marked as already on RideWithGPS. Are you sure you want to upload it?"));
-        layoutLabel->addWidget(label);
-
-        QPushButton *ok = new QPushButton(tr("OK"), dialog);
-        QPushButton *cancel = new QPushButton(tr("Cancel"), dialog);
-        QHBoxLayout *buttons = new QHBoxLayout();
-        buttons->addStretch();
-        buttons->addWidget(cancel);
-        buttons->addWidget(ok);
-
-        connect(ok, SIGNAL(clicked()), this, SLOT(okClicked()));
-        connect(cancel, SIGNAL(clicked()), this, SLOT(closeClicked()));
-
-        layout->addLayout(layoutLabel);
-        layout->addLayout(buttons);
-
-        dialog->setLayout(layout);
-
-        if (!dialog->exec()) return;
-    }
-
     requestUploadRideWithGPS();
 
     if(!uploadSuccessful)
@@ -573,7 +687,7 @@ RideWithGpsUploader::upload()
 void
 RideWithGpsUploader::requestUploadRideWithGPS()
 {
-    parent->progressLabel->setText(tr("Upload ride..."));
+    parent->progressLabel->setText(tr("Upload..."));
     parent->progressBar->setValue(parent->progressBar->value()+10/parent->shareSiteCount);
 
     QEventLoop eventLoop;
@@ -661,7 +775,7 @@ RideWithGpsUploader::requestUploadRideWithGPS()
     request.setHeader(QNetworkRequest::ContentTypeHeader, "application/json");
 
     parent->progressBar->setValue(parent->progressBar->value()+30/parent->shareSiteCount);
-    parent->progressLabel->setText(tr("Upload ride... Sending to RideWithGPS"));
+    parent->progressLabel->setText(tr("Upload... Sending to RideWithGPS"));
 
     networkMgr.post( request, out.toLatin1());
     eventLoop.exec();
@@ -715,69 +829,40 @@ RideWithGpsUploader::requestUploadRideWithGPSFinished(QNetworkReply *reply)
     }
 }
 
-void
-RideWithGpsUploader::okClicked()
-{
-    dialog->accept();
-    return;
-}
-
-void
-RideWithGpsUploader::closeClicked()
-{
-    dialog->reject();
-    return;
-}
-
 CyclingAnalyticsUploader::CyclingAnalyticsUploader(Context *context, RideItem *ride, ShareDialog *parent) :
-    context(context), ride(ride), parent(parent)
+    ShareDialogUploader(tr("CyclingAnalytics"), context, ride, parent)
 {
     cyclingAnalyticsUploadId = ride->ride()->getTag("CyclingAnalytics uploadId", "0").toInt();
 }
 
+bool
+CyclingAnalyticsUploader::canUpload( QString &err )
+{
+#ifdef GC_CYCLINGANALYTICS_CLIENT_SECRET
+    token = appsettings->cvalue(context->athlete->cyclist, GC_CYCLINGANALYTICS_TOKEN, "").toString();
+   if( token!="" )
+        return true;
+
+    err = tr("no CyclingAnalytics token set. Please authorize in Settings.");
+#else
+    err = tr("CyclingAnalytics support isn't enabled in this build");
+#endif
+    return false;
+}
+
+bool
+CyclingAnalyticsUploader::wasUploaded()
+{
+    return cyclingAnalyticsUploadId>0;
+}
+
 void
 CyclingAnalyticsUploader::upload()
 {
     // OAuth no more login
     token = appsettings->cvalue(context->athlete->cyclist, GC_CYCLINGANALYTICS_TOKEN, "").toString();
-    if (token=="") {
-
-        QMessageBox aMsgBox;
-        aMsgBox.setText(tr("Cannot login to CyclingAnalytics. Check permission"));
-        aMsgBox.exec();
+    if (token=="")
         return;
-    }
-
-    // already shared ?
-    if(cyclingAnalyticsUploadId>0) {
-
-        overwrite = false;
-
-        dialog = new QDialog();
-        QVBoxLayout *layout = new QVBoxLayout;
-
-        QVBoxLayout *layoutLabel = new QVBoxLayout();
-        QLabel *label = new QLabel();
-        label->setText(tr("This Ride is marked as already on CyclingAnalytics. Are you sure you want to upload it?"));
-        layoutLabel->addWidget(label);
-
-        QPushButton *ok = new QPushButton(tr("OK"), dialog);
-        QPushButton *cancel = new QPushButton(tr("Cancel"), dialog);
-        QHBoxLayout *buttons = new QHBoxLayout();
-        buttons->addStretch();
-        buttons->addWidget(cancel);
-        buttons->addWidget(ok);
-
-        connect(ok, SIGNAL(clicked()), this, SLOT(okClicked()));
-        connect(cancel, SIGNAL(clicked()), this, SLOT(closeClicked()));
-
-        layout->addLayout(layoutLabel);
-        layout->addLayout(buttons);
-
-        dialog->setLayout(layout);
-
-        if (!dialog->exec()) return;
-    }
 
     requestUploadCyclingAnalytics();
 
@@ -794,7 +879,7 @@ CyclingAnalyticsUploader::upload()
 void
 CyclingAnalyticsUploader::requestUploadCyclingAnalytics()
 {
-    parent->progressLabel->setText(tr("Upload ride to CyclingAnalytics..."));
+    parent->progressLabel->setText(tr("Upload to CyclingAnalytics..."));
     parent->progressBar->setValue(parent->progressBar->value()+10/parent->shareSiteCount);
 
     QEventLoop eventLoop;
@@ -842,7 +927,7 @@ CyclingAnalyticsUploader::requestUploadCyclingAnalytics()
     networkMgr.post(request, multiPart);
 
     parent->progressBar->setValue(parent->progressBar->value()+30/parent->shareSiteCount);
-    parent->progressLabel->setText(tr("Upload ride... Sending to CyclingAnalytics"));
+    parent->progressLabel->setText(tr("Upload... Sending to CyclingAnalytics"));
 
     eventLoop.exec();
 }
@@ -893,62 +978,33 @@ CyclingAnalyticsUploader::requestUploadCyclingAnalyticsFinished(QNetworkReply *r
     }
 }
 
-void
-CyclingAnalyticsUploader::okClicked()
-{
-    dialog->accept();
-    return;
-}
-
-void
-CyclingAnalyticsUploader::closeClicked()
-{
-    dialog->reject();
-    return;
-}
-
-
 SelfLoopsUploader::SelfLoopsUploader(Context *context, RideItem *ride, ShareDialog *parent) :
-    context(context), ride(ride), parent(parent)
+    ShareDialogUploader(tr("SelfLoops"),context, ride, parent)
 {
     selfloopsUploadId = ride->ride()->getTag("Selfloops uploadId", "0").toInt();
     selfloopsActivityId = ride->ride()->getTag("Selfloops activityId", "0").toInt();
 }
 
+bool
+SelfLoopsUploader::canUpload( QString &err )
+{
+    QString username = appsettings->cvalue(context->athlete->cyclist, GC_SELUSER).toString();
+    if( username.length() > 0 )
+        return true;
+
+    err = tr("no credentials set for SelfLoops. Please check Settings.");
+    return false;
+}
+
+bool
+SelfLoopsUploader::wasUploaded()
+{
+    return selfloopsActivityId>0;
+}
+
 void
 SelfLoopsUploader::upload()
 {
-    // allready shared ?
-    if(selfloopsActivityId>0) {
-
-        overwrite = false;
-
-        dialog = new QDialog();
-        QVBoxLayout *layout = new QVBoxLayout;
-
-        QVBoxLayout *layoutLabel = new QVBoxLayout();
-        QLabel *label = new QLabel();
-        label->setText(tr("This Ride is marked as already on Selfloops. Are you sure you want to upload it?"));
-        layoutLabel->addWidget(label);
-
-        QPushButton *ok = new QPushButton(tr("OK"), dialog);
-        QPushButton *cancel = new QPushButton(tr("Cancel"), dialog);
-        QHBoxLayout *buttons = new QHBoxLayout();
-        buttons->addStretch();
-        buttons->addWidget(cancel);
-        buttons->addWidget(ok);
-
-        connect(ok, SIGNAL(clicked()), this, SLOT(okClicked()));
-        connect(cancel, SIGNAL(clicked()), this, SLOT(closeClicked()));
-
-        layout->addLayout(layoutLabel);
-        layout->addLayout(buttons);
-
-        dialog->setLayout(layout);
-
-        if (!dialog->exec()) return;
-    }
-
     requestUploadSelfLoops();
 
     if(!uploadSuccessful)
@@ -976,7 +1032,7 @@ SelfLoopsUploader::upload()
 void
 SelfLoopsUploader::requestUploadSelfLoops()
 {
-    parent->progressLabel->setText(tr("Upload ride to Selfloops..."));
+    parent->progressLabel->setText(tr("Upload to Selfloops..."));
     parent->progressBar->setValue(parent->progressBar->value()+10/parent->shareSiteCount);
 
     QEventLoop eventLoop;
@@ -1022,7 +1078,7 @@ SelfLoopsUploader::requestUploadSelfLoops()
     networkMgr.post(request, multiPart);
 
     parent->progressBar->setValue(parent->progressBar->value()+30/parent->shareSiteCount);
-    parent->progressLabel->setText(tr("Upload ride... Sending to Selfloops"));
+    parent->progressLabel->setText(tr("Upload... Sending to Selfloops"));
 
     eventLoop.exec();
 }
@@ -1073,66 +1129,27 @@ SelfLoopsUploader::requestUploadSelfLoopsFinished(QNetworkReply *reply)
     }
 }
 
-void
-SelfLoopsUploader::okClicked()
-{
-    dialog->accept();
-    return;
-}
-
-void
-SelfLoopsUploader::closeClicked()
-{
-    dialog->reject();
-    return;
-}
-
 #if 0 // NOT AVAILABLE -- COMMENTED OUT FOR VERSION 3.1
 /******************/
 /* Garmin Connect */
 /******************/
 
 GarminUploader::GarminUploader(Context *context, RideItem *ride, ShareDialog *parent) :
-    context(context), ride(ride), parent(parent)
+    ShareDialogUploader( tr("Garmin Connect"), context, ride, parent)
 {
     garminUploadId = ride->ride()->getTag("Garmin Connect uploadId", "");
     garminActivityId = ride->ride()->getTag("Garmin Connect activityId", "");
 }
 
+bool
+GarminUploader::wasUploaded()
+{
+    return garminActivityId.length()>0;
+}
+
 void
 GarminUploader::upload()
 {
-    // allready shared ?
-    if(garminActivityId.length()>0)
-    {
-        overwrite = false;
-
-        dialog = new QDialog();
-        QVBoxLayout *layout = new QVBoxLayout;
-
-        QVBoxLayout *layoutLabel = new QVBoxLayout();
-        QLabel *label = new QLabel();
-        label->setText(tr("This Ride is marked as already on Garmin Connect. Are you sure you want to upload it?"));
-        layoutLabel->addWidget(label);
-
-        QPushButton *ok = new QPushButton(tr("OK"), dialog);
-        QPushButton *cancel = new QPushButton(tr("Cancel"), dialog);
-        QHBoxLayout *buttons = new QHBoxLayout();
-        buttons->addStretch();
-        buttons->addWidget(cancel);
-        buttons->addWidget(ok);
-
-        connect(ok, SIGNAL(clicked()), this, SLOT(okClicked()));
-        connect(cancel, SIGNAL(clicked()), this, SLOT(closeClicked()));
-
-        layout->addLayout(layoutLabel);
-        layout->addLayout(buttons);
-
-        dialog->setLayout(layout);
-
-        if (!dialog->exec()) return;
-    }
-
     requestFlowExecutionKey();
     //requestUploadGarmin();
 
@@ -1295,7 +1312,7 @@ GarminUploader::requestUploadGarmin()
 {
     qDebug() << "requestUploadGarmin()";
 
-    parent->progressLabel->setText(tr("Upload ride to Garmin Connect..."));
+    parent->progressLabel->setText(tr("Upload to Garmin Connect..."));
     parent->progressBar->setValue(parent->progressBar->value()+10/parent->shareSiteCount);
 
     QEventLoop eventLoop;
@@ -1338,7 +1355,7 @@ GarminUploader::requestUploadGarmin()
     networkMgr.post(request, multiPart);
 
     parent->progressBar->setValue(parent->progressBar->value()+30/parent->shareSiteCount);
-    parent->progressLabel->setText(tr("Upload ride... Sending to Garmin Connect"));
+    parent->progressLabel->setText(tr("Upload... Sending to Garmin Connect"));
 
     eventLoop.exec();
 }
@@ -1380,18 +1397,4 @@ GarminUploader::requestUploadGarminFinished(QNetworkReply *reply)
     }
 }
 
-void
-GarminUploader::okClicked()
-{
-    dialog->accept();
-    return;
-}
-
-void
-GarminUploader::closeClicked()
-{
-    dialog->reject();
-    return;
-}
-
 #endif
