@@ -25,6 +25,7 @@
 #include "Units.h"
 #include "DeviceTypes.h"
 #include "DeviceConfiguration.h"
+#include "RideImportWizard.h"
 #include <QApplication>
 #include <QtGui>
 #include <QRegExp>
@@ -56,7 +57,7 @@
 #include <QStyleFactory>
 #endif
 
-#include <math.h> // isnan and isinf
+#include <cmath> // isnan and isinf
 #include "TrainDB.h"
 #include "Library.h"
 
@@ -354,7 +355,7 @@ intensity->hide(); //XXX!!! temporary
                             this, SLOT(mediaTreeWidgetSelectionChanged()));
     connect(context, SIGNAL(selectMedia(QString)), this, SLOT(selectVideo(QString)));
 #endif
-    connect(context, SIGNAL(configChanged()), this, SLOT(configChanged()));
+    connect(context, SIGNAL(configChanged(qint32)), this, SLOT(configChanged(qint32)));
     connect(context, SIGNAL(selectWorkout(QString)), this, SLOT(selectWorkout(QString)));
     connect(trainDB, SIGNAL(dataChanged()), this, SLOT(refresh()));
 
@@ -388,15 +389,17 @@ intensity->hide(); //XXX!!! temporary
     hrcount = 0;
     spdcount = 0;
     lodcount = 0;
+    wbalr = wbal = 0;
     load_msecs = total_msecs = lap_msecs = 0;
     displayWorkoutDistance = displayDistance = displayPower = displayHeartRate =
     displaySpeed = displayCadence = slope = load = 0;
+    displayLRBalance = displayLTE = displayRTE = displayLPS = displayRPS = 0;
 
     connect(gui_timer, SIGNAL(timeout()), this, SLOT(guiUpdate()));
     connect(disk_timer, SIGNAL(timeout()), this, SLOT(diskUpdate()));
     connect(load_timer, SIGNAL(timeout()), this, SLOT(loadUpdate()));
 
-    configChanged(); // will reset the workout tree
+    configChanged(CONFIG_APPEARANCE | CONFIG_DEVICES | CONFIG_ZONES); // will reset the workout tree
     setLabels();
 
 #ifndef Q_OS_MAC
@@ -499,7 +502,7 @@ TrainSidebar::mediaPopup()
 }
 
 void
-TrainSidebar::configChanged()
+TrainSidebar::configChanged(qint32)
 {
     setProperty("color", GColor(CTRAINPLOTBACKGROUND));
 #if !defined GC_VIDEO_NONE
@@ -558,11 +561,16 @@ TrainSidebar::configChanged()
     // And select default workout to Ergo
     QModelIndex firstWorkout = sortModel->index(0, 0, QModelIndex());
     workoutTree->setCurrentIndex(firstWorkout);
+
     // Athlete
     FTP=285; // default to 285 if zones are not set
-    int range = context->athlete->zones()->whichRange(QDate::currentDate());
-    if (range != -1) FTP = context->athlete->zones()->getCP(range);
+    WPRIME = 20000;
 
+    int range = context->athlete->zones()->whichRange(QDate::currentDate());
+    if (range != -1) {
+        FTP = context->athlete->zones()->getCP(range);
+        WPRIME = context->athlete->zones()->getWprime(range);
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -867,6 +875,8 @@ void TrainSidebar::Start()       // when start button is pressed
         session_elapsed_msec = 0;
         lap_time.start();
         lap_elapsed_msec = 0;
+        wbalr = 0;
+        wbal = WPRIME;
         calibrating = false;
 
         if (status & RT_WORKOUT) {
@@ -883,7 +893,11 @@ void TrainSidebar::Start()       // when start button is pressed
             // setup file
             QString filename = now.toString(QString("yyyy_MM_dd_hh_mm_ss")) + QString(".csv");
 
-            QString fulltarget = context->athlete->home.absolutePath() + "/" + filename;
+            if (!context->athlete->home->records().exists())
+                context->athlete->home->createAllSubdirs();
+
+            QString fulltarget = context->athlete->home->records().canonicalPath() + "/" + filename;
+
             if (recordFile) delete recordFile;
             recordFile = new QFile(fulltarget);
             if (!recordFile->open(QFile::WriteOnly | QFile::Truncate)) {
@@ -893,7 +907,8 @@ void TrainSidebar::Start()       // when start button is pressed
                 // CSV File header
 
                 QTextStream recordFileStream(recordFile);
-                recordFileStream << "Minutes,Torq (N-m),Km/h,Watts,Km,Cadence,Hrate,ID,Altitude (m)\n";
+                recordFileStream << "secs, cad, hr, km, kph, nm, watts, alt, lon, lat, headwind, slope, temp, interval, lrbalance, lte, rte, lps, rps, smo2, thb, o2hb, hhb\n";
+
                 disk_timer->start(SAMPLERATE);  // start screen
             }
         }
@@ -985,7 +1000,12 @@ void TrainSidebar::Stop(int deviceStatus)        // when stop button is pressed
             // add to the view - using basename ONLY
             QString name;
             name = recordFile->fileName();
-            context->athlete->addRide(QFileInfo(name).fileName(), true);
+
+            QList<QString> list;
+            list.append(name);
+
+            RideImportWizard *dialog = new RideImportWizard (list, context);
+            dialog->process(); // do it!
         }
     }
 
@@ -1015,6 +1035,8 @@ void TrainSidebar::Stop(int deviceStatus)        // when stop button is pressed
     spdcount = 0;
     lodcount = 0;
     displayWorkoutLap = displayLap =0;
+    wbalr = 0;
+    wbal = WPRIME;
     session_elapsed_msec = 0;
     session_time.restart();
     lap_elapsed_msec = 0;
@@ -1034,6 +1056,11 @@ void TrainSidebar::updateData(RealtimeData &rtData)
     displayHeartRate = rtData.getHr();
     displaySpeed = rtData.getSpeed();
     load = rtData.getLoad();
+    displayLRBalance = rtData.getLRBalance();
+    displayLTE = rtData.getLTE();
+    displayRTE = rtData.getRTE();
+    displayLPS = rtData.getLPS();
+    displayRPS = rtData.getRPS();
     // Gradient not supported
     return;
 }
@@ -1090,6 +1117,10 @@ void TrainSidebar::guiUpdate()           // refreshes the telemetry
                     rtData.setSlope(local.getSlope()); // and get slope in case it was adjusted	
 					// to within defined limits					
 				}
+
+                if (Devices[dev].type == DEV_ANTLOCAL || Devices[dev].type == DEV_NULL) {
+                    rtData.setHb(local.getSmO2(), local.gettHb()); //only moxy data from ant and robot devices right now
+                }
 				
                 // what are we getting from this one?
                 if (dev == bpmTelemetry) rtData.setHr(local.getHr());
@@ -1101,6 +1132,11 @@ void TrainSidebar::guiUpdate()           // refreshes the telemetry
                 if (dev == wattsTelemetry) {
                     rtData.setWatts(local.getWatts());
                     rtData.setAltWatts(local.getAltWatts());
+                    rtData.setLRBalance(local.getLRBalance());
+                    rtData.setLTE(local.getLTE());
+                    rtData.setRTE(local.getRTE());
+                    rtData.setLPS(local.getLPS());
+                    rtData.setRPS(local.getRPS());
                 }
             }
 
@@ -1134,6 +1170,11 @@ void TrainSidebar::guiUpdate()           // refreshes the telemetry
             displaySpeed = rtData.getSpeed();
             load = rtData.getLoad();
             slope = rtData.getSlope();
+            displayLRBalance = rtData.getLRBalance();
+            displayLTE = rtData.getLTE();
+            displayRTE = rtData.getRTE();
+            displayLPS = rtData.getLPS();
+            displayRPS = rtData.getRPS();
 
             // virtual speed
             double crr = 0.004f; // typical for asphalt surfaces
@@ -1161,10 +1202,22 @@ void TrainSidebar::guiUpdate()           // refreshes the telemetry
                 (3.*pow(2,0.3333333333333333)*ad*cdA));
 
             // just in case...
-            if (isnan(vs) || isinf(vs)) vs = 0.00f;
-
+            if (std::isnan(vs) || std::isinf(vs)) vs = 0.00f;
             rtData.setVirtualSpeed(vs);
 
+            // W'bal on the fly
+            // using Dave Waterworth's reformulation
+            double TAU = appsettings->cvalue(context->athlete->cyclist, GC_WBALTAU, 300).toInt();
+
+            // any watts expended in last 200msec?
+            double JOULES = double(rtData.getWatts() - FTP) / 5.00f;
+            if (JOULES < 0) JOULES = 0;
+
+            // running total of replenishment
+            wbalr += JOULES * exp((total_msecs/1000.00f) / TAU);
+            wbal = WPRIME - (wbalr * exp((-total_msecs/1000.00f) / TAU));
+
+            rtData.setWbal(wbal);
 
             // go update the displays...
             context->notifyTelemetryUpdate(rtData); // signal everyone to update telemetry
@@ -1221,29 +1274,44 @@ void TrainSidebar::warnnoConfig()
 //----------------------------------------------------------------------
 void TrainSidebar::diskUpdate()
 {
-    double  Minutes;
+    int  secs;
 
-    long Torq = 0, Altitude = 0;
+    long torq = 0, altitude = 0;
     QTextStream recordFileStream(recordFile);
 
     if (calibrating) return;
 
-    // convert from milliseconds to minutes
+    // convert from milliseconds to secondes
     total_msecs = session_elapsed_msec + session_time.elapsed();
-    Minutes = total_msecs;
-    Minutes /= 1000.00;
-    Minutes *= (1.0/60);
+    secs = total_msecs;
+    secs /= 1000.0;
 
-    // PowerAgent Format "Minutes,Torq (N-m),Km/h,Watts,Km,Cadence,Hrate,ID,Altitude (m)"
-    recordFileStream    << Minutes
-                        << "," << Torq
-                        << "," << displaySpeed
-                        << "," << displayPower
-                        << "," << displayDistance
+    // GoldenCheetah CVS Format "secs, cad, hr, km, kph, nm, watts, alt, lon, lat, headwind, slope, temp, interval, lrbalance, lte, rte, lps, rps, smo2, thb, o2hb, hhb\n";
+
+    recordFileStream    << secs
                         << "," << displayCadence
                         << "," << displayHeartRate
+                        << "," << displayDistance
+                        << "," << displaySpeed
+                        << "," << torq
+                        << "," << displayPower
+                        << "," << altitude
+                        << "," // lon
+                        << "," // lat
+                        << "," // headwind
+                        << "," // slope
+                        << "," // temp
                         << "," << (displayLap + displayWorkoutLap)
-                        << "," << Altitude
+                        << "," << displayLRBalance
+                        << "," << displayLTE
+                        << "," << displayRTE
+                        << "," << displayLPS
+                        << "," << displayRPS
+                        << "," // smo2
+                        << "," // thb
+                        << "," // o2hb
+                        << "," // hhb\n
+
                         << "," << "\n";
 }
 
@@ -1698,7 +1766,7 @@ TrainSidebar::deleteDevice()
     all.writeConfig(list);
 
     // tell everyone
-    context->notifyConfigChanged();
+    context->notifyConfigChanged(CONFIG_DEVICES);
 }
 
 // we have been told to select this video (usually because
