@@ -36,7 +36,7 @@
 //    into the future -- but crucially, no need to bother if power above CP is 0.
 //    This typically reduces the cpu cycles by a factor of 4
 //
-// 2. Because the decay is calculated forward at time u we can do these in paralle;
+// 2. Because the decay is calculated forward at time u we can do these in parallel;
 //    i.e. run multiple threads for t=0 through t=time/nthreads. This reduced the
 //    elapsed time by a factor of about 2/3rds on a dual core processor.
 //
@@ -49,6 +49,7 @@
 
 
 #include "WPrime.h"
+#include "RideItem.h"
 #include "Units.h" // for MILES_PER_KM
 #include "Settings.h" // for GC_WBALFORM
 
@@ -60,6 +61,25 @@ const double E = 2.71828183;
 const int WprimeMatchSmoothing = 25; // 25 sec smoothing looking for matches
 const int WprimeMatchMinJoules = 100; 
 
+// used by summarise
+static struct WPRIMEZONES {
+
+    int lo, hi;
+    QString name;
+    const char *desc;
+
+} wbal_zones[] = {
+
+    { 0, 25, "W1", QT_TRANSLATE_NOOP("wbalzone", "Recovered") },
+    { 25, 50, "W2", QT_TRANSLATE_NOOP("wbalzone", "Moderate Fatigue") },
+    { 50, 75, "W3", QT_TRANSLATE_NOOP("wbalzone", "Heavy Fatigue") },
+    { 75, 100, "W4", QT_TRANSLATE_NOOP("wbalzone", "Severe Fatigue") }
+
+};
+
+QString WPrime::zoneName(int i) { return wbal_zones[i].name; }
+QString WPrime::zoneDesc(int i) { return qApp->translate("wbalzone", wbal_zones[i].desc); }
+
 WPrime::WPrime()
 {
     // XXX will need to reset metrics when they are added
@@ -110,62 +130,47 @@ WPrime::setRide(RideFile *input)
     last=0;
     double offset = 0; // always start from zero seconds (e.g. intervals start at and offset in ride)
     bool first = true;
-    if (input->recIntSecs() >= 1) {
-        RideFilePoint *lp=NULL;
-        foreach(RideFilePoint *p, input->dataPoints()) {
 
-            // yuck! nasty data
-            if (p->secs > (25*60*60)) return;
+    RideFilePoint *lp=NULL;
+    foreach(RideFilePoint *p, input->dataPoints()) {
 
-            if (first) {
-                offset = p->secs;
-                first = false;
-            }
+        // yuck! nasty data
+        if (p->secs > (25*60*60)) return;
 
-            // fill gaps in recording with zeroes
-            if (lp)
-                for(int t=lp->secs+input->recIntSecs();
-                    (t + input->recIntSecs()) < p->secs;
-                    t += input->recIntSecs()) {
-                    points << QPointF(t-offset, 0);
-                    pointsd << QPointF(t-offset, p->km * convert); // not zero !!!! this is a map from secs -> km not a series
-                }
-
-            // lets not go backwards -- or two sampls at the same time
-            if ((lp && p->secs > lp->secs) || !lp) {
-                points << QPointF(p->secs - offset, p->watts);
-                pointsd << QPointF(p->secs - offset, p->km * convert);
-            }
-
-            // update state
-            last = p->secs - offset;
-            lp = p;
+        if (first) {
+            offset = p->secs;
+            first = false;
         }
-    } else {
 
-        foreach(RideFilePoint *p, input->dataPoints()) {
-
-            // yuck! nasty data
-            if (p->secs > (25*60*60)) return;
-
-            if (first) {
-                offset = p->secs;
-                first = false;
+        // fill gaps in recording with zeroes
+        if (lp)
+            for(double t=lp->secs+input->recIntSecs();
+                (t + input->recIntSecs()) < p->secs;
+                t += input->recIntSecs()) {
+                points << QPointF(t-offset, 0);
+                pointsd << QPointF(t-offset, p->km * convert); // not zero !!!! this is a map from secs -> km not a series
             }
+
+        // lets not go backwards -- or two samples at the same time
+        if ((lp && p->secs > lp->secs) || !lp) {
             points << QPointF(p->secs - offset, p->watts);
             pointsd << QPointF(p->secs - offset, p->km * convert);
-            last = p->secs - offset;
         }
+
+        // update state
+        last = p->secs - offset;
+        lp = p;
     }
 
     // Create a spline
-    distance.setSplineType(QwtSpline::Natural);
+    distance.setSplineType(QwtSpline::Periodic);
     distance.setPoints(QPolygonF(pointsd));
-    smoothed.setSplineType(QwtSpline::Natural);
+    smoothed.setSplineType(QwtSpline::Periodic);
     smoothed.setPoints(QPolygonF(points));
 
     // Get CP
     CP = 250; // default
+    WPRIME = 20000;
     if (input->context->athlete->zones()) {
         int zoneRange = input->context->athlete->zones()->whichRange(input->startTime().date());
         CP = zoneRange >= 0 ? input->context->athlete->zones()->getCP(zoneRange) : 0;
@@ -174,6 +179,8 @@ WPrime::setRide(RideFile *input)
         // did we override CP in metadata / metrics ?
         int oCP = input->getTag("CP","0").toInt();
         if (oCP) CP=oCP;
+        int oW = input->getTag("W'","0").toInt();
+        if (oW) WPRIME=oW;
     }
     minY = maxY = WPRIME;
 
@@ -197,9 +204,9 @@ WPrime::setRide(RideFile *input)
     }
 
     if (countBelowCP > 0)
-        TAU = 546.00f * pow(E,-0.01*(CP - (totalBelowCP/countBelowCP))) + 316.00f;
+        TAU = 546.00f * exp(-0.01*(CP - (totalBelowCP/countBelowCP))) + 316.00f;
     else
-        TAU = 546.00f * pow(E,-0.01*(CP)) + 316.00f;
+        TAU = 546.00f * exp(-0.01*(CP)) + 316.00f;
 
     TAU = int(TAU); // round it down
 
@@ -217,21 +224,15 @@ WPrime::setRide(RideFile *input)
 
         QVector<double> myvalues(last+1);
 
-        int stop = last / 2;
-
-        WPrimeIntegrator a(inputArray, 0, stop, TAU);
-        WPrimeIntegrator b(inputArray, stop+1, last, TAU);
+        WPrimeIntegrator a(inputArray, 0, last, TAU);
 
         a.start();
-        b.start();
-
         a.wait();
-        b.wait();
 
         // sum values
         for (int t=0; t<=last; t++) {
-            values[t] = a.output[t] + b.output[t];
-            xvalues[t] = double(t) / 60.00f;
+            values[t] = a.output[t];
+            xvalues[t] = t / 60.00f;
         }
 
         // now subtract WPRIME and work out minimum etc
@@ -274,7 +275,7 @@ WPrime::setRide(RideFile *input)
     }
 
     if (minY < -30000) minY = 0; // the data is definitely out of bounds!
-                                 // so lets not excacerbate the problem - truncate
+                                 // so lets not exacerbate the problem - truncate
 
     //qDebug()<<"compute W'bal curve took"<<time.elapsed();
 
@@ -335,7 +336,7 @@ WPrime::setRide(RideFile *input)
                 match.stop = end;
                 match.secs = (match.stop-match.start) +1; // don't fencepost!
                 match.cost = values[match.start] - values[match.stop];
-
+                match.exhaust = values[match.stop] <= 500 ? true : false; // its to exhaustion!
                 if (match.cost >= WprimeMatchMinJoules) {
                     matches << match;
                 }
@@ -392,6 +393,8 @@ WPrime::setErg(ErgFile *input)
     if (integral) {
 
         last = input->Duration / 1000; 
+        values.resize(last);
+        xvalues.resize(last);
 
         // input array contains the actual W' expenditure
         // and will also contain non-zero values
@@ -413,12 +416,7 @@ WPrime::setErg(ErgFile *input)
             } else EXP += value; // total expenditure above CP
         }
 
-        if (countBelowCP > 0)
-            TAU = 546.00f * pow(E,-0.01*(CP - (totalBelowCP/countBelowCP))) + 316.00f;
-        else
-            TAU = 546.00f * pow(E,-0.01*(CP)) + 316.00f;
-
-        TAU = int(TAU); // round it down
+        TAU = appsettings->cvalue(input->context->athlete->cyclist, GC_WBALTAU, 300).toInt();
 
         // lets run forward from 0s to end of ride
         values.resize(last+1);
@@ -426,21 +424,15 @@ WPrime::setErg(ErgFile *input)
 
         QVector<double> myvalues(last+1);
 
-        int stop = last / 2;
-
-        WPrimeIntegrator a(inputArray, 0, stop, TAU);
-        WPrimeIntegrator b(inputArray, stop+1, last, TAU);
+        WPrimeIntegrator a(inputArray, 0, last, TAU);
 
         a.start();
-        b.start();
-
         a.wait();
-        b.wait();
 
         // sum values
         for (int t=0; t<=last; t++) {
-            values[t] = a.output[t] + b.output[t];
-            xvalues[t] = t * 1000;
+            values[t] = a.output[t];
+            xvalues[t] = t * 1000.00f;
         }
 
         // now subtract WPRIME and work out minimum etc
@@ -483,7 +475,7 @@ WPrime::setErg(ErgFile *input)
     }
 
     if (minY < -30000) minY = 0; // the data is definitely out of bounds!
-                                 // so lets not excacerbate the problem - truncate
+                                 // so lets not exacerbate the problem - truncate
 }
 
 double
@@ -549,25 +541,73 @@ void
 WPrimeIntegrator::run()
 {
     // run from start to stop adding decay to end
-    for (int t=begin; t<end; t++) {
+    double I = 0.00f;
+    for (int t=0; t<=end; t++) {
 
-        if (source[t] <= 0) continue;
-
-        // start at 1, since the actual value shouldn't be adjusted with itself !
-        for (int i=1; i < (TAU*3) /*WPrimeDecayPeriod*/ && t+i < source.size(); i++) {
-
-            double value = source[t] * pow(E, -(double(i)/TAU));
-
-            // diminishing returns - we're dealing in kJ, so 10J is nothing !
-            // this saves about 20% in calculation time typically
-            if (value < 10.00f) break;
- 
-            // integrate
-            output[t+i] += value;
-        }
+        I += exp(((double)(t) / TAU)) * source[t];
+        output[t] = exp(-((double)(t) / TAU)) * I;
     }
 }
 
+//
+// HTML zone summary
+//
+QString
+WPrime::summarize(int WPRIME, QVector<double> wtiz, QColor color)
+{
+    // if wtiz is not 4 big return empty
+    if (wtiz.count() != 4) return "";
+
+    // otherwise lets go
+    QString summary;
+
+    // W' used for summary
+    summary += "<table align=\"center\" width=\"70%\" border=\"0\">";
+    summary += "<tr><td align=\"center\">";
+    summary += QString(tr("W' (Joules): %1")).arg(WPRIME);
+    summary += "</td></tr></table>";
+
+    // Heading
+    summary += "<table align=\"center\" width=\"70%\" ";
+    summary += "border=\"0\">";
+    summary += "<tr>";
+    summary += tr("<td align=\"center\">Zone</td>");
+    summary += tr("<td align=\"center\">Description</td>");
+    summary += tr("<td align=\"center\">High (J)</td>");
+    summary += tr("<td align=\"center\">Low (J)</td>");
+    summary += tr("<td align=\"center\">Time</td>");
+    summary += tr("<td align=\"center\">%</td>");
+    summary += "</tr>";
+
+    // calc totals to use for percentages
+    double duration = 0;
+    foreach(double v, wtiz) {
+        duration += v;
+    }
+
+    for (int zone = 0; zone < 4; zone++) {
+
+        // alternating rows
+        if (zone % 2 == 0) summary += "<tr bgcolor='" + color.name() + "'>";
+        else summary += "<tr>"; 
+
+        // basics
+        summary += QString("<td align=\"center\">%1</td>").arg(wbal_zones[zone].name);
+        summary += QString("<td align=\"center\">%1</td>").arg(qApp->translate("wbalzone", wbal_zones[zone].desc));
+        summary += QString("<td align=\"center\">%1</td>").arg(WPRIME - (WPRIME / 100.0f * wbal_zones[zone].lo), 0, 'f', 0);
+
+        if (zone == 3)
+            summary += "<td align=\"center\">MIN</td>";
+        else
+            summary += QString("<td align=\"center\">%1</td>").arg(WPRIME - (WPRIME / 100.0f * wbal_zones[zone].hi), 0, 'f', 0); 
+        summary += QString("<td align=\"center\">%1</td>").arg(time_to_string((unsigned) round(wtiz[zone])));
+        summary += QString("<td align=\"center\">%1</td>").arg((double)wtiz[zone]/duration * 100, 0, 'f', 0);
+        summary += "</tr>";
+    }
+    summary += "</table>";
+    return summary;
+}
+
 //
 // Associated Metrics
 //
@@ -598,6 +638,7 @@ class MinWPrime : public RideMetric {
     }
 
     bool canAggregate() { return false; }
+    bool isRelevantForRide(const RideItem *ride) const { return !ride->isSwim && !ride->isRun; }
     RideMetric *clone() const { return new MinWPrime(*this); }
 };
 
@@ -627,6 +668,7 @@ class MaxWPrime : public RideMetric {
     }
 
     bool canAggregate() { return false; }
+    bool isRelevantForRide(const RideItem *ride) const { return !ride->isSwim && !ride->isRun; }
     RideMetric *clone() const { return new MaxWPrime(*this); }
 };
 
@@ -656,9 +698,42 @@ class MaxMatch : public RideMetric {
     }
 
     bool canAggregate() { return false; }
+    bool isRelevantForRide(const RideItem *ride) const { return !ride->isSwim && !ride->isRun; }
     RideMetric *clone() const { return new MaxMatch(*this); }
 };
 
+class Matches : public RideMetric {
+    Q_DECLARE_TR_FUNCTIONS(Matches);
+
+    public:
+
+    Matches()
+    {
+        setSymbol("skiba_wprime_matches");
+        setInternalName("W'bal Matches > 2KJ");
+    }
+    void initialize() {
+        setName(tr("W'bal Matches"));
+        setType(RideMetric::Total);
+        setPrecision(0);
+    }
+    void compute(const RideFile *r, const Zones *, int,
+                 const HrZones *, int,
+                 const QHash<QString,RideMetric*> &,
+                 const Context *) {
+
+        int matches=0;
+        foreach(Match m, const_cast<RideFile*>(r)->wprimeData()->matches) {
+            if (m.cost > 2000) matches++; // 2kj is minimum size
+        }
+        setValue(matches);
+    }
+
+    bool canAggregate() { return false; }
+    bool isRelevantForRide(const RideItem *ride) const { return !ride->isSwim && !ride->isRun; }
+    RideMetric *clone() const { return new Matches(*this); }
+};
+
 class WPrimeTau : public RideMetric {
     Q_DECLARE_TR_FUNCTIONS(WPrimeTau);
 
@@ -685,6 +760,7 @@ class WPrimeTau : public RideMetric {
     }
 
     bool canAggregate() { return false; }
+    bool isRelevantForRide(const RideItem *ride) const { return !ride->isSwim && !ride->isRun; }
     RideMetric *clone() const { return new WPrimeTau(*this); }
 };
 
@@ -726,9 +802,52 @@ class WPrimeExp : public RideMetric {
     }
 
     bool canAggregate() { return false; }
+    bool isRelevantForRide(const RideItem *ride) const { return !ride->isSwim && !ride->isRun; }
     RideMetric *clone() const { return new WPrimeExp(*this); }
 };
 
+class WPrimeWatts : public RideMetric {
+    Q_DECLARE_TR_FUNCTIONS(WPrimeWatts);
+
+    public:
+
+    WPrimeWatts()
+    {
+        setSymbol("skiba_wprime_watts");
+        setInternalName("W' Watts");
+    }
+    void initialize() {
+        setName(tr("W' Power"));
+        setType(RideMetric::Total);
+        setMetricUnits(tr("watts"));
+        setImperialUnits(tr("watts"));
+        setPrecision(0);
+    }
+    void compute(const RideFile *r, const Zones *zones, int zonerange,
+                 const HrZones *, int,
+                 const QHash<QString,RideMetric*> &,
+                 const Context *) {
+
+        int cp = r->getTag("CP","0").toInt();
+        if (!cp && zones && zonerange >=0) cp = zones->getCP(zonerange);
+
+        double total = 0;
+        double secs = 0;
+        foreach(const RideFilePoint *point, r->dataPoints()) {
+            if (cp && point->watts > cp)  {
+                total += r->recIntSecs() * (point->watts - cp);
+            }
+            secs += r->recIntSecs();
+        }
+        setValue(total/secs);
+        setCount(secs);
+    }
+
+    bool canAggregate() { return false; }
+    bool isRelevantForRide(const RideItem *ride) const { return !ride->isSwim && !ride->isRun; }
+    RideMetric *clone() const { return new WPrimeWatts(*this); }
+};
+
 class CPExp : public RideMetric {
     Q_DECLARE_TR_FUNCTIONS(CPExp);
 
@@ -770,16 +889,153 @@ class CPExp : public RideMetric {
     }
 
     bool canAggregate() { return false; }
+    bool isRelevantForRide(const RideItem *ride) const { return !ride->isSwim && !ride->isRun; }
     RideMetric *clone() const { return new CPExp(*this); }
 };
 
+// time in zone
+class WZoneTime : public RideMetric {
+    Q_DECLARE_TR_FUNCTIONS(WZoneTime)
+
+    int level;
+
+    public:
+
+    WZoneTime() : level(0)
+    {
+        setType(RideMetric::Total);
+        setMetricUnits(tr("seconds"));
+        setImperialUnits(tr("seconds"));
+        setPrecision(0);
+        setConversion(1.0);
+    }
+    bool isTime() const { return true; }
+    void setLevel(int level) { this->level=level-1; } // zones start from zero not 1
+    void compute(const RideFile *ride, const Zones *zones, int zoneRange,
+                 const HrZones *, int,
+                 const QHash<QString,RideMetric*> &,
+                 const Context *)
+    {
+
+        double WPRIME = zoneRange >= 0 ? zones->getWprime(zoneRange) : 20000;
+
+        // 4 zones
+        QVector<double> tiz(4);
+        tiz.fill(0.0f);
+
+        RideFile *cride = const_cast<RideFile*>(ride);
+        if (cride->wprimeData() && cride->wprimeData()->ydata().count()) {
+
+            foreach(int value, cride->wprimeData()->ydata()) {
+
+                // percent is PERCENT OF W' USED
+                double percent = 100.0f - ((double (value) / WPRIME) * 100.0f);
+                if (percent < 0.0f) percent = 0.0f;
+                if (percent > 100.0f) percent = 100.0f;
+
+                // and zones in 1s increments
+                if (percent <= 25.0f) tiz[0]++;
+                else if (percent <= 50.0f) tiz[1]++;
+                else if (percent <= 75.0f) tiz[2]++;
+                else tiz[3]++;
+            }
+
+        } 
+        setValue(tiz[level]);
+    }
+
+    bool canAggregate() { return false; }
+    void aggregateWith(const RideMetric &) {}
+    bool isRelevantForRide(const RideItem *ride) const { return !ride->isSwim && !ride->isRun; }
+    RideMetric *clone() const { return new WZoneTime(*this); }
+};
+
+class WZoneTime1 : public WZoneTime {
+    Q_DECLARE_TR_FUNCTIONS(WZoneTime1)
+
+    public:
+        WZoneTime1()
+        {
+            setLevel(1);
+            setSymbol("wtime_in_zone_L1");
+            setInternalName("W1 W'bal Low Fatigue");
+        }
+        void initialize ()
+        {
+            setName(tr("W1 W'bal Low Fatigue"));
+            setMetricUnits(tr("seconds"));
+            setImperialUnits(tr("seconds"));
+        }
+        RideMetric *clone() const { return new WZoneTime1(*this); }
+};
+class WZoneTime2 : public WZoneTime {
+    Q_DECLARE_TR_FUNCTIONS(WZoneTime2)
+
+    public:
+        WZoneTime2()
+        {
+            setLevel(2);
+            setSymbol("wtime_in_zone_L2");
+            setInternalName("W2 W'bal Moderate Fatigue");
+        }
+        void initialize ()
+        {
+            setName(tr("W2 W'bal Moderate Fatigue"));
+            setMetricUnits(tr("seconds"));
+            setImperialUnits(tr("seconds"));
+        }
+        RideMetric *clone() const { return new WZoneTime2(*this); }
+};
+class WZoneTime3 : public WZoneTime {
+    Q_DECLARE_TR_FUNCTIONS(WZoneTime3)
+
+    public:
+        WZoneTime3()
+        {
+            setLevel(3);
+            setSymbol("wtime_in_zone_L3");
+            setInternalName("W3 W'bal Heavy Fatigue");
+        }
+        void initialize ()
+        {
+            setName(tr("W3 W'bal Heavy Fatigue"));
+            setMetricUnits(tr("seconds"));
+            setImperialUnits(tr("seconds"));
+        }
+        RideMetric *clone() const { return new WZoneTime3(*this); }
+};
+class WZoneTime4 : public WZoneTime {
+    Q_DECLARE_TR_FUNCTIONS(WZoneTime4)
+
+    public:
+        WZoneTime4()
+        {
+            setLevel(4);
+            setSymbol("wtime_in_zone_L4");
+            setInternalName("W4 W'bal Severe Fatigue");
+        }
+        void initialize ()
+        {
+            setName(tr("W4 W'bal Severe Fatigue"));
+            setMetricUnits(tr("seconds"));
+            setImperialUnits(tr("seconds"));
+        }
+        RideMetric *clone() const { return new WZoneTime4(*this); }
+};
+
 // add to catalogue
 static bool addMetrics() {
     RideMetricFactory::instance().addMetric(MinWPrime());
     RideMetricFactory::instance().addMetric(MaxWPrime()); // same thing expressed as a maximum
+    RideMetricFactory::instance().addMetric(Matches());
     RideMetricFactory::instance().addMetric(MaxMatch());
     RideMetricFactory::instance().addMetric(WPrimeTau());
     RideMetricFactory::instance().addMetric(WPrimeExp());
+    RideMetricFactory::instance().addMetric(WPrimeWatts());
+    RideMetricFactory::instance().addMetric(WZoneTime1());
+    RideMetricFactory::instance().addMetric(WZoneTime2());
+    RideMetricFactory::instance().addMetric(WZoneTime3());
+    RideMetricFactory::instance().addMetric(WZoneTime4());
     RideMetricFactory::instance().addMetric(CPExp());
     return true;
 }
